@@ -125,7 +125,7 @@ int main(
   char extend[MAXPATHLEN];
   char *extend_ptr = NULL; /* only give value if extend has value */
 
-#define GETOPT_ARGS "a:A:c:e:h:j:k:l:m:M:N:o:p:qr:S:t:u:v:W:x:"
+#define GETOPT_ARGS "a:A:c:e:E:h:j:k:l:m:M:N:o:p:qr:S:t:u:v:W:x:"
 
   while ((c = getopt(argc, argv, GETOPT_ARGS)) != EOF)
     {
@@ -229,6 +229,30 @@ int main(
 
         break;
 
+      case 'E':
+
+        if (strlen(optarg) != 1)
+          {
+          fprintf(stderr, "qalter: illegal -E value\n");
+
+          errflg++;
+
+          break;
+          }
+
+        if ((*optarg != 'y') && (*optarg != 'n'))
+          {
+          fprintf(stderr, "qalter: illegal -r value\n");
+
+          errflg++;
+
+          break;
+          }
+
+        set_attr(&attrib, ATTR_node_exclusive, optarg);
+
+        break;
+
       case 'h':
 
         /* hold */
@@ -672,7 +696,7 @@ int main(
                           [-h hold_list] [-j y|n] [-k keep] [-l resource_list] [-m mail_options] \n\
                           [-M user_list] [-N jobname] [-o path] [-p priority] [-q] [-r y|n] [-S path] \n\
                           [t [array range]%slot limit] [-u user_list] [-v variable_list] \n\
-                          [-W additional_attributes] [-x exec_host] job_identifier...\n";
+                          [-W dependency_list] [-x exec_host] job_identifier...\n";
 
     fprintf(stderr, "%s", usage);
 
