@@ -1185,6 +1185,7 @@ int a_opt = FALSE;
 int b_opt = FALSE;
 int c_opt = FALSE;
 int e_opt = FALSE;
+int E_opt = FALSE;
 int f_opt = FALSE;
 int h_opt = FALSE;
 int j_opt = FALSE;
@@ -2319,7 +2320,7 @@ void catchchild(
 
   /* reset terminal to cooked mode */
 
-  if(have_terminal)
+  if (have_terminal)
     tcsetattr(0, TCSANOW, &oldtio);
 
   exit(0);
@@ -2834,9 +2835,9 @@ int process_opts(
   char search_string[256];
 
 #if !defined(PBS_NO_POSIX_VIOLATION)
-#define GETOPT_ARGS "a:A:b:c:C:d:D:e:fhIj:k:l:m:M:N:o:p:P:q:r:S:t:T:u:v:Vw:W:Xxz-:"
+#define GETOPT_ARGS "a:A:b:c:C:d:D:e:EfhIj:k:l:m:M:N:o:p:P:q:r:S:t:T:u:v:Vw:W:Xxz-:"
 #else
-#define GETOPT_ARGS "a:A:c:C:e:hj:k:l:m:M:N:o:p:q:r:S:u:v:VW:z"
+#define GETOPT_ARGS "a:A:c:C:e:Ehj:k:l:m:M:N:o:p:q:r:S:u:v:VW:z"
 #endif /* PBS_NO_POSIX_VIOLATION */
 
   /* The following macro, together the value of passet (pass + 1) is used */
@@ -3178,6 +3179,17 @@ int process_opts(
           }
 
         break;
+
+      case 'E':
+
+        if_cmd_line(E_opt)
+          {
+          E_opt = passet;
+          
+          set_attr(&attrib, ATTR_node_exclusive, "TRUE");
+          }
+
+        break;
         
 #if !defined(PBS_NO_POSIX_VIOLATION)
       
@@ -3273,7 +3285,7 @@ int process_opts(
         l_opt = passet;
 
         /* a ,procs= in the node spec is illegal. Validate the node spec */
-        if(strstr(optarg, ",procs="))
+        if (strstr(optarg, ",procs="))
           {
           printf("illegal node spec: %s\n", optarg);
           return(-1);
@@ -3298,9 +3310,9 @@ int process_opts(
         else
           {
           /* Normal evaluation of batch job resources. */
-          if(attrib)
+          if (attrib)
             {
-            if(attrib->resource)
+            if (attrib->resource)
               {
               /* right here we are looking for a +procs=x in the node spec */
               char *proc_ptr = NULL;
@@ -3312,14 +3324,14 @@ int process_opts(
               proc_val[0] = 0; /* Initialize */
 
               proc_ptr = strstr(attrib->value, "procs=");
-              if(proc_ptr)
+              if (proc_ptr)
                 {
                 /* we have the procs keyword in the node_spec. We need to take it out and
                    install it to the Resource_List */
                 patch_ptr = proc_ptr;
                 /* Get the procs value */
                 patch_ptr = strchr(patch_ptr, '=');
-                if(patch_ptr == NULL)
+                if (patch_ptr == NULL)
                   {
                   return(-1);
                   }
@@ -3354,7 +3366,7 @@ int process_opts(
                    the procs=x from the spec and splice the rest
                    of the specification back together
                  */
-                if(*patch_ptr == '+')
+                if (*patch_ptr == '+')
                   {
                   
                   patch_ptr++;
@@ -3367,15 +3379,15 @@ int process_opts(
                      If we are here procs=x is at the end
                      of the node specification
                    */
-                  if(*(proc_ptr - 1)  == '+')
+                  if (*(proc_ptr - 1)  == '+')
                     {
                     proc_ptr--;
                     }
                   *proc_ptr = 0;
                   }
-                } /* END if(proc_ptr)*/
-              } /* END if(attrib->resource ) */
-            } /* END if(attrib) */
+                } /* END if (proc_ptr)*/
+              } /* END if (attrib->resource ) */
+            } /* END if (attrib) */
 
           if (set_resources(&attrib, optarg, (pass == 0)) != 0)
             {                                                                  
@@ -4887,18 +4899,18 @@ int main(
 
   if (Interact_opt && ((isatty(0) == 0) || (isatty(1) == 0)))
     {
-      if (have_intr_cmd)
+    if (have_intr_cmd)
       {
-        have_terminal = FALSE;
+      have_terminal = FALSE;
       }
-      else
+    else
       {
       fprintf(stderr, "qsub:\tstandard input and output must be a terminal for \n\tinteractive job submission\n");
-
+      
       unlink(script_tmp);
-
+      
       close(inter_sock);
-
+      
       exit(1);
       }
     }
@@ -4910,7 +4922,7 @@ int main(
   if (parse_destination_id(destination, &q_n_out, &s_n_out))
     {
     fprintf(stderr, "qsub: illegally formed destination: %s\n",
-            destination);
+      destination);
 
     unlink(script_tmp);
 
@@ -4932,14 +4944,14 @@ int main(
   if (connect <= 0)
     {
     fprintf(stderr, "qsub: cannot connect to server %s (errno=%d) %s\n",
-            pbs_server,
-            pbs_errno,
-            pbs_strerror(pbs_errno));
+      pbs_server,
+      pbs_errno,
+      pbs_strerror(pbs_errno));
 
     if (getenv("PBSDEBUG") != NULL)
       {
       fprintf(stderr, "qsub: pbs_server daemon may not be running on host %s or hostname in file '$TORQUEHOME/server_name' may be incorrect)\n",
-              pbs_server);
+        pbs_server);
       }
 
     unlink(script_tmp);
