@@ -105,7 +105,6 @@
 #define ATTRIBUTE_H 1
 
 /* define the size of fields in the structures */
-#include "list_link.h"
 
 #define ATRFLAG 16
 #define ATRTYPE  6  /* sync w/ATR_TYPE_*  (see #defines below) */
@@ -429,7 +428,6 @@ extern char *parse_comma_string(char *);
 #define NULL_FUNC (int (*)())0
 
 /* other associated funtions */
-struct dynamic_string;
 
 extern int   acl_check(attribute *, char *canidate, int type);
 extern char *arst_string(char *str, attribute *pattr);
@@ -437,7 +435,6 @@ extern void  attrl_fixlink(tlist_head *svrattrl);
 extern void  recov_acl(attribute *, attribute_def *, char *, char *);
 extern int   save_acl(attribute *, attribute_def *,  char *, char *);
 extern int   save_attr(attribute_def *, attribute *, int);
-extern int   attr_to_str(struct dynamic_string *ds, attribute_def *, struct attribute, int);
 
 extern int      encode_state(attribute *, tlist_head *, char *, char *, int);
 extern int      encode_props(attribute*, tlist_head*, char*, char*, int);
@@ -455,13 +452,18 @@ extern int      set_node_props(attribute*, attribute*, enum batch_op);
 extern int      set_null(attribute*, attribute*, enum batch_op);
 extern int      node_state(attribute*, void*, int);
 extern int      node_np_action(attribute*, void*, int);
-extern int      node_gpus_action(attribute *,void *,int);
+extern int      node_mom_port_action(attribute*, void*, int);
+extern int      node_mom_rm_port_action(attribute*, void*, int);
+extern int      node_numa_action(attribute*, void*, int);
+extern int      node_gpus_action(attribute*, void*, int);
+extern int      numa_str_action(attribute *, void *,int);
 extern int      node_ntype(attribute*, void*, int);
 extern int      node_prop_list(attribute*, void*, int);
 extern int      node_status_list(attribute*, void*, int);
-extern int      node_gpustatus_list(attribute*, void*, int);
 extern int      node_note(attribute*, void*, int);
+extern int      node_alt_name(attribute*, void*, int);
 extern int      set_note_str(attribute *attr, attribute *new, enum batch_op);
+extern int      set_alt_name_str(attribute *attr, attribute *new, enum batch_op);
 extern void     replace_attr_string(attribute*, char*);
 
 /* Token manipulation functions */
@@ -469,10 +471,9 @@ extern void     replace_attr_string(attribute*, char*);
 extern int  decode_tokens(attribute *, char *, char *, char *);
 
 /* "type" to pass to acl_check() */
-#define ACL_Host      1
-#define ACL_User      2
-#define ACL_Group 	  3
-#define ACL_Gid   	  4
-#define ACL_User_Host 5
+#define ACL_Host  1
+#define ACL_User  2
+#define ACL_Group 3
+#define ACL_Gid   4
 
 #endif  /* ATTRIBUTE_H */
