@@ -99,6 +99,7 @@
 #include "libpbs.h"
 #include "attribute.h"
 #include "credential.h"
+#include "pbs_job.h"
 
 #ifndef BATCH_REQUEST_H
 #define BATCH_REQUEST_H 1
@@ -231,17 +232,6 @@ struct rq_track
   char  rq_state[2];
   };
 
-/* GPU Control */
-
-struct rq_gpuctrl
-  {
-  char  rq_momnode[PBS_MAXHOSTNAME + 1];
-  char  rq_gpuid[PBS_MAXGPUID + 1];
-  int   rq_gpumode;
-  int   rq_reset_perm;
-  int   rq_reset_vol;
-  };
-
 /* RegisterDependentJob */
 
 struct rq_register
@@ -377,8 +367,6 @@ struct batch_request
 
     struct rq_track  rq_track;
 
-    struct rq_gpuctrl  rq_gpuctrl;
-
     struct rq_cpyfile rq_cpyfile;
 
     struct rq_returnfiles rq_returnfiles;
@@ -404,7 +392,7 @@ extern void  req_stat_job (struct batch_request *);
 
 #ifndef PBS_MOM
 extern void  req_authenuser (struct batch_request *req);
-extern int   req_altauthenuser (struct batch_request *req);
+extern void  req_altauthenuser (struct batch_request *req);
 extern void  req_connect (struct batch_request *req);
 extern void  req_locatejob (struct batch_request *req);
 extern void  req_manager (struct batch_request *req);
@@ -421,7 +409,6 @@ extern void  req_stat_que (struct batch_request *req);
 extern void  req_stat_svr (struct batch_request *req);
 /* DIAGTODO: declr req_stat_diag() */
 extern void  req_trackjob (struct batch_request *req);
-extern void  req_gpuctrl (struct batch_request *req);
 #else
 extern void  req_cpyfile (struct batch_request *req);
 extern void  req_delfile (struct batch_request *req);
@@ -429,7 +416,7 @@ extern void req_returnfiles (struct batch_request *req);
 #endif
 
 #ifdef SERVER_LIMITS_H
-extern int   relay_to_mom (pbs_net_t, struct batch_request *, void (*)());
+int relay_to_mom (job *, struct batch_request *, void (*func)());
 #endif  /* SERVER_LIMITS_H */
 
 /* PBS Batch Request Decode/Encode routines */
@@ -458,7 +445,6 @@ extern int decode_DIS_Status (int socket, struct batch_request *);
 extern int decode_DIS_TrackJob (int socket, struct batch_request *);
 extern int decode_DIS_replySvr (int socket, struct batch_reply *);
 extern int decode_DIS_svrattrl (int socket, tlist_head *);
-extern int decode_DIS_GpuCtrl (int socket, struct batch_request *);
 
 extern int encode_DIS_CopyFiles (int socket, struct batch_request *);
 extern int encode_DIS_JobObit (int socket, struct batch_request *);
