@@ -86,8 +86,6 @@
 #ifndef _PBS_IFL_DEF
 #define _PBS_IFL_DEF
 
-#include <sys/socket.h>
-
 /* Attribute Names used by user commands */
 
 #define ATTR_a "Execution_Time"
@@ -109,7 +107,6 @@
 #define ATTR_u "User_List"
 #define ATTR_v "Variable_List"
 #define ATTR_A "Account_Name"
-#define ATTR_args "job_arguments"
 #define ATTR_M "Mail_Users"
 #define ATTR_N "Job_Name"
 #define ATTR_S "Shell_Path_List"
@@ -118,13 +115,12 @@
 #define ATTR_stagein  "stagein"
 #define ATTR_stageout "stageout"
 #define ATTR_jobtype  "jobtype"
-#define ATTR_submit_host "submit_host"
-#define ATTR_init_work_dir "init_work_dir"
 
 /* additional job and general attribute names */
 
 #define ATTR_ctime      "ctime"
 #define ATTR_exechost   "exec_host"
+#define ATTR_execport	"exec_port"
 #define ATTR_mtime      "mtime"
 #define ATTR_qtime      "qtime"
 #define ATTR_session    "session_id"
@@ -161,23 +157,18 @@
 #define ATTR_checkpoint_name  "checkpoint_name"   /* user specified name of checkpoint file */
 #define ATTR_checkpoint_time  "checkpoint_time"   /* timestamp of start of last checkpoint */
 #define ATTR_checkpoint_restart_status  "checkpoint_restart_status"   /* checkpoint restart status */
-#define ATTR_restart_name  "restart_name"   /* saved name of checkpoint restart file */
-#define ATTR_comp_time   "comp_time"
-#define ATTR_reported    "reported"
+#define ATTR_restart_name	"restart_name"   /* saved name of checkpoint restart file */
+#define ATTR_comp_time   	"comp_time"
+#define ATTR_reported    	"reported"
 #define ATTR_intcmd      "inter_cmd"
 #define ATTR_P           "proxy_user"
+#define ATTR_node_exclusive "node_exclusive"
 #define ATTR_exec_gpus   "exec_gpus"
-#define ATTR_J           "job_id"
 
 #ifdef USEJOBCREATE
 #define ATTR_pagg         "pagg_id"
 #endif /* USEJOBCREATE */
 
-#ifdef NVIDIA_GPUS
-  /* JOB_ATR_gpu_flags */
-#define ATTR_gpu_flags		"gpu_flags"
-#endif  /* NVIDIA_GPUS */
-
 /* additional queue attributes names */
 
 #define ATTR_aclgren    "acl_group_enable"
@@ -298,18 +289,20 @@
   #define ATTR_authusers       "authorized_users"
 #endif
 #define ATTR_moabarraycompatible "moab_array_compatible"
-#define ATTR_nomailforce       "no_mail_force"
 /* additional node "attributes" names */
 
-#define ATTR_NODE_state         "state"
-#define ATTR_NODE_np            "np"
-#define ATTR_NODE_properties    "properties"
-#define ATTR_NODE_ntype         "ntype"
-#define ATTR_NODE_jobs          "jobs"
-#define ATTR_NODE_status        "status"
-#define ATTR_NODE_note          "note"
+#define ATTR_NODE_state          "state"
+#define ATTR_NODE_np             "np"
+#define ATTR_NODE_properties     "properties"
+#define ATTR_NODE_ntype          "ntype"
+#define ATTR_NODE_jobs           "jobs"
+#define ATTR_NODE_status         "status"
+#define ATTR_NODE_note           "note"
+#define ATTR_NODE_mom_port	     "mom_service_port"
+#define ATTR_NODE_mom_rm_port    "mom_manager_port" 
+#define ATTR_NODE_num_numa_nodes "num_numa_nodes"
+#define ATTR_NODE_numa_str       "numa_node_str"
 #define ATTR_NODE_gpus           "gpus"
-#define ATTR_NODE_gpustatus      "gpu_status"
 
 /* notification email formating */
 #define ATTR_mailsubjectfmt "mail_subject_fmt"
@@ -325,16 +318,16 @@
 #define MAIL_AT_ABORT "a"
 
 
-#define ARRAY_RANGE "array_range="      /* see qdel.c */
-#define DELDELAY  "deldelay="           /* see qdel.c */
-#define DELPURGE  "delpurge="           /* see qdel.c */
-#define DELASYNC  "delasync"            /* see qdel.c */
-#define PURGECOMP  "purgecomplete="     /* see req_delete.c */
-#define EXECQUEONLY  "exec_queue_only"  /* see req_stat.c */
+#define ARRAY_RANGE "array_range=" /* see qdel.c */
+#define DELDELAY  "deldelay=" /* see qdel.c */
+#define DELPURGE  "delpurge="   /* see qdel.c */
+#define DELASYNC  "delasync"   /* see req_delete.c */
+#define PURGECOMP  "purgecomplete="   /* see req_delete.c */
+#define EXECQUEONLY  "exec_queue_only"   /* see req_stat.c */
 #define RERUNFORCE "force"
 
-#define USER_HOLD   "u"
-#define OTHER_HOLD  "o"
+#define USER_HOLD "u"
+#define OTHER_HOLD "o"
 #define SYSTEM_HOLD "s"
 
 /* node-attribute values (state,ntype) */
@@ -401,6 +394,11 @@
 #ifndef MAXPATHLEN
 #define MAXPATHLEN  1024 /* max path name length */
 #endif
+
+#ifndef MAXPORTLEN
+#define MAXPORTLEN  5
+#endif
+
 #ifndef MAXNAMLEN
 #define MAXNAMLEN  255
 #endif
@@ -409,7 +407,6 @@
 
 #define PBS_MAXUSER  32 /* max user name length */
 #define PBS_MAXGRPN  16 /* max group name length */
-#define PBS_MAXGPUID 16 /* max gpu id length */
 #define PBS_MAXQUEUENAME 15 /* max queue name length */
 #define PBS_MAXSERVERNAME PBS_MAXHOSTNAME /* max server name length */
 #define PBS_MAXJOBARRAYLEN      7       /* number of characters allowed in jobarray portion of job id, including '[]' */
@@ -427,8 +424,8 @@
 
 #define PBS_MAXCREDENTIAL_LEN 2048 /* Maximum authentication credential length. */
 /* constants used to indicate version of job ji_qs struct written to disk.
-   starting with 2.4.0 we stopped encoding the torque version number into the
-   hex value e.g. 0x00020300 for torque v2.3.0. Now we just increment a value
+   starting with 2.4.0 we stopped encoding the torque version number into the 
+   hex value e.g. 0x00020300 for torque v2.3.0. Now we just increment a value 
    which is added to the last version encoded that way */
 #define PBS_QS_VERSION_BASE 0x00020300 /* magic number do not change */
 #define PBS_QS_VERSION_INT 2 /* increment this version number with every change to the ji_qs struct */
@@ -476,7 +473,7 @@
 
 #define NO_MOM_RELAY 1
 
-enum batch_op { SET, UNSET, INCR, DECR, EQ, NE, GE, GT, LE, LT, DFLT, MERGE, INCR_OLD };
+enum batch_op { SET, UNSET, INCR, DECR, EQ, NE, GE, GT, LE, LT, DFLT, MERGE };
 
 /*
 ** This structure is identical to attropl so they can be used
@@ -620,11 +617,6 @@ extern int
 extern int
   usepool(int connect, int update);
 
-extern int pbs_gpumode(int connect, char *node, char *gpuid, int gpumode);
-
-extern int trq_set_preferred_network_interface(char *if_name, struct sockaddr *preferred_addr);
-extern char *trq_get_if_name(void);
-
 #endif /* _PBS_IFL_DEF */
 
 /*  end of pbs_ifl.h  */
