@@ -90,6 +90,13 @@
 #ifndef PBS_JOB_H
 #define PBS_JOB_H 1
 
+#include "server_limits.h"
+
+#ifndef MAX_LINE
+#define MAX_LINE 1024
+#endif
+
+/* anything including job.h also needs array.h so lets just include it this way*/
 #ifndef SUCCESS
 #define SUCCESS 1
 #endif 
@@ -190,6 +197,12 @@ struct array_depend_job
 #define JOB_RECOV_RUNNING     2  /* -p option */
 #define JOB_RECOV_DELETE      3  /* -P option */
 
+/* These are flags to distinguish Mother Superior from intermediate moms
+   for a job_radix */
+#define MOTHER_SUPERIOR       0
+#define INTERMEDIATE_MOM      1
+#define LEAF_MOM              2
+
 /*
  * The badplace structure is used to keep track of destinations
  * which have been tried by a route queue and given a "reject"
@@ -271,8 +284,9 @@ enum job_atr
   JOB_ATR_checkpoint, /* alphabetic order for no good reason */
   JOB_ATR_ctime,
   JOB_ATR_depend,
-  JOB_ATR_errpath,
+  JOB_ATR_errpath, /* (10) */
   JOB_ATR_exec_host,
+  JOB_ATR_exec_port,
   JOB_ATR_exec_gpus,
   JOB_ATR_exectime,
   JOB_ATR_grouplst,
@@ -281,7 +295,7 @@ enum job_atr
   JOB_ATR_join,
   JOB_ATR_keep,
   JOB_ATR_mailpnts,
-  JOB_ATR_mailuser,
+  JOB_ATR_mailuser,   /* (20) */
   JOB_ATR_mtime,         /* what triggers modification of mtime? (20) */
   JOB_ATR_outpath,
   JOB_ATR_priority,
@@ -291,10 +305,10 @@ enum job_atr
   JOB_ATR_session_id,
   JOB_ATR_shell,
   JOB_ATR_stagein,
-  JOB_ATR_stageout,
+  JOB_ATR_stageout,           /* (30) */
   JOB_ATR_substate,
   JOB_ATR_userlst,
-  JOB_ATR_variables,    /* (32) */
+  JOB_ATR_variables,    /* (33) */
   /* this set contains private attributes,  */
   /* as such not sent to clients (status)   */
 
@@ -304,7 +318,7 @@ enum job_atr
   JOB_ATR_hopcount,
   JOB_ATR_qrank,
   JOB_ATR_queuetype,
-  JOB_ATR_sched_hint,   /* 39 */
+  JOB_ATR_sched_hint,   /* 40 */
   JOB_ATR_security,
   JOB_ATR_Comment,
   JOB_ATR_Cookie,
@@ -314,8 +328,8 @@ enum job_atr
   JOB_ATR_forwardx11,
   JOB_ATR_submit_args,
   JOB_ATR_job_array_id,
-  JOB_ATR_job_array_request,
-  JOB_ATR_umask,        /* 50 */
+  JOB_ATR_job_array_request, /* (50) */
+  JOB_ATR_umask,
   JOB_ATR_start_time,  /* time when job was first started */
   JOB_ATR_start_count, /* number of times the job has been started */
   JOB_ATR_checkpoint_dir,    /* directory where job checkpoint file is stored */
@@ -329,16 +343,10 @@ enum job_atr
   JOB_ATR_jobtype,     /* opaque job type string */
   JOB_ATR_inter_cmd,      /* command for interactive job */
   JOB_ATR_proxy_user,
-  JOB_ATR_submit_host,  /* host which submitted the job */
-  JOB_ATR_init_work_dir,    /* initial working directory */
+  JOB_ATR_node_exclusive,
 #ifdef USEJOBCREATE
   JOB_ATR_pagg_id,
 #endif /* USEJOBCREATE */
-  JOB_ATR_job_id,
-#ifdef NVIDIA_GPUS
-  JOB_ATR_gpu_flags,		/* gpu flags - mode and reset flags */
-#endif  /* NVIDIA_GPUS */
-  JOB_ATR_arguments,
 #include "site_job_attr_enum.h"
 
   JOB_ATR_UNKN,  /* the special "unknown" type    */
@@ -366,6 +374,7 @@ typedef struct hnodent
   char *hn_host; /* hostname of node */
   int  hn_stream; /* stream to MOM on node */
   int  hn_sister; /* save error for KILL_JOB event */
+  unsigned short hn_port; /*  resmom port default 15003 */
   tlist_head hn_events; /* pointer to list of events */
   } hnodent;
 
@@ -492,6 +501,7 @@ struct job
   /* MOM: time job suspend (Cray) */
   int  ji_modified; /* struct changed, needs to be saved */
   int  ji_momhandle; /* open connection handle to MOM */
+  int  ji_radix;    /* number of nodes in a job radix. used for qsub -W job_radix option  */
 #ifdef PBS_MOM    /* MOM ONLY */
 
   struct grpcache *ji_grpcache; /* cache of user's groups */
@@ -502,21 +512,38 @@ struct job
   int (*ji_mompost)();        /* ptr to post processing func  */
 
   struct batch_request *ji_preq; /* hold request until finish_exec */
-  int  ji_numnodes; /* number of nodes (at least 1) */
-  int  ji_numvnod; /* number of virtual nodes */
-  tm_node_id ji_nodeid; /* my node id */
-  tm_task_id ji_taskid; /* generate task id's for job */
-  char ji_altid[PBS_MAXSVRJOBID + 1];
-  tm_event_t ji_obit; /* event for end-of-job */
+  int            ji_numnodes; /* number of nodes (at least 1) */
+  int            ji_numsisternodes; /* number of nodes nodes for job_radix (at least 1) */
+  int            ji_numvnod; /* number of virtual nodes */
+  int            ji_numsistervnod; /* number of virtual nodes job_radix*/
+  int            ji_outstanding;  /* number of nodes left to reply to an IM request*/
+  tm_node_id     ji_im_nodeid;   /* node id of an intermediate mom. If set to 1 is intermediate mom. 0 leaf or mother superior */
+  tm_node_id     ji_nodeid; /* my node id */
+  tm_task_id     ji_taskid; /* generate task id's for job */
+  char           ji_altid[PBS_MAXSVRJOBID + 1];
+  tm_event_t     ji_obit; /* event for end-of-job */
   hnodent        *ji_hosts; /* ptr to job host management stuff */
+  hnodent        *ji_sisters; /* ptr to job host management stuff for intermediate moms */
   vnodent        *ji_vnods; /* ptr to job vnode management stuff */
+  vnodent        *ji_sister_vnods; /* ptr to job vnode management stuff */
   noderes        *ji_resources; /* ptr to array of node resources */
-  tlist_head      ji_tasks; /* list of task structs */
-  tm_node_id ji_nodekill; /* set to nodeid requesting job die */
-  int  ji_flags; /* mom only flags */
-  char        *ji_globid; /* global job id */
-  int  ji_stdout; /* port for stdout */
-  int  ji_stderr; /* port for stderr */
+  tlist_head     ji_tasks; /* list of task structs */
+  tm_node_id     ji_nodekill; /* set to nodeid requesting job die */
+  int            ji_flags; /* mom only flags */
+  char           *ji_globid; /* global job id */
+  int            ji_portout; /* socket port allocated for ji_stdout */
+  int            ji_porterr; /* socket port allocated for ji_stderr */
+  int            ji_stdout; /* socket for stdout */
+  int            ji_stderr; /* socket for stderr */
+  int            ji_im_stdout;
+  int            ji_im_stderr;
+  int            ji_im_portout; /* for job_radix intermediate mom demux port for ji_stdout */
+  int            ji_im_porterr; /* for job_radix intermediate mom demux port for ji_stderr */
+#ifdef PENABLE_LINUX26_CPUSETS
+  int            ji_mempressure_curr;  /* current memory_pressure value */
+  int            ji_mempressure_cnt;   /* counts MOM cycles memory_pressure is over threshold */
+#endif
+
 #else     /* END MOM ONLY */
   tlist_head ji_svrtask; /* links to svr work_task list */
 
@@ -568,6 +595,8 @@ struct job
       struct   /* if in execution queue .. */
         {
         pbs_net_t ji_momaddr;  /* host addr of Server */
+        unsigned short ji_momport;  /* host port of Server default 15002 */
+        unsigned short ji_mom_rmport; /* host mom manager port of Server default 15003 */
         int       ji_exitstat; /* job exit status from MOM */
         } ji_exect;
 
@@ -695,27 +724,6 @@ typedef struct infoent
   list_link ie_next; /* link to next one */
   } infoent;
 
-
-
-
-
-#ifdef PBS_MOM
-typedef struct job_file_delete_info
-  {
-  char           jobid[PBS_MAXSVRJOBID+1];
-  char           prefix[PBS_JOBBASE + 1];
-  char          *checkpoint_dir;
-  unsigned char  has_temp_dir;
-  unsigned char  has_node_file;
-  gid_t          gid;
-  uid_t          uid;
-  } job_file_delete_info;
-#endif
-
-
-
-
-
 #define TI_FLAGS_INIT           1  /* task has called tm_init */
 #define TI_FLAGS_CHECKPOINT     2  /* task has checkpointed */
 
@@ -728,40 +736,38 @@ typedef struct job_file_delete_info
 #define TI_STATE_EXITED  2  /* ti_exitstat valid */
 #define TI_STATE_DEAD    3
 
+
 /*
 **      Here is the set of commands for InterMOM (IM) requests.
 */
 
 /* sync w/PMOMCommand[] */
 
-#define IM_ALL_OKAY 0
-#define IM_JOIN_JOB 1
-#define IM_KILL_JOB 2
-#define IM_SPAWN_TASK 3
-#define IM_GET_TASKS 4
-#define IM_SIGNAL_TASK 5
-#define IM_OBIT_TASK 6
-#define IM_POLL_JOB 7
-#define IM_GET_INFO 8
-#define IM_GET_RESC 9
-#define IM_ABORT_JOB 10
-#define IM_GET_TID 11
+#define IM_ALL_OKAY        0
+#define IM_JOIN_JOB        1
+#define IM_KILL_JOB        2
+#define IM_SPAWN_TASK      3
+#define IM_GET_TASKS       4
+#define IM_SIGNAL_TASK     5
+#define IM_OBIT_TASK       6
+#define IM_POLL_JOB        7
+#define IM_GET_INFO        8
+#define IM_GET_RESC        9
+#define IM_ABORT_JOB      10
+#define IM_GET_TID        11
 #define IM_ERROR 99
 
 #define IM_MAX          12
 
-eventent *event_alloc(
-                           int  command,
+eventent *event_alloc( int  command,
                            hnodent *pnode,
                            tm_event_t event,
                            tm_task_id taskid);
 
-task *pbs_task_create(
-                           job *pjob,
+task *pbs_task_create( job *pjob,
                            tm_task_id taskid);
 
-task *task_find(
-                     job *pjob,
+task *task_find( job *pjob,
                      tm_task_id taskid);
 
 #endif /* MOM */
@@ -914,7 +920,6 @@ dir so that job can be restarted */
 #define JOB_EXEC_BADRESRT -7 /* job restart failed */
 #define JOB_EXEC_CMDFAIL  -8 /* exec() of user command failed */
 #define JOB_EXEC_STDOUTFAIL -9 /* could not create/open stdout stderr files */
-#define JOB_EXEC_OVERLIMIT      -10 /* job aborted after successful start */
 
 extern void  add_dest(job *);
 extern void  depend_clrrdy(job *);
@@ -935,7 +940,7 @@ extern job  *job_clone(job *,struct job_array *, int);
 extern void  job_free(job *);
 extern void  job_purge(job *);
 extern job  *job_recov(char *);
-extern int   job_save(job *, int);
+extern int   job_save(job *, int, int);
 extern int   modify_job_attr(job *, svrattrl *, int, int *);
 extern char *prefix_std_file(job *, int);
 extern char *add_std_filename(job *, char *, int);
