@@ -185,11 +185,12 @@ enum srv_atr
   SRV_ATR_RecordJobScript,
   SRV_ATR_JobLogFileMaxSize,
   SRV_ATR_JobLogFileRollDepth,
-  SRV_ATR_JobLogKeepDays,        /* 80 */
+  SRV_ATR_JobLogKeepDays,
 #ifdef MUNGE_AUTH
   SRV_ATR_authusers,
 #endif
   SRV_ATR_MoabArrayCompatible,
+
 #include "site_svr_attr_enum.h"
   /* This must be last */
   SRV_ATR_LAST
@@ -254,7 +255,6 @@ extern struct server server;
 /* function prototypes */
 
 extern int svr_recov_xml(char *, int);
-extern int svr_recov(char *, int);
 extern int svr_save(struct server *, int);
 
 
@@ -273,6 +273,7 @@ extern int svr_save(struct server *, int);
 #define JobStatRate MAX(PBS_JOBSTAT_MIN,server.sv_attr[(int)SRV_ATR_JobStatRate].at_val.at_long)
 #endif
 
+
 /* maintain a list of new nodes */
 
 typedef struct new_node
