@@ -149,7 +149,7 @@ int decode_arst_direct(
     return(PBSE_SYSTEM);
     }
 
-  memset(tmpval, 0, ssize);
+  tmpval[0] = 0;
 
   strcpy(tmpval,val);
 
@@ -182,20 +182,15 @@ int decode_arst_direct(
     return(PBSE_SYSTEM);
     }
 
-  memset(pbuf, 0, ssize);
-
   bksize = (ns - 1) * sizeof(char *) + sizeof(struct array_strings);
 
-  if (( patr->at_val.at_arst = (struct array_strings *)malloc(bksize)) == NULL)
+  if ((stp = (struct array_strings *)malloc(bksize)) == NULL)
     {
     /* FAILURE */
 
     return(PBSE_SYSTEM);
     }
 
-  stp = patr->at_val.at_arst;
-  memset(stp, 0, bksize);
-
   stp->as_npointers = ns;
 
   /* number of slots (sub strings) */
@@ -239,7 +234,7 @@ int decode_arst_direct(
 
   patr->at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
 
-/*  patr->at_val.at_arst = stp;*/
+  patr->at_val.at_arst = stp;
 
   free(tmpval);
  
@@ -283,13 +278,12 @@ int decode_arst(
     return(0);
     }
 
-  memset(&temp, 0, sizeof(attribute));
-
   if ((patr->at_flags & ATR_VFLAG_SET) && (patr->at_val.at_arst))
     {
     /* already have values, decode new into temp */
     /* then use set(incr) to add new to existing */
 
+    temp.at_val.at_arst = NULL;
 
     /* convert value string into attribute array */
 
@@ -506,6 +500,9 @@ int encode_arst(
       case '\n':
 
         *pc++ = '\\';
+        *pc   = separator;
+
+        break;
 
       default:
 
@@ -588,7 +585,7 @@ int set_arst(
       return(PBSE_INTERNAL);
       }
 
-    need = (int)sizeof(struct array_strings) + (j - 1) * sizeof(char *);
+    need = sizeof(struct array_strings) + (j - 1) * sizeof(char *);
 
     pas = (struct array_strings *)malloc((size_t)need);
 
@@ -662,9 +659,8 @@ int set_arst(
 
       pas->as_next = pas->as_buf;
 
-      /* no break, "SET" falls into "MERGE" to add strings */
+      /* no break, "SET" falls into "INCR" to add strings */
 
-    case INCR_OLD:
     case MERGE:
 
       nsize = newpas->as_next - newpas->as_buf;   /* space needed */
@@ -779,8 +775,6 @@ int set_arst(
       if (pc == NULL)
         return(PBSE_SYSTEM);
 
-      memset(pc, 0, need);
-
       tmp_arst->as_buf = pc;
       tmp_arst->as_next = pc;
       tmp_arst->as_npointers = j;
@@ -821,7 +815,7 @@ int set_arst(
           }
         if (MatchFound == 0)
           {
-          strcpy(tmp_arst->as_next, pas->as_string[i]);
+          strcpy(tmp_arst->as_next,pas->as_string[i]);
           
           tmp_arst->as_string[tmp_arst->as_usedptr++] = tmp_arst->as_next;
           tmp_arst->as_next += strlen(tmp_arst->as_next) + 1;
