@@ -86,11 +86,6 @@
 
 #include <pbs_config.h>   /* the master config generated by configure */
 
-#include <sys/ioctl.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <net/if.h>
-#include <arpa/inet.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <stdio.h>
@@ -101,7 +96,9 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
 #include <sys/un.h>
+#include <netinet/in.h>
 #include <sys/param.h>
 #if HAVE_SYS_UCRED_H
 #include <sys/ucred.h>
@@ -119,7 +116,6 @@
 #include "csv.h"
 #include "dis.h"
 #include "net_connect.h"
-#include "log.h"
 
 
 
@@ -363,98 +359,121 @@ static char *PBS_get_server(
  * a user connection with the server. 
  */
 #ifdef MUNGE_AUTH
-int PBSD_munge_authenticate(
-
-  int psock,  /* I */
-  int handle) /* I */
-
+static int PBSD_munge_authenticate(
+  int psock) /* I */
   {
-  static char        *id = "PBSD_munge_authenticate";
-  int                 rc;
+  int fd_pipe[2];
 
-  int                 fd;
-  FILE               *munge_pipe;
-  char                munge_buf[MUNGE_SIZE];
-  char                munge_command[MUNGE_SIZE];
-  char               *ptr; /* pointer to the current place to copy data into munge_buf */
-  int                 bytes_read;
-  int                 total_bytes_read = 0;        
+  char execname[20]; /* for execvp. will be "munge" */
+  char *options[3];  /* argument list for execvp */
+  char com1[20];     /* first argument to execvp */
+  char com2[5];      /* second argument to execvp */
+  int  rc;
 
+  /* These variables are used to create a pseudo-random file name
+     to store the munge certificate */
+  int newfd;
+  pid_t pid;
+  char buf[MUNGE_SIZE];
+  char munge_buf[MUNGE_SIZE];
+  char *ptr; /* pointer to the current place to copy data into munge_buf */
+  int bytes_read;
+  int total_bytes_read = 0;        
+  
   /* user id and name stuff */
-  struct passwd      *pwent;
-  uid_t               myrealuid;
-  struct batch_reply *reply;
-  unsigned short      user_port = 0;
-  struct sockaddr_in  sockname;
-  socklen_t           socknamelen = sizeof(sockname);
+  struct passwd *pwent;
+  uid_t   myrealuid;
+  struct batch_reply   *reply;
+  unsigned short user_port = 0;
+  struct sockaddr_in sockname;
+  socklen_t socknamelen = sizeof(sockname);
+       
 
-  snprintf(munge_command, sizeof(munge_command), "munge -n 2>/dev/null");
-  
-  memset(munge_buf, 0, sizeof(munge_buf));
-  ptr = munge_buf;
+  memset(buf, 0, MUNGE_SIZE);
+  memset(munge_buf, 0, MUNGE_SIZE);
+  ptr = munge_buf; 
 
-  if (((munge_pipe) = popen(munge_command, "r")) == NULL)
+  rc = pipe(fd_pipe);
+  if(rc == -1)
     {
-    /* FAILURE */
+    fprintf(stderr, "error opening pipe in PBSD_munge_authenticate: errno = %d\n", errno);
     return(-1);
     }
+ 
+  /* Time to Fork. The child will call munge and write the certificate to the file
+     just created for mungeFileName. The parent will wait for the file to be written
+     and then copy the certificate to the batch request and send it to the server */
+  pid = fork();
+  if(pid != 0)    {
+    /* This is the parent*/
+    /* set up the pipe to be able to read from the child */
+    close(fd_pipe[1]);
+    do
+      {
+      bytes_read = read(fd_pipe[0], buf, MUNGE_SIZE);
+      if(bytes_read > 0)
+        {
+        total_bytes_read += bytes_read;
+        memcpy(ptr, buf, bytes_read);
+        ptr += bytes_read;
+        }
+      }while(bytes_read > 0);
 
-  fd = fileno(munge_pipe);
+      if(bytes_read == -1)
+        {
+        /* read failed */
+        fprintf(stderr, "error reading pipe in PBSD_munge_authenticate: errno = %d\n", errno);
+        return(-1);
+        }
 
-  while ((bytes_read = read(fd, ptr, MUNGE_SIZE - total_bytes_read)) > 0)
-    {
-    total_bytes_read += bytes_read;
-    ptr += bytes_read;
-    }
+      /* We got the certificate. Now make the PBS_BATCH_AltAuthenUser request */
+      myrealuid = getuid();
 
-  pclose(munge_pipe);
+      pwent = getpwuid(myrealuid);
 
-  if (bytes_read == -1)
-    {
-    /* read failed */
-    log_err(errno, id, "error reading pipe in PBSD_munge_authenticate");
-    return(-1);
-    }
+      rc = getsockname(psock, 
+                       (struct sockaddr *)&sockname,
+                       &socknamelen);
 
-  /* if we got no bytes back then Munge may not be installed etc. */
-  if (total_bytes_read == 0)
-    {
-    return(PBSE_MUNGE_NOT_FOUND);
-    }
-  
-  /* We got the certificate. Now make the PBS_BATCH_AltAuthenUser request */
-  myrealuid = getuid();
-  pwent = getpwuid(myrealuid);
-  
-  rc = getsockname(psock, (struct sockaddr *)&sockname, &socknamelen);
-  
-  if (rc == -1)
-    {
-    fprintf(stderr, "getsockname failed: %d\n", errno);
-    return(-1);
-    }
-  
-  user_port = ntohs(sockname.sin_port);
-  
-  DIS_tcp_setup(psock);
-  
-  if ((rc = encode_DIS_ReqHdr(psock, PBS_BATCH_AltAuthenUser, pwent->pw_name)) ||
-      (rc = diswui(psock, user_port)) ||
-      (rc = diswst(psock, munge_buf)) ||
-      (rc = encode_DIS_ReqExtend(psock, NULL)) ||
-      (rc = DIS_tcp_wflush(psock)))
-    {
-    /* ERROR */
-    return(rc);
+      if(rc == -1)
+        {
+        fprintf(stderr, "getsockname failed: %d\n", errno);
+        return(-1);
+        }
+
+      user_port = ntohs(sockname.sin_port);
+
+      DIS_tcp_setup(psock);
+
+      rc = encode_DIS_ReqHdr(psock, PBS_BATCH_AltAuthenUser, pwent->pw_name);
+      rc = diswui(psock, user_port);
+      rc = diswst(psock, munge_buf);
+      rc = encode_DIS_ReqExtend(psock, NULL);
+      rc = DIS_tcp_wflush(psock);
+
+      /* read the reply */
+      reply = PBSD_rdrpy(1);
     }
   else
     {
-    /* read the reply */
-    if ((reply = PBSD_rdrpy(handle)) != NULL)
-      free(reply);
-    
-    return(PBSE_NONE);
+    close(fd_pipe[0]);
+    newfd = dup2(fd_pipe[1], 1);
+    strcpy(execname, "munge");
+    strcpy(com1, "munge");
+    strcpy(com2, "-n");
+    options[0] = com1;
+    options[1] = com2;
+    options[2] = NULL;
+
+    rc = execvp(execname, options);
+
+    /* Something went wrong. Let the user know*/
+    fprintf(stderr, "execve: %d: errno: %d\n", rc, errno);
+
+    exit(0);
     }
+
+  return(0);  
   }
 #endif /* ifdef MUNGE_AUTH */
 
@@ -464,7 +483,7 @@ int PBSD_munge_authenticate(
  */
 
 #ifndef MUNGE_AUTH
-int PBSD_authenticate(
+static int PBSD_authenticate(
 
   int psock)  /* I */
 
@@ -475,8 +494,6 @@ int PBSD_authenticate(
   int    j;
   FILE *piff;
   char  *ptr;
-  char  *path_copy;
-  char  *token;
 
   struct stat buf;
 
@@ -508,26 +525,18 @@ int PBSD_authenticate(
         {
         ptr = strtok(ptr, ":");
 
-        if ((path_copy = strdup(ptr)) == NULL)
+        while (ptr != NULL)
           {
-          fprintf(stderr,"Cannot allocate memory, FAILURE");
-
-          return(-1);
-          }
-
-        token = strtok(path_copy, ":");
-
-        while (token != NULL)
-          {
-          snprintf(iffpath, sizeof(iffpath), "%s/pbs_iff", token);
+          snprintf(iffpath, sizeof(iffpath), "%s/pbs_iff",
+                   ptr);
 
           rc = stat(iffpath, &buf);
 
           if (rc != -1)
             break;
 
-          token = strtok(NULL, ":");
-          }  /* END while (token != NULL) */
+          ptr = strtok(NULL, ":");
+          }  /* END while (ptr != NULL) */
         }    /* END if ((ptr = getenv("PATH")) != NULL) */
 
       if (rc == -1)
@@ -665,79 +674,6 @@ ssize_t    send_unix_creds(int sd)
 
 #endif /* END ENABLE_UNIX_SOCKETS */
 
-/*
- * Set a preferred outbound interface for communication
- * 
- * The current solution uses a Linux ioctl call to
- * find the named interfaces address. We will need
- * to conditionally compile this for other platforms
- *
- * Return 0 on success. Non-zero on failure
- */
-#define MAX_IFS 64
-
-int trq_set_preferred_network_interface(
-    
-  char            *if_name,
-  struct sockaddr *preferred_addr)
-
-  {
-	struct ifreq *ifr;
-  struct ifreq *ifend;
-	struct ifreq  ifreqx;
-	struct ifconf ifc;
-	struct ifreq  ifs[MAX_IFS];
-	int           sockfd;
-
-	/* make sure we have a valid name for the interface */
-  if ((if_name == NULL) ||
-      (preferred_addr == NULL))
-		return(PBSE_IVALREQ);	
-
-  memset(preferred_addr, 0, sizeof(struct sockaddr));
-
-	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
-	if (sockfd < 0)
-    {
-    /* We can't do this without a socket */
-    return(PBSE_SYSTEM);
-    }
-
-	ifc.ifc_len = sizeof(ifs);
-	ifc.ifc_req = ifs;
-	if (ioctl(sockfd, SIOCGIFCONF, &ifc) < 0)
-    {
-    close(sockfd);
-    return(PBSE_SYSTEM);
-    }
-
-	/* We will walk through all the interfaces until we find the one we are looking for */
-  ifend = ifs + (ifc.ifc_len / sizeof(struct ifreq));
-  for (ifr = ifc.ifc_req; ifr < ifend; ifr++)
-    {
-    if (ifr->ifr_addr.sa_family == AF_INET)
-      {
-      if (!strncmp(if_name, ifr->ifr_name, IF_NAMESIZE))
-        {
-        strncpy(ifreqx.ifr_name, ifr->ifr_name,sizeof(ifreqx.ifr_name));
-
-        if (ioctl(sockfd, SIOCGIFADDR, &ifreqx) < 0)
-        	{
-        	close(sockfd);
-        	return(PBSE_SYSTEM);
-          }
-
-        /* get the address */
-        memcpy(preferred_addr, &ifreqx.ifr_ifru.ifru_addr, sizeof(struct sockaddr));
-        }
-      }
-    }
-
-  close(sockfd);
-
-  return(PBSE_NONE);
-  } /* END trq_set_preferred_network_interface() */
-
 
 /* returns socket descriptor or negative value (-1) on failure */
 
@@ -753,14 +689,12 @@ int pbs_original_connect(
 
   {
   struct sockaddr_in server_addr;
-  char *if_name;
+
   struct hostent *hp;
   int out;
   int i;
   int auth;
-  int rc;
 
-  struct sockaddr preferred_addr; /* set if TRQ_IFNAME set in torque.cfg */
   struct passwd *pw;
   int use_unixsock = 0;
 #ifdef ENABLE_UNIX_SOCKETS
@@ -970,38 +904,9 @@ int pbs_original_connect(
                 strerror(errno));
         }
 
-      return(PBSE_SYSTEM);
+      return(-1);
       }
 
-    if_name = trq_get_if_name();
-    if(if_name)
-      {
-      rc = trq_set_preferred_network_interface(if_name, &preferred_addr);
-      if(rc != PBSE_NONE)
-        {
-        fprintf(stderr, "could not set preferred network interface (%s): %d\n",
-                if_name, rc);
-        if(if_name)
-          free(if_name);
-        return(rc);
-        }
-
-      rc = bind(connection[out].ch_socket, &preferred_addr, sizeof(struct sockaddr));
-      if(rc < 0)
-        {
-        fprintf(stderr, "ERROR: could not bind preferred network interface (%s): errno: %d",
-                if_name, errno);
-        if(if_name)
-          free(if_name);
-        return(PBSE_SYSTEM);
-        }
-      }
-
-    /* we are done with if_name at this point. trq_get_if_name allocated
-       space for it. We need to free it */
-    if(if_name)
-      free(if_name);
-
     memcpy((char *)&server_addr.sin_addr, hp->h_addr_list[0], hp->h_length);
 
     server_addr.sin_port = htons(server_port);
@@ -1037,7 +942,7 @@ int pbs_original_connect(
 #endif
 
 #ifdef MUNGE_AUTH
-    auth = PBSD_munge_authenticate(connection[out].ch_socket, out);
+    auth = PBSD_munge_authenticate(connection[out].ch_socket);
     if(auth != 0)
       {
       close(connection[out].ch_socket);
@@ -1137,7 +1042,7 @@ int pbs_disconnect(
 
   {
   int  sock;
-  int  rc;
+  static char x[THE_BUF_SIZE / 4];
 
   /* send close-connection message */
 
@@ -1145,18 +1050,42 @@ int pbs_disconnect(
 
   DIS_tcp_setup(sock);
 
-  rc = encode_DIS_ReqHdr(sock, PBS_BATCH_Disconnect, pbs_current_user);
-  if(rc)
+  if ((encode_DIS_ReqHdr(sock, PBS_BATCH_Disconnect, pbs_current_user) == 0) &&
+      (DIS_tcp_wflush(sock) == 0))
     {
-    return(rc);
+    int atime;
+
+    struct sigaction act;
+
+    struct sigaction oldact;
+
+    /* set alarm to break out of potentially infinite read */
+
+/*    act.sa_handler = SIG_IGN; */
+    act.sa_handler = empty_alarm_handler;  /* need SOME handler or blocking read never gets interrupted */
+
+    sigemptyset(&act.sa_mask);
+    act.sa_flags = 0;
+    sigaction(SIGALRM, &act, &oldact);
+
+    atime = alarm(pbs_tcp_timeout);
+
+    while (1)
+      {
+      /* wait for server to close connection */
+
+      /* NOTE:  if read of 'sock' is blocking, request below may hang forever
+         -- hence the signal handler above */
+
+      if (read(sock, &x, sizeof(x)) < 1)
+        break;
+      }
+
+    alarm(atime);
+
+    sigaction(SIGALRM, &oldact, NULL);
     }
 
-   rc = DIS_tcp_wflush(sock);
-   if(rc)
-     {
-     return(rc);
-     }
-    
   close(sock);
 
   if (connection[connect].ch_errtxt != (char *)NULL)
@@ -1198,8 +1127,6 @@ int pbs_connect(char *server_name_ptr)    /* I (optional) */
   if (server_name_ptr && server_name_ptr[0])
     {
     strncpy(server_name_list, server_name_ptr, sizeof(server_name_list) - 1);
-    strcat(server_name_list, ",");
-    strncat(server_name_list, pbs_get_server_list(), sizeof(server_name_list) -1 - strlen(server_name_ptr) - 1);
 
     if (getenv("PBSDEBUG"))
       {
