@@ -92,7 +92,7 @@
 #include "server_limits.h"
 #include "server.h"
 #include "queue.h"
-#include "job.h"
+#include "pbs_job.h"
 #include "log.h"
 #include "batch_request.h"
 #include "pbs_nodes.h"
@@ -107,8 +107,6 @@ extern char *pbs_o_host;
 extern char  server_host[];
 extern char *msg_orighost; /* error message: no PBS_O_HOST */
 
-extern int   LOGLEVEL;
-
 /*
  * site_check_u - site_check_user_map()
  *
@@ -140,6 +138,10 @@ int site_check_user_map(
 
   char  *dptr;
 
+#ifdef MUNGE_AUTH
+  char  uh[PBS_MAXUSER + PBS_MAXHOSTNAME + 2];
+#endif
+
   if (EMsg != NULL)
     EMsg[0] = '\0';
 
@@ -172,8 +174,8 @@ int site_check_user_map(
     return(-1);
     }
 
-  if ((server.sv_attr[(int)SRV_ATR_AllowProxyUser].at_flags & ATR_VFLAG_SET) && \
-      (server.sv_attr[(int)SRV_ATR_AllowProxyUser].at_val.at_long == 1))
+  if ((server.sv_attr[SRV_ATR_AllowProxyUser].at_flags & ATR_VFLAG_SET) && \
+      (server.sv_attr[SRV_ATR_AllowProxyUser].at_val.at_long == 1))
     {
     ProxyAllowed = 1;
     }
@@ -193,18 +195,6 @@ int site_check_user_map(
       }
 
     /* host is fine, must validate proxy via ruserok() */
-    sprintf(log_buffer,
-            "Setting HostAllowed for originating host %s - server host",
-            orighost);
-
-    if (LOGLEVEL >= 7)
-      {
-      log_event(
-        PBSEVENT_ERROR | PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
-        log_buffer);
-      }
 
     HostAllowed = 1;
     }
@@ -232,60 +222,24 @@ int site_check_user_map(
       }
 
     /* host is fine, must validate proxy via ruserok() */
-    sprintf(log_buffer,
-            "Setting HostAllowed for originating host %s - AllowNodeSubmit is set",
-            orighost);
-
-    if (LOGLEVEL >= 7)
-      {
-      log_event(
-        PBSEVENT_ERROR | PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
-        log_buffer);
-      }
 
     HostAllowed = 1;
     }
 
-  if (server.sv_attr[(int)SRV_ATR_SubmitHosts].at_flags & ATR_VFLAG_SET)
-    {
-    sprintf(log_buffer, "Submit hosts is ALLOWED");
-
-    if (LOGLEVEL >= 7)
-      {
-      log_event(
-        PBSEVENT_ERROR | PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
-        log_buffer);
-      }
-    }
-
   if ((HostAllowed == 0) &&
-      (server.sv_attr[(int)SRV_ATR_SubmitHosts].at_flags & ATR_VFLAG_SET))
+      (server.sv_attr[SRV_ATR_SubmitHosts].at_flags & ATR_VFLAG_SET))
     {
 
     struct array_strings *submithosts = NULL;
     char                 *testhost;
     int                   hostnum = 0;
 
-    submithosts = server.sv_attr[(int)SRV_ATR_SubmitHosts].at_val.at_arst;
+    submithosts = server.sv_attr[SRV_ATR_SubmitHosts].at_val.at_arst;
 
     for (hostnum = 0;hostnum < submithosts->as_usedptr;hostnum++)
       {
       testhost = submithosts->as_string[hostnum];
 
-      if (LOGLEVEL >= 7)
-        {
-        sprintf(log_buffer, "Checking submit host [%d] - %s", hostnum, testhost);
-        log_event(
-          PBSEVENT_ERROR | PBSEVENT_JOB,
-          PBS_EVENTCLASS_JOB,
-          pjob->ji_qs.ji_jobid,
-          log_buffer);
-        }
-
       if (!strcasecmp(testhost, orighost))
         {
         /* job submitted from host found in trusted submit host list, access allowed */
@@ -293,17 +247,6 @@ int site_check_user_map(
         if (dptr != NULL)
           *dptr = '.';
 
-        if (LOGLEVEL >= 7)
-          {
-          sprintf(log_buffer, "Found submit host %s (%d %d)", testhost,
-                  ProxyRequested, ProxyAllowed);
-          log_event(
-            PBSEVENT_ERROR | PBSEVENT_JOB,
-            PBS_EVENTCLASS_JOB,
-            pjob->ji_qs.ji_jobid,
-            log_buffer);
-          }
-
         if ((ProxyRequested == 0) || (ProxyAllowed == 1))
           {
           return(0);
@@ -321,6 +264,28 @@ int site_check_user_map(
   if (dptr != NULL)
     *dptr = '.';
 
+#ifdef MUNGE_AUTH
+  sprintf(uh, "%s@%s", owner, orighost);
+  rc = acl_check(&server.sv_attr[SRV_ATR_authusers], uh, ACL_User);
+  if(rc <= 0)
+    {
+    /* rc == 0 means we did not find a match.
+       this is a failure */
+    if(EMsg != NULL)
+      {
+      snprintf(EMsg, 1024, "could not authorize user %s from %s",
+               owner, orighost);
+      }
+    rc = -1; /* -1 is what set_jobexid is expecting for a failure*/
+    }
+  else
+    {
+    /*SUCCESS*/
+    rc = 0; /* the call to ruserok below was in the code first. ruserok returns 
+               0 on success but acl_check returns a positive value on success. 
+               We set rc to 0 to be consistent with the original ruserok functionality */
+    }
+#else
   rc = ruserok(orighost, 0, owner, luser);
 
   if (rc != 0 && EMsg != NULL)
@@ -333,18 +298,10 @@ int site_check_user_map(
              owner,
              luser,
              orighost);
-
-    if (LOGLEVEL >= 7)
-      {
-      sprintf(log_buffer, "Failed submit host %s (%d %d %d)", orighost,
-              ProxyRequested, ProxyAllowed, rc);
-      log_event(
-        PBSEVENT_ERROR | PBSEVENT_JOB,
-        PBS_EVENTCLASS_JOB,
-        pjob->ji_qs.ji_jobid,
-        log_buffer);
-      }
     }
+#endif
+
+   
 
 #ifdef sun
   /* broken Sun ruserok() sets process so it appears to be owned */
