@@ -134,10 +134,12 @@ extern int  exiting_tasks;
 extern char  *msg_daemonname;
 extern int  termin_child;
 
-extern struct connection svr_conn[];
 extern int  resc_access_perm;
 extern char  *path_aux;
 
+extern int   multi_mom;
+extern int   pbs_rm_port;
+
 extern int   LOGLEVEL;
 
 extern char  *PJobSubState[];
@@ -296,8 +298,8 @@ hnodent *get_node(
  *   JOB_SUBSTATE_OBIT (58) - preobit_reply()
  */
 
-void scan_for_exiting(void)
-
+void
+scan_for_exiting(void)
   {
   char         *id = "scan_for_exiting";
 
@@ -313,6 +315,7 @@ void scan_for_exiting(void)
   task *task_find(job *, tm_task_id);
   int im_compose(int, char *, char *, int, tm_event_t, tm_task_id);
 
+  unsigned int momport = 0;
   static int ForceObit    = -1;   /* boolean - if TRUE, ObitsAllowed will be enforced */
   static int ObitsAllowed = 1;
 
@@ -408,12 +411,12 @@ void scan_for_exiting(void)
       }
 
 
-    if (!(pjob->ji_wattr[(int)JOB_ATR_Cookie].at_flags & ATR_VFLAG_SET))
+    if (!(pjob->ji_wattr[JOB_ATR_Cookie].at_flags & ATR_VFLAG_SET))
       {
       continue;
       }
 
-    cookie = pjob->ji_wattr[(int)JOB_ATR_Cookie].at_val.at_str;
+    cookie = pjob->ji_wattr[JOB_ATR_Cookie].at_val.at_str;
 
     /*
     ** Check each EXITED task.  They transition to DEAD here.
@@ -435,10 +438,7 @@ void scan_for_exiting(void)
         {
         /* master task is in state TI_STATE_EXITED */
 
-        if(pjob->ji_qs.ji_un.ji_momt.ji_exitstat != JOB_EXEC_OVERLIMIT)
-          {
-          pjob->ji_qs.ji_un.ji_momt.ji_exitstat = ptask->ti_qs.ti_exitstat;
-          }
+        pjob->ji_qs.ji_un.ji_momt.ji_exitstat = ptask->ti_qs.ti_exitstat;
 
         LOG_EVENT(
           PBSEVENT_JOB,
@@ -463,12 +463,16 @@ void scan_for_exiting(void)
 
           pjob->ji_qs.ji_substate = JOB_SUBSTATE_EXITING;
 
-          job_save(pjob, SAVEJOB_QUICK);
+          if (multi_mom)
+            {
+            momport = pbs_rm_port;
+            }
+          job_save(pjob, SAVEJOB_QUICK, momport);
           }
         else if (LOGLEVEL >= 3)
           {
-          snprintf(log_buffer, 1024, "master task has exited - sent kill job request to %d sisters",
-                   NumSisters);
+          snprintf(log_buffer, 1024, "%s: master task has exited - sent kill job request to %d sisters",
+                   id, NumSisters);
 
           LOG_EVENT(
             PBSEVENT_JOB,
@@ -492,7 +496,11 @@ void scan_for_exiting(void)
 
         /* see if this is me or another MOM */
 
+#ifndef NUMA_SUPPORT
+        /* for NUMA, we are always the mother superior and the correct
+         * node for everything to happen */
         if (pjob->ji_nodeid == pnode->hn_node)
+#endif /* ndef NUMA_SUPPORT */
           {
           task *tp;
 
@@ -511,6 +519,7 @@ void scan_for_exiting(void)
             DIS_tcp_wflush(tp->ti_fd);
             }
           }
+#ifndef NUMA_SUPPORT 
         else if (pnode->hn_stream != -1)
           {
           /*
@@ -530,6 +539,7 @@ void scan_for_exiting(void)
 
           rpp_flush(pnode->hn_stream);
           }
+#endif /* ndef NUMA_SUPPORT */
 
         delete_link(&pobit->oe_next);
 
@@ -563,9 +573,9 @@ void scan_for_exiting(void)
       {
       if (LOGLEVEL >= 3)
         {
-        snprintf(log_buffer, 1024, "job is in non-exiting substate %s, no obit sent at this time",
-                 PJobSubState[pjob->ji_qs.ji_substate]);
-
+        snprintf(log_buffer, 1024, "%s:job is in non-exiting substate %s, no obit sent at this time",
+                 id, PJobSubState[pjob->ji_qs.ji_substate]);
+    
         LOG_EVENT(
           PBSEVENT_JOB,
           PBS_EVENTCLASS_JOB,
@@ -584,6 +594,7 @@ void scan_for_exiting(void)
     ** so I can send the obit (unless she died).
     */
 
+#ifndef NUMA_SUPPORT
     if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0)
       {
       int stream;
@@ -609,7 +620,7 @@ void scan_for_exiting(void)
             "connection to server lost - no obit sent - job will be purged");
           }
 
-        if(pjob->ji_qs.ji_substate != JOB_SUBSTATE_NOTERM_REQUE)
+        if (pjob->ji_qs.ji_substate != JOB_SUBSTATE_NOTERM_REQUE)
           {
           kill_job(pjob, SIGKILL, id, "connection to server lost - no obit sent");
           }
@@ -668,8 +679,8 @@ void scan_for_exiting(void)
         continue;
         }
 
-      if ((pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) &&
-          pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long)
+      if ((pjob->ji_wattr[JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) &&
+          pjob->ji_wattr[JOB_ATR_interactive].at_val.at_long)
         {
 
         if (run_pelog(PE_EPILOGUSER, path_epiloguserp, pjob, PE_IO_TYPE_NULL) != 0)
@@ -695,7 +706,6 @@ void scan_for_exiting(void)
           log_err(-1, id, "parallel epilog failed");
           }
         }
-
       /*
       ** No tasks running ... format and send a
       ** reply to the mother superior and get rid of
@@ -734,7 +744,8 @@ void scan_for_exiting(void)
       job_purge(pjob);
 
       continue;
-      }  /* END if ((pjob->ji_qs.ji_svrflags & JOB_SVFLG_HERE) == 0) */
+	    }
+#endif /* NUMA_SUPPORT */
 
     /*
      * At this point, we know we are Mother Superior for this
@@ -743,7 +754,7 @@ void scan_for_exiting(void)
 
     pjob->ji_qs.ji_svrflags &= ~JOB_SVFLG_Suspend;
 
-    if(pjob->ji_qs.ji_substate != JOB_SUBSTATE_NOTERM_REQUE)
+    if (pjob->ji_qs.ji_substate != JOB_SUBSTATE_NOTERM_REQUE)
       kill_job(pjob, SIGKILL, id, "local task termination detected");
     else
       {
@@ -762,10 +773,7 @@ void scan_for_exiting(void)
               "kill_job found a task to kill");
             }
 
-          if (pjob->ji_qs.ji_un.ji_momt.ji_exitstat != 0)
-            ptask->ti_qs.ti_exitstat = pjob->ji_qs.ji_un.ji_momt.ji_exitstat;
-          else
-            ptask->ti_qs.ti_exitstat = 0;  /* assume successful completion */
+          ptask->ti_qs.ti_exitstat = 0;  /* assume successful completion */
           ptask->ti_qs.ti_status   = TI_STATE_EXITED;
 
           task_save(ptask);
@@ -974,6 +982,11 @@ int post_epilogue(
     return(1);
     }
 
+  strcpy(preq->rq_ind.rq_jobobit.rq_jid, pjob->ji_qs.ji_jobid);
+
+  preq->rq_ind.rq_jobobit.rq_status =
+    pjob->ji_qs.ji_un.ji_momt.ji_exitstat;
+
   CLEAR_HEAD(preq->rq_ind.rq_jobobit.rq_attr);
 
   resc_access_perm = ATR_DFLAG_RDACC;
@@ -982,28 +995,6 @@ int post_epilogue(
 
   encode_flagged_attrs(pjob, &preq->rq_ind.rq_jobobit.rq_attr);
 
-  strcpy(preq->rq_ind.rq_jobobit.rq_jid, pjob->ji_qs.ji_jobid);
-
-  if (pjob->ji_job_is_being_rerun)
-    {
-    pjob->ji_qs.ji_un.ji_momt.ji_exitstat = 0;
-    }
-
-  preq->rq_ind.rq_jobobit.rq_status = pjob->ji_qs.ji_un.ji_momt.ji_exitstat;
-
-  if (LOGLEVEL > 5)
-    {
-    sprintf(log_buffer, "job id %s exit status %d",
-                preq->rq_ind.rq_jobobit.rq_jid,
-                preq->rq_ind.rq_jobobit.rq_status);
-    LOG_EVENT(
-       PBSEVENT_DEBUG,
-       PBSEVENT_JOB,
-       id,
-       log_buffer);
-    }
-
-
   DIS_tcp_setup(sock);
 
   if (encode_DIS_ReqHdr(sock, PBS_BATCH_JobObit, pbs_current_user) ||
@@ -1299,8 +1290,8 @@ static void preobit_reply(
       pjob->ji_qs.ji_jobid,
       log_buffer);
 
-    if (!(pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) ||
-        (pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long == 0))
+    if (!(pjob->ji_wattr[JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) ||
+        (pjob->ji_wattr[JOB_ATR_interactive].at_val.at_long == 0))
       {
       int x; /* dummy */
 
@@ -1386,19 +1377,19 @@ static void preobit_reply(
 
   /* check epilog script */
 
-  if ((pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) &&
-      pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long)
+  if ((pjob->ji_wattr[JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) &&
+      pjob->ji_wattr[JOB_ATR_interactive].at_val.at_long)
     {
     /* job is interactive */
 
     presc = find_resc_entry(
-          &pjob->ji_wattr[(int)JOB_ATR_resource],
+          &pjob->ji_wattr[JOB_ATR_resource],
           find_resc_def(svr_resc_def, "epilogue", svr_resc_size));
-    if((presc != NULL))
-      if((presc->rs_value.at_flags & ATR_VFLAG_SET) && (presc->rs_value.at_val.at_str != NULL))
+    if ((presc != NULL))
+      if ((presc->rs_value.at_flags & ATR_VFLAG_SET) && (presc->rs_value.at_val.at_str != NULL))
         {
         path_epiloguserjob = get_local_script_path(pjob, presc->rs_value.at_val.at_str);
-        if(path_epiloguserjob)
+        if (path_epiloguserjob)
           {
           if (run_pelog(PE_EPILOGUSERJOB, path_epiloguserjob, pjob, PE_IO_TYPE_NULL) != 0)
             {
@@ -1426,14 +1417,14 @@ static void preobit_reply(
     int rc;
 
     presc = find_resc_entry(
-          &pjob->ji_wattr[(int)JOB_ATR_resource],
+          &pjob->ji_wattr[JOB_ATR_resource],
           find_resc_def(svr_resc_def, "epilogue", svr_resc_size));
-    if((presc != NULL))
-      if((presc->rs_value.at_flags & ATR_VFLAG_SET) && (presc->rs_value.at_val.at_str != NULL))
+    if ((presc != NULL))
+      if ((presc->rs_value.at_flags & ATR_VFLAG_SET) && (presc->rs_value.at_val.at_str != NULL))
         {
         path_epiloguserjob = get_local_script_path(pjob, presc->rs_value.at_val.at_str);
 
-        if(path_epiloguserjob)
+        if (path_epiloguserjob)
           {
           if (run_pelog(PE_EPILOGUSERJOB, path_epiloguserjob, pjob, PE_IO_TYPE_STD) != 0)
             {
@@ -1490,6 +1481,7 @@ static void obit_reply(
   job   *nxjob;
   job   *pjob;
   attribute  *pattr;
+  unsigned int momport = 0;
 
   struct batch_request *preq;
   int    x; /* dummy */
@@ -1541,7 +1533,12 @@ static void obit_reply(
 
           pjob->ji_qs.ji_substate = JOB_SUBSTATE_EXITED;
 
-          job_save(pjob, SAVEJOB_QUICK);
+          if (multi_mom)
+            {
+            momport = pbs_rm_port;
+            }
+
+          job_save(pjob, SAVEJOB_QUICK, momport);
 
           if (LOGLEVEL >= 4)
             {
@@ -1570,7 +1567,12 @@ static void obit_reply(
 
           pjob->ji_qs.ji_substate = JOB_SUBSTATE_EXITED;
 
-          job_save(pjob, SAVEJOB_QUICK);
+          if (multi_mom)
+            {
+            momport = pbs_rm_port;
+            }
+
+          job_save(pjob, SAVEJOB_QUICK, momport);
 
           break;
 
@@ -1578,7 +1580,7 @@ static void obit_reply(
 
           /* all jobs discarded by server, discard job */
 
-          pattr = &pjob->ji_wattr[(int)JOB_ATR_interactive];
+          pattr = &pjob->ji_wattr[JOB_ATR_interactive];
 
           if (((pattr->at_flags & ATR_VFLAG_SET) == 0) ||
               (pattr->at_val.at_long == 0))
@@ -1711,14 +1713,18 @@ void init_abort_jobs(
 
   DIR  *dir;
   int            i;
+  int            rc;
+#ifndef NUMA_SUPPORT
   int            j;
-  int            sisters, rc;
+  int            sisters;
+#endif /* ndef NUMA_SUPPORT */
 
   struct dirent *pdirent;
   job  *pj;
   char  *job_suffix = JOB_FILE_SUFFIX;
   int            job_suf_len = strlen(job_suffix);
   char  *psuffix;
+  unsigned int momport = 0;
 
   if (LOGLEVEL >= 6)
     {
@@ -1774,30 +1780,6 @@ void init_abort_jobs(
 
       continue;
       }
-     
-    /* code moved to here because even when we're canceling jobs, if there is a 
-     * user epilogue we'll attempt to become the user, so if ji_grpcache is 
-     * NULL then we'll get a crash */
-    if (pj->ji_grpcache == NULL)
-      {
-      DBPRT(("init_abort_jobs: setting grpcache for job %s\n",
-        pj->ji_qs.ji_jobid));
-      
-      if (check_pwd(pj) == NULL)
-        {
-        /* somehow a job that was legally executing (had a password entry)
-         * no longer has a password entry?? */
-        snprintf(log_buffer, sizeof(log_buffer),
-          "job %s no longer has valid password entry - deleting",
-          pj->ji_qs.ji_jobid);
-        
-        log_err(-1, id, log_buffer);
-        
-        mom_deljob(pj);
-
-        continue;
-        }
-      }
 
     /* PW:  mpiexec patch - set the globid so mom does not coredump in response to tm_spawn */
 
@@ -1832,6 +1814,7 @@ void init_abort_jobs(
      * Note: this only works after the job_nodes() call above.
      */
 
+#ifndef NUMA_SUPPORT 
     for (j = 0;j < pj->ji_numnodes;j++)
       {
       if (LOGLEVEL >= 6)
@@ -1849,6 +1832,7 @@ void init_abort_jobs(
 
       addclient(pj->ji_hosts[j].hn_host);
       }  /* END for (j) */
+#endif /* ndef NUMA_SUPPORT */
 
     if (LOGLEVEL >= 4)
       {
@@ -1950,6 +1934,7 @@ void init_abort_jobs(
         pj->ji_qs.ji_un.ji_momt.ji_exitstat = JOB_EXEC_INITABT;
         }
 
+#ifndef NUMA_SUPPORT 
       sisters = pj->ji_numnodes - 1;
 
       /*
@@ -1966,13 +1951,19 @@ void init_abort_jobs(
 
         continue;
         }
+#endif /* ndef NUMA_SUPPORT */
 
       /* If mom was initialized with a -r any running processes have already
          been killed. We set substate to JOB_SUBSTATE_NOTERM_REQUE so scan_for_exiting
          will not try to kill the running processes for this job */
       pj->ji_qs.ji_substate = JOB_SUBSTATE_NOTERM_REQUE;
 
-      job_save(pj, SAVEJOB_QUICK);
+      if (multi_mom)
+        {
+        momport = pbs_rm_port;
+        }
+
+      job_save(pj, SAVEJOB_QUICK, momport);
 
       exiting_tasks = 1;
       }  /* END if ((recover != 2) && ...) */
@@ -2001,18 +1992,33 @@ void init_abort_jobs(
           log_buffer);
         }
 
+#ifndef NUMA_SUPPORT 
       sisters = pj->ji_numnodes - 1;
 
       if (sisters > 0)
         pj->ji_resources = (noderes *)calloc(sisters, sizeof(noderes));
+#endif /* ndef NUMA_SUPPORT */
 
       if (mom_do_poll(pj) && (recover == JOB_RECOV_RUNNING))
         append_link(&mom_polljobs, &pj->ji_jobque, pj);
+
+      if (pj->ji_grpcache == NULL)
+        {
+        DBPRT(("init_abort_jobs: setting grpcache for job %s\n",
+               pj->ji_qs.ji_jobid));
+        check_pwd(pj);
+        }
       }
     }    /* while ((pdirent = readdir(dir)) != NULL) */
 
   closedir(dir);
 
+#if defined(PENABLE_LINUX26_CPUSETS)
+  /* Create the top level torque cpuset if it doesn't already exist. */
+  initialize_root_cpuset();
+
+#endif
+
   return;
   }  /* END init_abort_jobs() */
 
