@@ -232,6 +232,7 @@
 #include "server_limits.h"
 #include "pbs_job.h"
 #include "utils.h"
+#include "u_tree.h"
 
 #include        "mcom.h"
 
@@ -241,6 +242,10 @@
 #define MAX_RETRY_TIME_IN_SECS  (5 * 60)
 #define STARTING_RETRY_INTERVAL_IN_SECS  2
 
+#ifdef NUMA_SUPPORT
+extern int numa_index;
+extern int num_numa_nodes;
+#endif /* NUMA_SUPPORT */
 
 
 typedef struct mom_server
@@ -287,7 +292,10 @@ extern int             alarm_time; /* time before alarm */
 extern int             rm_errno;
 extern time_t          time_now;
 extern int             verbositylevel;
-extern tree           *okclients;  /* accept connections from */
+/* extern tree           *okclients; */  /* accept connections from */
+extern AvlTree         okclients;
+extern tlist_head      svr_alljobs;
+extern tlist_head      mom_polljobs;
 
 extern struct config *rm_search(struct config *where, char *what);
 
@@ -588,7 +596,12 @@ int mom_server_add(
       ipaddr = ntohl(saddr.s_addr);
 
       if (ipaddr != 0)
-        tinsert(ipaddr, NULL, &okclients);
+        {
+  /*        tinsert(ipaddr, NULL, &okclients); */
+        okclients = AVL_insert(ipaddr, 0, NULL, okclients);
+        
+        }
+
       }
     }    /* END BLOCK */
 
@@ -1127,6 +1140,14 @@ void generate_server_status(
   char *BPtr = buffer;
   int   BSpace = buffer_size;
 
+  /* identify which vnode this is */
+#ifdef NUMA_SUPPORT
+  MUSNPrintF(&BPtr,&BSpace,"%s%d",NUMA_KEYWORD,numa_index);
+  /* advance the buffer values past the NULL */
+  BPtr++;
+  BSpace--;
+#endif /* NUMA_SUPPORT */
+
   for (i = 0;stats[i].name != NULL;i++)
     {
     alarm(alarm_time);
@@ -1166,6 +1187,7 @@ void mom_server_update_stat(
   {
   static char *id = "mom_server_update_stat";
   char *cp;
+  int ret;
 
   if (pms->pbs_servername[0] == 0)
     {
@@ -1193,6 +1215,18 @@ void mom_server_update_stat(
     return;
     }
 
+  ret = diswus(pms->SStream, pbs_mom_port);
+  if (ret)
+    {
+    return;
+    }
+  
+  ret = diswus(pms->SStream, pbs_rm_port);
+  if (ret)
+    {
+    return;
+    }
+  
   /* For each string, put it into the message. */
 
   for (cp = status_strings;cp && *cp;cp += strlen(cp) + 1)
@@ -1257,7 +1291,11 @@ void mom_server_all_update_stat(void)
   {
   static char *id = "mom_server_all_update_stat";
 
+#ifndef NUMA_SUPPORT
   static char status_strings[16 * 1024];  /* Big but smaller than before in is_update_stat */
+#else
+  static char status_strings[MAXLINE << 7];
+#endif
   int sindex;
 
   /* We generate the status once, because this might be costly.
@@ -1270,13 +1308,18 @@ void mom_server_all_update_stat(void)
     log_record(PBSEVENT_SYSTEM, 0, id, "composing status update for server");
     }
 
-  memset(status_strings, 0, sizeof(status_strings));
-
-  generate_server_status(status_strings, sizeof(status_strings));
-
-  for (sindex = 0;sindex < PBS_MAXSERVER;sindex++)
+#ifdef NUMA_SUPPORT
+  for (numa_index = 0; numa_index < num_numa_nodes; numa_index++)
+#endif /* NUMA_SUPPORT */
     {
-    mom_server_update_stat(&mom_servers[sindex],status_strings);
+    memset(status_strings, 0, sizeof(status_strings));
+
+    generate_server_status(status_strings, sizeof(status_strings));
+
+    for (sindex = 0;sindex < PBS_MAXSERVER;sindex++)
+      {
+      mom_server_update_stat(&mom_servers[sindex],status_strings);
+      }
     }
 
   return;
@@ -1351,12 +1394,39 @@ int mom_server_send_hello(
 
   {
   static char id[] = "mom_server_send_hello";
+  int ret;
+
+  if (LOGLEVEL >= 6)
+    {
+    sprintf(log_buffer, "%s",
+            id);
+
+    log_record(
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_SERVER,
+      id,
+      log_buffer);
+    }
+
+
 
   if (is_compose(pms, IS_HELLO) == -1)
     {
     return(-1);
     }
 
+  ret = diswus(pms->SStream, pbs_mom_port);
+  if (ret)
+    {
+    return(-1);
+    }
+  
+  ret = diswus(pms->SStream, pbs_rm_port);
+  if (ret)
+    {
+    return(-1);
+    }
+
   if (mom_server_flush_io(pms, id, "flush") != DIS_SUCCESS)
     {
     return(-1);
@@ -1364,6 +1434,18 @@ int mom_server_send_hello(
 
   pms->sent_hello_count++;
 
+  if (LOGLEVEL >= 6)
+    {
+    sprintf(log_buffer, "%s done. Sent count = %d",
+            id, pms->sent_hello_count);
+
+    log_record(
+      PBSEVENT_JOB,
+      PBS_EVENTCLASS_SERVER,
+      id,
+      log_buffer);
+    }
+
   return(0);
   }  /* END mom_server_send_hello() */
 
@@ -1397,17 +1479,19 @@ int mom_server_check_connection(
     return(0);
     }
 
-  if ((pms->SStream != -1) && 
-      (time_now >= (pms->MOMLastSendToServerTime + (ServerStatUpdateInterval*2))))
+  if (pms->SStream != -1) 
     {
-    sprintf(log_buffer,"connection to server %s timeout", 
-      pms->pbs_servername);
+    if (time_now >= (pms->MOMLastSendToServerTime + (ServerStatUpdateInterval*2)))
+      {
+      sprintf(log_buffer,"connection to server %s timeout", 
+        pms->pbs_servername);
 
-    log_record(PBSEVENT_SYSTEM,0,id,log_buffer);
+      log_record(PBSEVENT_SYSTEM,0,id,log_buffer);
 
-    rpp_close(pms->SStream);
+      rpp_close(pms->SStream);
 
-    pms->SStream = -1;
+      pms->SStream = -1;
+      }
     }
 
   if (pms->SStream == -1)
@@ -1450,6 +1534,7 @@ int mom_server_check_connection(
         }
 
       }
+
     }    /* END if (pms->SStream == -1) */
 
   return(pms->received_cluster_address_count);
@@ -1711,7 +1796,8 @@ void mom_server_update_receive_time_by_ip(
 ** Modified by Tom Proett <proett@nas.nasa.gov> for PBS.
 */
 
-tree *okclients = NULL; /* tree of ip addrs */
+/*tree *okclients = NULL;*/ /* tree of ip addrs */
+AvlTree okclients = NULL;
 
 
 
@@ -1826,7 +1912,9 @@ mom_server *mom_server_valid_message_source(
 
             if (ipaddr == server_ip)
               {
-              tinsert(ipaddr, NULL, &okclients);
+/*              tinsert(ipaddr, NULL, &okclients); */
+              okclients = AVL_insert(ipaddr, 0, NULL, okclients);
+
               pms->SStream = stream;
               return(pms);
               }
@@ -1968,6 +2056,10 @@ void is_request(
         break;
         }
 
+      diswus(pms->SStream, pbs_mom_port);
+      
+      diswus(pms->SStream, pbs_rm_port);
+
       if (mom_server_flush_io(pms, id, "flush") != DIS_SUCCESS)
         break;
 
@@ -1987,7 +2079,8 @@ void is_request(
         if (ret != DIS_SUCCESS)
           break;
 
-        tinsert(ipaddr, NULL, &okclients);
+/*        tinsert(ipaddr, NULL, &okclients); */
+        okclients = AVL_insert(ipaddr, 0, NULL, okclients);
 
         if (LOGLEVEL >= 4)
           {
@@ -2286,8 +2379,6 @@ void check_state(
 
   static char tmpPBSNodeMsgBuf[1024];
 
-  char *id = "check_state";
-
   if (Force)
     {
     ICount = 0;
@@ -2334,7 +2425,8 @@ void check_state(
 
     if (ICount == 0)
       {
-      /* only clear when we're checking again */
+      /* only do this when running the check script, otherwise down nodes are 
+       * marked as up */
       /* clear node state and node messages */
 
       internal_state &= ~INUSE_DOWN;
@@ -2380,21 +2472,7 @@ void check_state(
                 proper action may be context sensitive */
 
       if (IsError == 1)
-        {
         internal_state |= INUSE_DOWN;
-
-        snprintf(log_buffer,sizeof(log_buffer),
-          "Setting node to down. The node health script output the following message:\n%s\n",
-          tmpPBSNodeMsgBuf);
-        log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_NODE,id,log_buffer);
-        }
-      else
-        {
-        snprintf(log_buffer,sizeof(log_buffer),
-          "Node health script ran and says the node is healthy with this message:\n%s\n",
-          tmpPBSNodeMsgBuf);
-        log_event(PBSEVENT_SYSTEM,PBS_EVENTCLASS_NODE,id,log_buffer);
-        }
       }
     }      /* END if (PBSNodeCheckPath[0] != '\0') */
 
@@ -2424,6 +2502,7 @@ void state_to_server(
   {
   static char id[] = "state_to_server";
   mom_server *pms = &mom_servers[ServerIndex];
+  int ret;
 
   if ((force == 0) && (pms->ReportMomState == 0))
     {
@@ -2435,6 +2514,18 @@ void state_to_server(
     return;
     }
 
+  ret = diswus(pms->SStream, pbs_mom_port);
+  if (ret)
+    {
+    return;
+    }
+  
+  ret = diswus(pms->SStream, pbs_rm_port);
+  if (ret)
+    {
+    return;
+    }
+
   if (diswui(pms->SStream, internal_state) != DIS_SUCCESS)
     {
     mom_server_stream_error(pms, id, "writing internal state");
@@ -2538,7 +2629,7 @@ int mom_open_socket_to_jobs_server(
 
   /* See if the server address string has a ':' implying a port number. */
 
-  serverAddr = pjob->ji_wattr[(int)JOB_ATR_at_server].at_val.at_str;
+  serverAddr = pjob->ji_wattr[JOB_ATR_at_server].at_val.at_str;
   if (serverAddr != NULL)
     {
     svrport = strchr(serverAddr, (int)':');
@@ -2579,10 +2670,6 @@ int mom_open_socket_to_jobs_server(
       if (pms->SStream != -1)
         {
         addr = rpp_getaddr(pms->SStream);
-        if (addr == NULL)
-        {
-          continue;
-        }
         ipaddr = ntohl(addr->sin_addr.s_addr);
         ipport = ntohs(addr->sin_port);
 
