@@ -164,23 +164,23 @@ static char *acct_job(
 	 	 PBS_ACCT_MAX_RCD + 1 in size. We will make the assumption that the following
 	 	 strncat calls have ample buffer space to complete successfully */
   sprintf(ptr, "user=%s ",
-          pjob->ji_wattr[(int)JOB_ATR_euser].at_val.at_str);
+          pjob->ji_wattr[JOB_ATR_euser].at_val.at_str);
 
   ptr += strlen(ptr);
 
   /* group */
 
   sprintf(ptr, "group=%s ",
-          pjob->ji_wattr[(int)JOB_ATR_egroup].at_val.at_str);
+          pjob->ji_wattr[JOB_ATR_egroup].at_val.at_str);
 
   ptr += strlen(ptr);
 
   /* account */
 
-  if (pjob->ji_wattr[(int)JOB_ATR_account].at_flags & ATR_VFLAG_SET)
+  if (pjob->ji_wattr[JOB_ATR_account].at_flags & ATR_VFLAG_SET)
     {
     sprintf(ptr, "account=%s ",
-            pjob->ji_wattr[(int)JOB_ATR_account].at_val.at_str);
+            pjob->ji_wattr[JOB_ATR_account].at_val.at_str);
 
     ptr += strlen(ptr);
     }
@@ -188,7 +188,7 @@ static char *acct_job(
   /* job name */
 
   sprintf(ptr, "jobname=%s ",
-          pjob->ji_wattr[(int)JOB_ATR_jobname].at_val.at_str);
+          pjob->ji_wattr[JOB_ATR_jobname].at_val.at_str);
 
   ptr += strlen(ptr);
 
@@ -202,21 +202,21 @@ static char *acct_job(
   /* create time */
 
   sprintf(ptr, "ctime=%ld ",
-          pjob->ji_wattr[(int)JOB_ATR_ctime].at_val.at_long);
+          pjob->ji_wattr[JOB_ATR_ctime].at_val.at_long);
 
   ptr += strlen(ptr);
 
   /* queued time */
 
   sprintf(ptr, "qtime=%ld ",
-          pjob->ji_wattr[(int)JOB_ATR_qtime].at_val.at_long);
+          pjob->ji_wattr[JOB_ATR_qtime].at_val.at_long);
 
   ptr += strlen(ptr);
 
   /* eligible time, how long ready to run */
 
   sprintf(ptr, "etime=%ld ",
-          pjob->ji_wattr[(int)JOB_ATR_etime].at_val.at_long);
+          pjob->ji_wattr[JOB_ATR_etime].at_val.at_long);
 
   ptr += strlen(ptr);
 
@@ -230,7 +230,7 @@ static char *acct_job(
   /* user */
 
   sprintf(ptr, "owner=%s ",
-          pjob->ji_wattr[(int)JOB_ATR_job_owner].at_val.at_str);
+          pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str);
 
 
  
@@ -239,21 +239,21 @@ static char *acct_job(
 
   runningBufSize -= strlen(*Buf);
 
-  newStringLen = strlen("exec_host=")+strlen(pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
-  if(runningBufSize <= newStringLen+1)
-  {
+  newStringLen = strlen("exec_host=")+strlen(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
+  if (runningBufSize <= newStringLen+1)
+    {
     Len = AdjustAcctBufSize(Buf, BufSize, newStringLen, pjob);
-    if(Len == 0)
+    if (Len == 0)
       return(ptr);
     runningBufSize += newStringLen+EXTRA_PAD;
-  }
+    }
 
   ptr = *Buf;
   ptr += strlen(*Buf);
 
   /* execution host name */
   snprintf(ptr, runningBufSize, "exec_host=%s ",
-           pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
+           pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
 
   Len = strlen(ptr);
 
@@ -277,10 +277,10 @@ static char *acct_job(
 
   resc_access_perm = READ_ONLY;
 
-  job_attr_def[(int)JOB_ATR_resource].at_encode(
-    &pjob->ji_wattr[(int)JOB_ATR_resource],
+  job_attr_def[JOB_ATR_resource].at_encode(
+    &pjob->ji_wattr[JOB_ATR_resource],
     &attrlist,
-    job_attr_def[(int)JOB_ATR_resource].at_name,
+    job_attr_def[JOB_ATR_resource].at_name,
     NULL,
     ATR_ENCODE_CLIENT);
 
@@ -290,10 +290,10 @@ static char *acct_job(
 		 	 we still have enough to make these small strncpy calls */
 
 		newStringLen = strlen(pal->al_name);
-		if(runningBufSize <= newStringLen+1)
+		if (runningBufSize <= newStringLen+1)
 			{
 			Len = AdjustAcctBufSize(Buf, BufSize, newStringLen, pjob);
-			if(Len == 0)
+			if (Len == 0)
 				return(ptr);
 
 			runningBufSize += newStringLen+EXTRA_PAD;
@@ -305,10 +305,10 @@ static char *acct_job(
 
     if (pal->al_resc != NULL)
       {
-			if(runningBufSize <= (int)(strlen(pal->al_resc)+2)) /*one for 0 and one for the '.' for 2 */
+			if (runningBufSize <= (int)(strlen(pal->al_resc)+2)) /*one for 0 and one for the '.' for 2 */
 				{
 				Len = AdjustAcctBufSize(Buf, BufSize, newStringLen, pjob);
-				if(Len == 0)
+				if (Len == 0)
 					return(ptr);
 
 				runningBufSize += newStringLen+EXTRA_PAD;
@@ -319,10 +319,10 @@ static char *acct_job(
 			ptr += strlen(ptr);
       }
 
-		if(runningBufSize <= 2)
+		if (runningBufSize <= 2)
 			{
 			Len = AdjustAcctBufSize(Buf, BufSize, newStringLen, pjob);
-			if(Len == 0)
+			if (Len == 0)
 				return(ptr);
 
 			runningBufSize += newStringLen+EXTRA_PAD;
@@ -332,14 +332,14 @@ static char *acct_job(
 
     runningBufSize -= strlen(ptr);
     newStringLen = strlen(pal->al_value);
-    if(runningBufSize <= newStringLen+1)
-    {
+    if (runningBufSize <= newStringLen+1)
+      {
       Len = AdjustAcctBufSize(Buf, BufSize, newStringLen, pjob);
-      if(Len == 0)
+      if (Len == 0)
         return(ptr);
 
       runningBufSize += newStringLen+EXTRA_PAD;
-    }
+      }
 
     strncat(ptr, pal->al_value, runningBufSize);
     strncat(ptr, " ", runningBufSize);
@@ -371,10 +371,10 @@ static char *acct_job(
 
   /* x attributes */
 
-  if (pjob->ji_wattr[(int)JOB_SITE_ATR_x].at_flags & ATR_VFLAG_SET)
+  if (pjob->ji_wattr[JOB_SITE_ATR_x].at_flags & ATR_VFLAG_SET)
     {
     sprintf(ptr, "x=%s ",
-            pjob->ji_wattr[(int)JOB_SITE_ATR_x].at_val.at_str);
+            pjob->ji_wattr[JOB_SITE_ATR_x].at_val.at_str);
 
     ptr += strlen(ptr);
     }
@@ -567,7 +567,7 @@ void account_jobstr(
   /* pack in general information about the job */
 
   buf = (char *)malloc(bufSize+1);
-  if(!buf)
+  if (!buf)
     return;
   acct_job(pjob, &buf, &bufSize);
 
@@ -607,7 +607,7 @@ void account_jobend(
   /* pack in general information about the job */
 
   buf = (char *)malloc(bufSize);
-  if(!buf)
+  if (!buf)
     return;
 
   pb = acct_job(pjob, &buf, &bufSize);
@@ -615,16 +615,16 @@ void account_jobend(
   /* session */
 
   sprintf(pb, "session=%ld ",
-          pjob->ji_wattr[(int)JOB_ATR_session_id].at_val.at_long);
+          pjob->ji_wattr[JOB_ATR_session_id].at_val.at_long);
 
   pb += strlen(pb);
 
   /* Alternate id if present */
 
-  if (pjob->ji_wattr[(int)JOB_ATR_altid].at_flags & ATR_VFLAG_SET)
+  if (pjob->ji_wattr[JOB_ATR_altid].at_flags & ATR_VFLAG_SET)
     {
     sprintf(pb, "alt_id=%s ",
-            pjob->ji_wattr[(int)JOB_ATR_altid].at_val.at_str);
+            pjob->ji_wattr[JOB_ATR_altid].at_val.at_str);
 
     pb += strlen(pb);
     }
@@ -632,7 +632,7 @@ void account_jobend(
   /* add the execution end time */
 
 #ifdef USESAVEDRESOURCES
-  pattr = &pjob->ji_wattr[(int)JOB_ATR_resc_used];
+  pattr = &pjob->ji_wattr[JOB_ATR_resc_used];
 
   if (pattr->at_flags & ATR_VFLAG_SET)
     {
@@ -703,7 +703,7 @@ void acct_cleanup(
 	 char *newBuf;
 
    newBuf = (char *)realloc(*Buf, *BufSize+newStringLen+EXTRA_PAD); /* add 1000 so we don't have to realloc for the small strings */
-   if(newBuf == NULL)
+   if (newBuf == NULL)
      {
      char tmpLine[1024];
 
