@@ -135,7 +135,7 @@
 #include "net_connect.h"
 #include "portability.h"
 #include "array.h"
-#include "dynamic_string.h"
+#include "pbs_job.h"
 
 
 #ifndef TRUE
@@ -151,9 +151,11 @@ void job_purge(job *);
 /* External functions */
 
 extern void cleanup_restart_file(job *);
+extern struct batch_request *setup_cpyfiles(struct batch_request *,job *,char*,char *,int,int);
 extern int job_log_open(char *, char *);
-extern int log_job_record(char *buf);
+extern void log_job_record(char *buf);
 extern void check_job_log(struct work_task *ptask);
+int attr_to_str(char *out, int size, attribute_def *at_def, struct attribute  attr, int  XML);
 
 /* Local Private Functions */
 
@@ -194,14 +196,14 @@ void send_qsub_delmsg(
   attribute *pattri;
   int        qsub_sock;
 
-  phost = arst_string("PBS_O_HOST", &pjob->ji_wattr[(int)JOB_ATR_variables]);
+  phost = arst_string("PBS_O_HOST", &pjob->ji_wattr[JOB_ATR_variables]);
 
   if ((phost == NULL) || ((phost = strchr(phost, '=')) == NULL))
     {
     return;
     }
 
-  pattri = &pjob->ji_wattr[(int)JOB_ATR_interactive];
+  pattri = &pjob->ji_wattr[JOB_ATR_interactive];
 
   qsub_sock = conn_qsub(phost + 1, pattri->at_val.at_long, NULL);
 
@@ -426,8 +428,8 @@ int job_abt(
     svr_mailowner(pjob, MAIL_ABORT, MAIL_NORMAL, text);
 
     if ((pjob->ji_qs.ji_state == JOB_STATE_QUEUED) &&
-        ((pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) &&
-         pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long))
+        ((pjob->ji_wattr[JOB_ATR_interactive].at_flags & ATR_VFLAG_SET) &&
+         pjob->ji_wattr[JOB_ATR_interactive].at_val.at_long))
       {
       /* interactive and not yet running... send a note to qsub */
 
@@ -450,12 +452,12 @@ int job_abt(
         {
         /* notify creator that job is exited */
 
-        pjob->ji_wattr[(int)JOB_ATR_state].at_val.at_char = 'E';
+        pjob->ji_wattr[JOB_ATR_state].at_val.at_char = 'E';
 
         issue_track(pjob);
         }
 
-      if (pjob->ji_wattr[(int)JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
+      if (pjob->ji_wattr[JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
         {
         depend_on_term(pjob);
         }
@@ -494,7 +496,7 @@ int job_abt(
       issue_track(pjob);
       }
 
-    if (pjob->ji_wattr[(int)JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
+    if (pjob->ji_wattr[JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
       {
       depend_on_term(pjob);
       }
@@ -661,7 +663,7 @@ void job_free(
 
   /* remove any malloc working attribute space */
 
-  for (i = 0;i < (int)JOB_ATR_LAST;i++)
+  for (i = 0;i < JOB_ATR_LAST;i++)
     {
     job_attr_def[i].at_free(&pj->ji_wattr[i]);
     }
@@ -1031,7 +1033,7 @@ void job_clone_wt(
         job_purge(pjobclone);
         }
 
-      if (job_save(pjobclone, SAVEJOB_FULL) != 0)
+     if (job_save(pjobclone, SAVEJOB_FULL, 0) != 0)
         {
         /* XXX need more robust error handling */
         job_purge(pjobclone);
@@ -1107,7 +1109,7 @@ void job_clone_wt(
 
       svr_setjobstate(pjob, newstate, newsub);
 
-      job_save(pjob, SAVEJOB_FULL);
+      job_save(pjob, SAVEJOB_FULL, 0);
       }
     }
 
@@ -1129,7 +1131,7 @@ static void job_init_wattr(
   {
   int i;
 
-  for (i = 0;i < (int)JOB_ATR_LAST;i++)
+  for (i = 0;i < JOB_ATR_LAST;i++)
     {
     clear_attr(&pj->ji_wattr[i], &job_attr_def[i]);
     }
@@ -1157,22 +1159,22 @@ struct batch_request *cpy_checkpoint(
   char       *to = NULL;
   attribute  *pattr;
   mode_t     saveumask = 0;
-
-  pattr = &pjob->ji_wattr[(int)ati];
+  
+  pattr = &pjob->ji_wattr[ati];
 
   if ((pattr->at_flags & ATR_VFLAG_SET) == 0)
     {
     /* no file to transfer */
-
+    
     return(preq);
     }
-
+    
   /* build up the name used for SERVER file */
 
   strcpy(serverfile, path_checkpoint);
   strcat(serverfile, pjob->ji_qs.ji_fileprefix);
   strcat(serverfile, JOB_CHECKPOINT_SUFFIX);
-
+  
   /*
    * We need to make sure the jobs checkpoint directory exists.  If it does
    * not we need to add it since this is the first time we are copying a
@@ -1187,13 +1189,13 @@ struct batch_request *cpy_checkpoint(
   umask(saveumask);
 
   strcat(serverfile, "/");
-  strcat(serverfile, pjob->ji_wattr[(int)JOB_ATR_checkpoint_name].at_val.at_str);
+  strcat(serverfile, pjob->ji_wattr[JOB_ATR_checkpoint_name].at_val.at_str);
 
   /* build up the name used for MOM file */
 
-  if (pjob->ji_wattr[(int)JOB_ATR_checkpoint_dir].at_flags & ATR_VFLAG_SET)
+  if (pjob->ji_wattr[JOB_ATR_checkpoint_dir].at_flags & ATR_VFLAG_SET)
     {
-    strcpy(momfile, pjob->ji_wattr[(int)JOB_ATR_checkpoint_dir].at_val.at_str);
+    strcpy(momfile, pjob->ji_wattr[JOB_ATR_checkpoint_dir].at_val.at_str);
     strcat(momfile, "/");
     strcat(momfile, pjob->ji_qs.ji_fileprefix);
     strcat(momfile, JOB_CHECKPOINT_SUFFIX);
@@ -1208,7 +1210,7 @@ struct batch_request *cpy_checkpoint(
     strcat(momfile, pjob->ji_qs.ji_fileprefix);
     strcat(momfile, JOB_CHECKPOINT_SUFFIX);
     strcat(momfile, "/");
-    strcat(momfile, pjob->ji_wattr[(int)JOB_ATR_checkpoint_name].at_val.at_str);
+    strcat(momfile, pjob->ji_wattr[JOB_ATR_checkpoint_name].at_val.at_str);
     if (LOGLEVEL >= 7)
       {
       sprintf(log_buffer, "Job has NO checkpoint dir specified, using file %s",
@@ -1326,8 +1328,8 @@ void remove_checkpoint(
     /* have files to delete  */
 
     sprintf(log_buffer,"Removing checkpoint file (%s/%s)",
-      pjob->ji_wattr[(int)JOB_ATR_checkpoint_dir].at_val.at_str,
-      pjob->ji_wattr[(int)JOB_ATR_checkpoint_name].at_val.at_str);
+      pjob->ji_wattr[JOB_ATR_checkpoint_dir].at_val.at_str,
+      pjob->ji_wattr[JOB_ATR_checkpoint_name].at_val.at_str);
     log_ext(-1, id, log_buffer, LOG_DEBUG);
 
     /* change the request type from copy to delete  */
@@ -1337,7 +1339,7 @@ void remove_checkpoint(
     preq->rq_extra = NULL;
 
     if (relay_to_mom(
-          pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
+          pjob,
           preq,
           release_req) == 0)
       {
@@ -1382,139 +1384,105 @@ void cleanup_restart_file(
     pjob->ji_wattr[(int)JOB_ATR_restart_name].at_flags &= ~ATR_VFLAG_SET;
     pjob->ji_modified = 1;
 
-    job_save(pjob, SAVEJOB_FULL);
+    job_save(pjob, SAVEJOB_FULL, 0);
 
   return;
   }  /* END cleanup_restart_file() */
 
 
-int record_jobinfo(
-    
-  job *pjob)
-
+int record_jobinfo(job *pjob)
   {
-  static char     id[] = "record_jobinfo";
-  attribute      *pattr;
-  int             i;
-  int             rc;
-  dynamic_string *buffer;
-  char            job_script_buf[MAXPATHLEN << 4];
-  char            namebuf[MAXPATHLEN + 1];
-  int             fd;
-  size_t          bytes_read = 0;
+    attribute    *pattr;
+    int i;
+    int rc;
+    char buf[MAXLINE << 3];
+    char valbuf[MAXLINE << 2];
+    char namebuf[MAXPATHLEN + 1];
+    int fd;
+    size_t bytes_read = 0;
+
+
+    job_log_open(job_log_file, path_jobinfo_log);
+
+    strcpy(buf, "<Jobinfo>\n");
+    sprintf(valbuf, "\t<Job_Id>%s</Job_Id>", pjob->ji_qs.ji_jobid);
+    strcat(buf, valbuf);
+    log_job_record(buf);
   
-  if ((rc = job_log_open(job_log_file, path_jobinfo_log)) < 0)
-    {
-    log_err(rc, id, "Could not open job log ");
-    return(rc);
-    }
- 
-  if ((buffer = get_dynamic_string(MAXLINE << 3, NULL)) == NULL)
-    {
-    log_err(ENOMEM, id, "Can't allocate memory");
-    return(-1);
-    }
- 
-  append_dynamic_string(buffer, "<Jobinfo>\n");
-  append_dynamic_string(buffer, "\t<Job_Id>");
-  append_dynamic_string(buffer, pjob->ji_qs.ji_jobid);
-  append_dynamic_string(buffer, "</JobId>");
-  
-  if ((rc = log_job_record(buffer->str)) != PBSE_NONE)
-    {
-    log_err(rc, id, "log_job_record failed");
-    free_dynamic_string(buffer);
-    return(rc);
-    }
-  
-  for (i = 0; i < JOB_ATR_LAST; i++)
-    {
-    clear_dynamic_string(buffer);
-      
-    pattr = &(pjob->ji_wattr[i]);
-    if (pattr->at_flags & ATR_VFLAG_SET)
+    for(i = 0; i < JOB_ATR_LAST; i++)
       {
-      if (!strcmp(job_attr_def[i].at_name, "depend"))
+      pattr = &(pjob->ji_wattr[i]);
+      if (pattr->at_flags & ATR_VFLAG_SET)
         {
-        /* we don't want this attribute in our log
-           The dependecies will show on the submit_args attribute */
-        continue;
-        }
+        if (!strcmp(job_attr_def[i].at_name, "depend"))
+          {
+          /* we don't want this attribute in our log
+             The dependecies will show on the submit_args
+             element */
+          continue;
+          }
 
-      append_dynamic_string(buffer, "\t<");
-      append_dynamic_string(buffer, job_attr_def[i].at_name);
-      append_dynamic_string(buffer, ">");
-      
-      if (pattr->at_type == ATR_TYPE_RESC)
-        append_dynamic_string(buffer, "\n");
-      
-      rc = attr_to_str(buffer, job_attr_def+i, pjob->ji_wattr[i], 1);
-      
-      if (pattr->at_type == ATR_TYPE_RESC)
-        append_dynamic_string(buffer, "\t");
-
-      append_dynamic_string(buffer, "</");
-      append_dynamic_string(buffer, job_attr_def[i].at_name);
-      append_dynamic_string(buffer, ">");
-
-      if ((rc = log_job_record(buffer->str)) != PBSE_NONE)
-        {
-        log_err(rc, id, "log_job_record failed recording attributes");
-        free_dynamic_string(buffer);
-        return(rc);
+        strcpy(buf, "\t<");
+        strcat(buf, job_attr_def[i].at_name);
+        strcat(buf, ">");
+        if (pattr->at_type == ATR_TYPE_RESC)
+          {
+          strcat(buf, "\n");
+          }
+        rc = attr_to_str(valbuf, sizeof(valbuf), job_attr_def+i, pjob->ji_wattr[i], 1);
+        strcat(buf, valbuf);
+        if (pattr->at_type == ATR_TYPE_RESC)
+          {
+          strcat(buf, "\t");
+          }
+        strcat(buf, "</");
+        strcat(buf, job_attr_def[i].at_name);
+        strcat(buf, ">");
+        log_job_record(buf);
         }
       }
-    } /* END for each attribute */
 
-  if (server.sv_attr[SRV_ATR_RecordJobScript].at_val.at_long)
-    {    
-    /* This is for Baylor. We will make it a server parameter eventually
-       Write the contents of the script to our log file*/
+    if (server.sv_attr[SRV_ATR_RecordJobScript].at_val.at_long)
+      {
+  
+      /* This is for Baylor. We will make it a server parameter eventually
+         Write the contents of the script to our log file*/
+      strcpy(buf, "\t<job_script>");
+      strcpy(namebuf, path_jobs);
+      strcat(namebuf, pjob->ji_qs.ji_fileprefix);
+      strcat(namebuf, JOB_SCRIPT_SUFFIX);
+  
+      fd = open(namebuf, O_RDONLY);
+      if (fd > 0)
+        {
+        do
+          {
+          bytes_read = read(fd, valbuf, sizeof(valbuf));
+          if (bytes_read > 0)
+            {
+            valbuf[bytes_read] = 0; /* null terminate the buffer for strcpy */
+            strcat(buf, valbuf);
+            }
+          } while(bytes_read > 0);
+          close(fd);
+        }
+      else
+        {
+        strcat(buf, "unable to open script file\n");
+        }
+  
+  
+      strcat(buf, "\t</job_script>\n");
+      log_job_record(buf);
+      }
+
+    strcpy(buf, "</Jobinfo>\n");
+    log_job_record(buf);
 
-    append_dynamic_string(buffer, "\t<job_script>");
     
-    strcpy(namebuf, path_jobs);
-    strcat(namebuf, pjob->ji_qs.ji_fileprefix);
-    strcat(namebuf, JOB_SCRIPT_SUFFIX);
     
-    if ((fd = open(namebuf, O_RDONLY)) > 0)
-      {
-      while ((bytes_read = read(fd, job_script_buf, sizeof(job_script_buf))) > 0)
-        {
-        rc = append_dynamic_string(buffer, job_script_buf);
-        }
-
-      close(fd);
-      }
-    else
-      {
-      append_dynamic_string(buffer, "unable to open script file\n");
-      }
-        
-    append_dynamic_string(buffer, "\t</job_script>\n");
-
-    if ((rc = log_job_record(buffer->str)) != PBSE_NONE)
-      {
-      free_dynamic_string(buffer);
-      log_err(rc, id, "log_job_record failed");
-      return(rc);
-      }
-    }
-  
-  clear_dynamic_string(buffer);
-  
-  if ((rc = append_dynamic_string(buffer, "</Jobinfo>\n")) != PBSE_NONE)
-    {
-    log_err(rc, id, "");
-    free_dynamic_string(buffer);
-    return(rc);
-    }
- 
-  rc = log_job_record(buffer->str);
-  free_dynamic_string(buffer);
-
-  return(rc);  
-  } /* END record_jobinfo() */
+    return(0);  
+  }
 
 
 
@@ -1538,12 +1506,12 @@ void job_purge(
   extern char  *msg_err_purgejob;
 
   /* check to see if we are keeping a log of all jobs completed */
-  if(server.sv_attr[(int)SRV_ATR_RecordJobInfo].at_val.at_long)
+  if (server.sv_attr[SRV_ATR_RecordJobInfo].at_val.at_long)
     {
     record_jobinfo(pjob);
 
     /* Start a task to monitor job log roll over if it is not already started */
-    if(check_job_log_started == 0)
+    if (check_job_log_started == 0)
       {
       set_task(WORK_Timed, time_now + 10, check_job_log, NULL);
       check_job_log_started = 1;
@@ -1567,11 +1535,11 @@ void job_purge(
 
     /* if there are no more jobs in the arry,
      * then we can clean that up too */
-    pa->ai_qs.num_purged++;
-    if (pa->ai_qs.num_purged == pa->ai_qs.num_jobs)
+    if (++pa->ai_qs.jobs_done == pa->ai_qs.num_jobs)
+      {
       array_delete(pjob->ji_arraystruct);
-    else
-      array_save(pa);
+      }
+    array_save(pa);
     }
 
   if ((pjob->ji_is_array_template == TRUE) ||
@@ -1645,15 +1613,15 @@ void job_purge(
     }
 
   /* remove checkpoint restart file if there is one */
-
-  if (pjob->ji_wattr[(int)JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
+  
+  if (pjob->ji_wattr[JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
     {
     cleanup_restart_file(pjob);
     }
 
   /* delete checkpoint file directory if there is one */
-
-  if (pjob->ji_wattr[(int)JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET)
+  
+  if (pjob->ji_wattr[JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET)
     {
     strcpy(namebuf, path_checkpoint);
     strcat(namebuf, pjob->ji_qs.ji_fileprefix);
