@@ -165,18 +165,25 @@ static const unsigned int quicksize = sizeof(struct jobfix);
 int job_save(
 
   job *pjob,  /* pointer to job structure */
-  int  updatetype) /* 0=quick, 1=full     */
+  int  updatetype, /* 0=quick, 1=full     */
+  int  mom_port)   /* if 0 ignore otherwise append to end of job name. this is for multi-mom mode */
 
   {
   int fds;
   int i;
   char namebuf1[MAXPATHLEN];
   char namebuf2[MAXPATHLEN];
+  char portname[MAXPATHLEN];
   int openflags;
   int redo;
 
   strcpy(namebuf1, path_jobs); /* job directory path */
   strcat(namebuf1, pjob->ji_qs.ji_fileprefix);
+  if (mom_port)
+    {
+    sprintf(portname, "%d", mom_port);
+    strcat(namebuf1, portname);
+    }
   strcpy(namebuf2, namebuf1); /* setup for later */
 
 #ifdef PBS_MOM
@@ -300,7 +307,7 @@ int job_save(
         {
         redo++;
         }
-      else if (save_attr(job_attr_def, pjob->ji_wattr, (int)JOB_ATR_LAST) != 0)
+      else if (save_attr(job_attr_def, pjob->ji_wattr, JOB_ATR_LAST) != 0)
         {
         redo++;
         }
@@ -499,8 +506,8 @@ job *job_recov(
         pj,
         job_attr_def,
         pj->ji_wattr,
-        (int)JOB_ATR_LAST,
-        (int)JOB_ATR_UNKN,
+        JOB_ATR_LAST,
+        JOB_ATR_UNKN,
         TRUE) != 0) 
     {
     sprintf(log_buffer, "unable to recover %s (file is likely corrupted)",
@@ -528,7 +535,7 @@ job *job_recov(
 
 #else /* PBS_MOM */
 
-  if (pj->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)
+  if (pj->ji_wattr[JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)
     {
     /* job is part of an array.  We need to put a link back to the server
     job array struct for this array. We also have to link this job into
@@ -553,7 +560,7 @@ job *job_recov(
         }
       else
         {
-        pa->jobs[(int)pj->ji_wattr[JOB_ATR_job_array_id].at_val.at_long] = (void *)pj;
+        pa->jobs[pj->ji_wattr[JOB_ATR_job_array_id].at_val.at_long] = (void *)pj;
         pj->ji_arraystruct = pa;
         pa->jobs_recovered++;
         }
@@ -568,7 +575,7 @@ job *job_recov(
 
   if (qs_upgrade == TRUE)
     {
-    job_save(pj, SAVEJOB_FULL);
+    job_save(pj, SAVEJOB_FULL, 0);
     }
 
   return(pj);
