@@ -123,25 +123,13 @@
 #include "pbs_proto.h"
 #include "net_connect.h"
 #include "utils.h"
+#include "u_tree.h"
 
 #if !defined(H_ERRNO_DECLARED) && !defined(_AIX)
 extern int h_errno;
 #endif
 extern time_t  time_now;
 
-#ifndef MAX_LINE
-#define MAX_LINE 1024
-#endif
-
-typedef struct _node_info_
-  {
-  char     *nodename;
-  svrattrl *plist;
-  int      perms;
-  tlist_head atrlist;
-  }node_info;
-
-
 
 /* Global Data */
 
@@ -154,6 +142,8 @@ extern char *path_nodestate;
 extern char *path_nodenote;
 extern int       LOGLEVEL;
 extern attribute_def  node_attr_def[];   /* node attributes defs */
+extern AvlTree ipaddrs;
+extern AvlTree streams;
 
 
 /* Functions in this file
@@ -198,9 +188,9 @@ struct pbsnode *PGetNodeFromAddr(
   int nindex;
   int aindex;
 
-  for (nindex = 0;nindex < svr_totnodes;nindex++)
+  for (nindex = 0; nindex < svr_totnodes; nindex++)
     {
-    if ((pbsndlist[nindex] == NULL) ||
+    if ((pbsndlist[nindex] == NULL) || 
         (pbsndlist[nindex]->nd_state & INUSE_DELETED))
       continue;
 
@@ -227,12 +217,17 @@ void bad_node_warning(
   pbs_net_t addr)  /* I */
 
   {
-  int i;
+  int i = 0;
   time_t now, last;
 
-  for (i = 0;i < svr_totnodes;i++)
+  node_iterator iter;
+  struct pbsnode *pnode;
+
+  reinitialize_node_iterator(&iter);
+
+  while ((pnode = next_node(&iter)) != NULL)
     {
-    if (pbsndlist[i] == NULL || pbsndlist[i]->nd_addrs == NULL)
+    if (pnode->nd_addrs == NULL)
       {
       sprintf(log_buffer, "ALERT:  node table is corrupt at index %d",
               i);
@@ -246,14 +241,14 @@ void bad_node_warning(
       continue;
       }
 
-    if (pbsndlist[i]->nd_state & INUSE_DELETED)
+    if (pnode->nd_state & INUSE_DELETED)
       {
       /* node was deleted */
 
       continue;
       }
 
-    if (pbsndlist[i]->nd_addrs[0] != addr)
+    if (pnode->nd_addrs[0] != addr)
       {
       /* node does not match */
 
@@ -264,7 +259,7 @@ void bad_node_warning(
 
     now = time(0);
 
-    last = pbsndlist[i]->nd_warnbad;
+    last = pnode->nd_warnbad;
 
     if (last && (now - last < 3600))
       {
@@ -279,7 +274,7 @@ void bad_node_warning(
     */
 
     sprintf(log_buffer, "ALERT: unable to contact node %s",
-            pbsndlist[i]->nd_name);
+            pnode->nd_name);
 
     log_event(
       PBSEVENT_ADMIN,
@@ -291,7 +286,9 @@ void bad_node_warning(
      *  (void)set_task(WORK_Timed,now + 5,ping_nodes,NULL);
      */
 
-    pbsndlist[i]->nd_warnbad = now;
+    pnode->nd_warnbad = now;
+
+    i++;
 
     break;
     }    /* END for (i = 0) */
@@ -312,46 +309,55 @@ int addr_ok(
   pbs_net_t addr)  /* I */
 
   {
-  int i;
   int status = 1;  /* assume destination host is healthy */
 
+  node_iterator iter;
+  struct pbsnode *pnode;
+
+  reinitialize_node_iterator(&iter);
+
   if (pbsndlist != NULL)
     {
-    for (i = 0;i < svr_totnodes;i++)
+    while ((pnode = next_node(&iter)) != NULL)
       {
       /* NOTE:  should walk thru all nd_addrs for multi-homed hosts */
 
-      if (pbsndlist[i]->nd_state & INUSE_DELETED)
+      if (pnode->nd_state & INUSE_DELETED)
         continue;
 
       /* NOTE:  deleted node may have already freed nd_addrs -
                 check should be redundant */
 
-      if ((pbsndlist[i]->nd_addrs == NULL) || (pbsndlist[i]->nd_addrs[0] != addr))
+      if ((pnode->nd_addrs == NULL) || (pnode->nd_addrs[0] != addr))
         continue;
 
       /* node matches addr */
 
-      if (pbsndlist[i]->nd_state & (INUSE_DELETED | INUSE_UNKNOWN))
+      if (pnode->nd_state & (INUSE_DELETED | INUSE_UNKNOWN))
         {
         /* definitely not ok */
 
         status = 0;
         }
-      else if (pbsndlist[i]->nd_state & INUSE_DOWN)
+      else if (pnode->nd_state & INUSE_DOWN)
         {
         /* the node is ok if it is still talking to us */
 
         int chk_len;
 
-        chk_len = server.sv_attr[(int)SRV_ATR_check_rate].at_val.at_long;
+        chk_len = server.sv_attr[SRV_ATR_check_rate].at_val.at_long;
 
-        if (pbsndlist[i]->nd_lastupdate <= (time_now - chk_len))
+        if (pnode->nd_lastupdate == 0)
+          {
+          continue;
+          }
+
+        if (pnode->nd_lastupdate <= (time_now - chk_len))
           {
           status = 0;
           }
 
-        if (pbsndlist[i]->nd_stream < 0)
+        if (pnode->nd_stream < 0)
           {
           status = 0;
           }
@@ -376,25 +382,24 @@ struct pbsnode *find_nodebyname(
         char *nodename) /* I */
 
   {
-  int  i;
   char  *pslash;
 
   struct pbsnode *pnode;
 
+  node_iterator iter;
+
   if ((pslash = strchr(nodename, (int)'/')) != NULL)
     *pslash = '\0';
 
-  pnode = NULL;
+  reinitialize_node_iterator(&iter);
 
-  for (i = 0;i < svr_totnodes;i++)
+  while ((pnode = next_node(&iter)) != NULL)
     {
-    if (pbsndlist[i]->nd_state & INUSE_DELETED)
+    if (pnode->nd_state & INUSE_DELETED)
       continue;
 
-    if (strcasecmp(nodename, pbsndlist[i]->nd_name) == 0)
+    if (strcasecmp(nodename, pnode->nd_name) == 0)
       {
-      pnode = pbsndlist[i];
-
       break;
       }
     }
@@ -420,15 +425,17 @@ struct pbssubn *find_subnodebyname(
         char *nodename)
 
   {
-
   struct pbsnode  *pnode;
+  struct pbssubn *tmp;
 
   if ((pnode = find_nodebyname(nodename)) == NULL)
     {
     return(NULL);
     }
 
-  return(pnode->nd_psn);
+  tmp = pnode->nd_psn;
+
+  return(tmp);
   }  /* END find_subnodebyname() */
 
 
@@ -646,8 +653,15 @@ int status_nodeattrib(
       atemp[i].at_val.at_long = pnode->nd_nsn;
     else if (!strcmp((padef + i)->at_name, ATTR_NODE_note))
       atemp[i].at_val.at_str  = pnode->nd_note;
-    else if (!strcmp((padef + i)->at_name, ATTR_NODE_gpustatus))
-      atemp[i].at_val.at_arst = pnode->nd_gpustatus;
+    else if (!strcmp((padef + i)->at_name, ATTR_NODE_mom_port))
+      atemp[i].at_val.at_long  = pnode->nd_mom_port;
+    else if (!strcmp((padef + i)->at_name, ATTR_NODE_mom_rm_port))
+      atemp[i].at_val.at_long  = pnode->nd_mom_rm_port;
+    /* skip NUMA attributes */
+    else if (!strcmp((padef + i)->at_name, ATTR_NODE_num_numa_nodes))
+      continue;
+    else if (!strcmp((padef + i)->at_name, ATTR_NODE_numa_str))
+      continue;
     else if (!strcmp((padef + i)->at_name, ATTR_NODE_gpus))
       {
       atemp[i].at_val.at_long  = pnode->nd_ngpus;
@@ -746,9 +760,6 @@ int status_nodeattrib(
   }  /* END status_nodeattrib() */
 
 
-
-
-
 /*
  * initialize_pbsnode - carries out initialization on a new
  * pbs node.  The assumption is that all the parameters are valid.
@@ -758,48 +769,50 @@ static void initialize_pbsnode(
 
   struct pbsnode *pnode,
   char           *pname, /* node name */
-  u_long         *pul,  /* I - required - host byte order array */
+  u_long         *pul,  /* host byte order array */
   /* ipaddrs for this node */
   int             ntype) /* time-shared or cluster */
 
   {
-  char *id = "initialize_pbsnode";
+/*  char *id = "initialize_pbsnode";*/
 
-  int i;
+/*  int i; */
 
   memset(pnode, 0, sizeof(struct pbsnode));
 
-  pnode->nd_name    = pname;
-  pnode->nd_stream  = -1;
-  pnode->nd_addrs   = pul;       /* list of host byte order */
-  pnode->nd_ntype   = ntype;
-  pnode->nd_nsn     = 0;
-  pnode->nd_nsnfree = 0;
-  pnode->nd_needed  = 0;
-  pnode->nd_order   = 0;
-  pnode->nd_prop    = NULL;
-  pnode->nd_status  = NULL;
-  pnode->nd_note    = NULL;
-  pnode->nd_psn     = NULL;
-  pnode->nd_state   = INUSE_NEEDS_HELLO_PING | INUSE_DOWN;
-  pnode->nd_first   = init_prop(pnode->nd_name);
-  pnode->nd_last    = pnode->nd_first;
-  pnode->nd_f_st    = init_prop(pnode->nd_name);
-  pnode->nd_l_st    = pnode->nd_f_st;
-  pnode->nd_nprops  = 0;
-  pnode->nd_nstatus = 0;
-  pnode->nd_warnbad = 0;
-  pnode->nd_ngpus   = 0;
-  pnode->nd_gpustatus = NULL;
-  pnode->nd_ngpustatus = 0;
+  pnode->nd_name        = pname;
+  pnode->nd_stream      = -1;
+  pnode->nd_mom_port    = PBS_MOM_SERVICE_PORT;
+  pnode->nd_mom_rm_port = PBS_MANAGER_SERVICE_PORT;
+  pnode->nd_addrs       = pul;       /* list of host byte order */
+  pnode->nd_ntype       = ntype;
+  pnode->nd_nsn         = 0;
+  pnode->nd_nsnfree     = 0;
+  pnode->nd_needed      = 0;
+  pnode->nd_order       = 0;
+  pnode->nd_prop        = NULL;
+  pnode->nd_status      = NULL;
+  pnode->nd_note        = NULL;
+  pnode->nd_psn         = NULL;
+  pnode->nd_state       = INUSE_NEEDS_HELLO_PING | INUSE_DOWN;
+  pnode->nd_first       = init_prop(pnode->nd_name);
+  pnode->nd_last        = pnode->nd_first;
+  pnode->nd_f_st        = init_prop(pnode->nd_name);
+  pnode->nd_l_st        = pnode->nd_f_st;
+  pnode->nd_nprops      = 0;
+  pnode->nd_nstatus     = 0;
+  pnode->nd_warnbad     = 0;
 
-  for (i = 0;pul[i];i++)
+  /*for (i = 0;pul[i];i++)
     {
     if (LOGLEVEL >= 6)
       {
-      sprintf(log_buffer, "node '%s' allows trust for ipaddr %s\n",
+      sprintf(log_buffer, "node '%s' allows trust for ipaddr %ld.%ld.%ld.%ld\n",
               pnode->nd_name,
-              netaddr_pbs_net_t(pul[i]));
+              (pul[i] & 0xff000000) >> 24,
+              (pul[i] & 0x00ff0000) >> 16,
+              (pul[i] & 0x0000ff00) >> 8,
+              (pul[i] & 0x000000ff));
 
       log_record(
         PBSEVENT_SCHED,
@@ -809,23 +822,19 @@ static void initialize_pbsnode(
       }
 
     tinsert(pul[i], pnode, &ipaddrs);
-    }  /* END for (i) */
+    }*/  /* END for (i) */
 
   return;
   }  /* END initialize_pbsnode() */
 
 
-
-
 /*
  * subnode_delete - delete the specified subnode
  * by marking it deleted
  */
 
 static void subnode_delete(
-
   struct pbssubn *psubn)
-
   {
 
   struct jobinfo *jip, *jipt;
@@ -847,8 +856,6 @@ static void subnode_delete(
   }
 
 
-
-
 void effective_node_delete(
 
   struct pbsnode *pnode)
@@ -885,8 +892,9 @@ void effective_node_delete(
       {
       /* del node's IP addresses from tree  */
 
-      tdelete(*up, &ipaddrs);
-      }
+       /* tdelete(*up, &ipaddrs);  */
+      ipaddrs = AVL_delete_node( *up, pnode->nd_mom_port, ipaddrs);
+      } 
 
     if (pnode->nd_addrs != NULL)
       {
@@ -898,7 +906,8 @@ void effective_node_delete(
       }
     }
 
-  tdelete((u_long)pnode->nd_stream, &streams); /*take stream out of tree*/
+  /* tdelete((u_long)pnode->nd_stream, &streams); */ /*take stream out of tree*/
+  streams = AVL_delete_node((u_long)pnode->nd_stream, 0, streams);
 
   rpp_close(pnode->nd_stream);
   free(pnode->nd_name);
@@ -913,8 +922,62 @@ void effective_node_delete(
   }  /* END effective_node_delete() */
 
 
+/* free_node() - clean up and free all elements of a pbsnode that has not been
+   entered into any tables. called from create_pbs_node() */
+void free_node(
 
+  struct pbsnode *pnode)
 
+  {
+
+  struct pbssubn  *psubn;
+
+  struct pbssubn  *pnxt;
+
+  psubn = pnode->nd_psn;
+
+  while (psubn != NULL)
+    {
+    pnxt = psubn->next;
+
+    subnode_delete(psubn);
+
+    psubn = pnxt;
+    }
+
+  pnode->nd_last->next = NULL;      /* just in case */
+
+  pnode->nd_last       = NULL;
+
+  free_prop_list(pnode->nd_first);
+
+  pnode->nd_first = NULL;
+
+  if (pnode->nd_addrs != NULL)
+    {
+
+    if (pnode->nd_addrs != NULL)
+      {
+      /* remove array of IP addresses */
+
+      free(pnode->nd_addrs);
+
+      pnode->nd_addrs = NULL;
+      }
+    }
+
+  free(pnode->nd_name);
+
+  pnode->nd_name    = NULL;
+  pnode->nd_stream  = -1;
+  pnode->nd_state   = INUSE_DELETED;
+  pnode->nd_nsn     = 0;
+  pnode->nd_nsnfree = 0;
+
+  /* NYI - free numa nodes if they exist */
+
+  return;
+  }  /* END free_node() */
 
 /**
  *  NOTE:  pul can return NULL even on SUCCESS of routine
@@ -1003,12 +1066,13 @@ static int process_host_name_part(
 
   memcpy((char *)&addr, hp->h_addr, hp->h_length);
 
-  if (hp->h_addr_list[0] != NULL)
+  if (hp->h_addr_list[1] == NULL)
     {
+    /* weren't given canonical name */
 
     char *hname;
 
-/*    if ((hp = gethostbyaddr(
+    if ((hp = gethostbyaddr(
                 (void *) & addr,
                 sizeof(struct in_addr),
                 hp->h_addrtype)) == NULL)
@@ -1025,7 +1089,7 @@ static int process_host_name_part(
       phostname = NULL;
 
       return(PBSE_UNKNODE);
-      }*/
+      }
 
     hname = (char *)strdup(hp->h_name); /* canonical name in theory */
 
@@ -1041,10 +1105,10 @@ static int process_host_name_part(
 
     if (NodeSuffixIsSet == 0)
       {
-      if (((server.sv_attr[(int)SRV_ATR_NodeSuffix].at_flags & ATR_VFLAG_SET) != 0) &&
-          (server.sv_attr[(int)SRV_ATR_NodeSuffix].at_val.at_str != NULL))
+      if (((server.sv_attr[SRV_ATR_NodeSuffix].at_flags & ATR_VFLAG_SET) != 0) &&
+          (server.sv_attr[SRV_ATR_NodeSuffix].at_val.at_str != NULL))
         {
-        NodeSuffix = strdup(server.sv_attr[(int)SRV_ATR_NodeSuffix].at_val.at_str);
+        NodeSuffix = strdup(server.sv_attr[SRV_ATR_NodeSuffix].at_val.at_str);
         }
 
       NodeSuffixIsSet = 1;
@@ -1184,8 +1248,7 @@ static int process_host_name_part(
  *       the nodes file.
 */
 
-int
-update_nodes_file(void)
+int update_nodes_file(void)
 
   {
 #ifndef NDEBUG
@@ -1227,6 +1290,8 @@ update_nodes_file(void)
     }
 
   /* for each node ... */
+  /* NOTE: DO NOT change this loop to iterate over numa nodes. Since they
+   * aren't real hosts they should NOT appear in the nodes file */
 
   for (i = 0;i < svr_totnodes;++i)
     {
@@ -1251,13 +1316,21 @@ update_nodes_file(void)
               ATTR_NODE_np,
               np->nd_nsn);
 
-    /* if number of gpus is gt 0, write that; if none,   */
-    /* don't write to maintain compatability with old style file */
-
-    if (np->nd_ngpus > 0)
+    /* write out the numa attributes if needed */
+    if (np->num_numa_nodes > 0)
+      {
       fprintf(nin, " %s=%d",
-              ATTR_NODE_gpus,
-              np->nd_ngpus);
+        ATTR_NODE_num_numa_nodes,
+        np->num_numa_nodes);
+      }
+
+    if ((np->numa_str != NULL) &&
+        (np->numa_str[0] != '\0'))
+      {
+      fprintf(nin, " %s=%s",
+        ATTR_NODE_numa_str,
+        np->numa_str);
+      }
 
     /* write out properties */
 
@@ -1316,20 +1389,21 @@ recompute_ntype_cnts(void)
   {
   int   svr_loc_clnodes = 0;
   int   svr_loc_tsnodes = 0;
-  int   i;
 
   struct pbsnode  *pnode;
 
+  node_iterator iter;
+
+  reinitialize_node_iterator(&iter);
+
   if (svr_totnodes)
     {
-    for (i = 0; i < svr_totnodes; ++i)
+    while ((pnode = next_node(&iter)) != NULL)
       {
-
-      pnode = pbsndlist[i];
-
       if (pnode->nd_state & INUSE_DELETED)
         continue;
 
+      /* count normally */
       if (pnode->nd_ntype == NTYPE_CLUSTER)
         svr_loc_clnodes += pnode->nd_nsn;
       else if (pnode->nd_ntype == NTYPE_TIMESHARED)
@@ -1428,58 +1502,251 @@ static struct pbssubn *create_subnode(
   }  /* END create_subnode() */
 
 
-/* recheck_for_node :
- * This function is called whenever an entry in the nodes file does
- * not resolve on server initialization. This function is called
- * periodically to see if the node is now resolvable and if so
- * add it to the list of available MOM nodes. */
-void recheck_for_node( struct work_task *ptask)
-  {
-  node_info *host_info;
-  int rc;
-  int bad;
 
-  host_info = ptask->wt_parm1;
-  if(host_info == NULL)
+
+int create_a_gpusubnode(
+    
+  struct pbsnode *pnode)
+
+  {
+  static char *id = "create_a_gpusubnode";
+  struct gpusubn *tmp;
+ 
+  tmp = malloc(sizeof(struct gpusubn) * (1 + pnode->nd_ngpus));
+
+  if (tmp == NULL)
     {
-    return;
+    log_err(ENOMEM,id,"Couldn't allocate memory for a subnode. EPIC FAILURE");
+    return(ENOMEM);
     }
 
-
-  rc = create_pbs_node( host_info->nodename, host_info->plist, host_info->perms, &bad);
-  if(rc)
+  if (pnode->nd_ngpus > 0)
     {
-    /* we created a new host_info in create_pbs_node. We
-       need to free this one */
-    free_attrlist(&host_info->atrlist);
-    if(host_info->nodename)
+    /* copy old memory to the new place */
+    memcpy(tmp,pnode->nd_gpusn,(sizeof(struct gpusubn) * pnode->nd_ngpus));
+    }
+
+  /* now use the new memory */
+  free(pnode->nd_gpusn);
+  pnode->nd_gpusn = tmp;
+
+  /* initialize the node */
+  pnode->nd_gpusn[pnode->nd_ngpus].pjob = NULL;
+  pnode->nd_gpusn[pnode->nd_ngpus].inuse = FALSE;
+  pnode->nd_gpusn[pnode->nd_ngpus].flag = 0;
+  pnode->nd_gpusn[pnode->nd_ngpus].index = pnode->nd_ngpus;
+
+  /* increment the number of gpu subnodes and gpus free */
+  pnode->nd_ngpus++;
+  pnode->nd_ngpus_free++;
+
+  return(PBSE_NONE);
+  } /* END create_a_gpusubnode() */
+
+
+
+
+/*
+ * copy the properties of node src to node dest
+ *
+ * @param dest - the node where the properties will be copied to
+ * @param src  - the node whose properties will be copied from
+ */
+int copy_properties(
+
+  struct pbsnode *dest, /* I */
+  struct pbsnode *src)  /* O */
+
+  {
+  int                    need;
+  int                    i;
+
+  struct prop           *pdest;
+  struct prop          **plink;
+
+  struct array_strings  *sub;
+  struct array_strings  *main_node;
+
+  /* copy features/properties */
+  if (src->nd_prop == NULL)
+    return(PBSE_NONE);
+
+  main_node = src->nd_prop;
+ 
+  /* allocate the properties for the numa node */
+  need = sizeof(struct array_strings) + main_node->as_npointers - 1;
+  dest->nd_prop = (struct array_strings *)malloc(need);
+  sub  = dest->nd_prop;
+
+  /* copy simple values */
+  sub->as_npointers = main_node->as_npointers;
+  sub->as_usedptr   = main_node->as_usedptr;
+  sub->as_bufsize   = main_node->as_bufsize;
+
+  /* allocate the buffer */
+  sub->as_buf = (char *)malloc(sub->as_bufsize);
+  memcpy(sub->as_buf,main_node->as_buf,sub->as_bufsize);
+
+  /* set sub's offset to the same as main_nodes. Ugly and convoluted
+   * but it works. Same process below when setting sub's as_string 
+   * values */
+  sub->as_next= sub->as_buf + (main_node->as_next - main_node->as_buf);
+
+  plink = &dest->nd_first;
+
+  for (i = 0; i < src->nd_nprops-1; i++)
+    {
+    sub->as_string[i] = sub->as_buf + (main_node->as_string[i] - main_node->as_buf);
+
+    pdest = init_prop(sub->as_string[i]);
+
+    *plink = pdest;
+    plink = &pdest->next;
+    }
+
+  /* now add in name as last prop */
+  pdest  = init_prop(dest->nd_name);
+  *plink = pdest;
+  dest->nd_last = pdest;
+
+  return(PBSE_NONE);
+  } /* END copy_properties() */
+
+
+
+
+
+/* creates the private numa nodes on this node 
+ *
+ * @param pnode - the node that will house the numa nodes
+ *
+ * @return 0 on success, -1 on failure
+ */
+int setup_numa_nodes(
+
+  struct pbsnode *pnode,
+  u_long         *pul)
+
+  {
+  int             i;
+  int             j;
+  struct pbsnode *pn;
+  char            pname[MAX_LINE];
+  char           *np_ptr = NULL;
+  char           *allocd_name;
+  int             np;
+  char           *delim = ",";
+
+  char           *id = "setup_numa_nodes";
+
+  if (pnode == NULL)
+    return(-1);
+
+  /* if this isn't a numa node, return no error */
+  if ((pnode->num_numa_nodes == 0) &&
+      (pnode->numa_str == NULL))
+    {
+    return(PBSE_NONE);
+    }
+
+  /* determine the number of cores per node */
+  if (pnode->numa_str != NULL)
+    {
+    np_ptr = strtok(pnode->numa_str,delim);
+    np = atoi(np_ptr);
+    }
+  else
+    {
+    np = pnode->nd_nsn / pnode->num_numa_nodes;
+    }
+
+  for (i = 0; i < pnode->num_numa_nodes; i++)
+    {
+    pn = (struct pbsnode *)malloc(sizeof(struct pbsnode));
+
+    /* each numa node just has a number for a name */
+    snprintf(pname,sizeof(pname),"%s-%d",
+      pnode->nd_name,
+      i);
+
+    allocd_name = strdup(pname);
+    if (allocd_name == NULL)
       {
-      free(host_info->nodename);
+      /* no memory error */
+      log_err(PBSE_SYSTEM,id,"Cannot allocate memory for node name\n");
+
+      return(PBSE_SYSTEM);
       }
 
-    free(host_info);
+    initialize_pbsnode(pn, allocd_name, pul, NTYPE_CLUSTER);
+
+    /* make sure the server communicates on the correct ports */
+    pn->nd_mom_port = pnode->nd_mom_port;
+    pn->nd_mom_rm_port = pnode->nd_mom_rm_port;
+
+    /* create the subnodes for this node */
+    for (j = 0; j < np; j++)
+      {
+      if (create_subnode(pn) == NULL)
+        {
+        /* ERROR */
+        pn->nd_state = INUSE_DELETED;
+
+        return(PBSE_SYSTEM);
+        }
+      }
+
+    /* set the number of processors */
+    if (np_ptr != NULL)
+      {
+      np_ptr = strtok(NULL,delim);
+      np = atoi(np_ptr);
+      }
+
+    copy_properties(pn,pnode);
+
+    /* add the node to the private tree */
+    pnode->numa_nodes = AVL_insert(i,
+        pn->nd_mom_port,
+        pn,
+        pnode->numa_nodes);
+    } /* END for each numa_node */
+
+  if (LOGLEVEL >= 3)
+    {
+    snprintf(log_buffer,sizeof(log_buffer),
+      "Successfully created %d numa nodes for node %s\n",
+      pnode->num_numa_nodes,
+      pnode->nd_name);
+
+    log_event(
+      PBSEVENT_SYSTEM,
+      PBS_EVENTCLASS_NODE,
+      id,
+      log_buffer);
     }
 
-  return;
-
-  }
+  return(PBSE_NONE);
+  } /* END setup_numa_nodes() */
 
 
-/**
+
+
+
+/*
  * create_pbs_node - create pbs node structure, i.e. add a node
- *
- * @see initialize_pbsnode() - child
  */
 
 int create_pbs_node(
 
-  char     *objname,  /* I */
+  char     *objname,
   svrattrl *plist,
-  int       perms,    /* I */
-  int      *bad)      /* O */
+  int       perms,
+  int      *bad)
 
   {
 
+  char *id = "create_pbs_node"; 
   struct pbsnode  *pnode = NULL;
 
   struct pbsnode **tmpndlist;
@@ -1488,65 +1755,14 @@ int create_pbs_node(
   u_long          *pul;  /* 0 terminated host adrs array*/
   int              rc;
   int              iht;
-  node_info        *host_info;
+  int              i;
+  int              reused_entry = 0;
+  u_long           addr;
 
   if ((rc = process_host_name_part(objname, &pul, &pname, &ntype)) != 0)
     {
-    svrattrl *pal, *pattrl;
-
     log_err(-1, "process_host_name_part", log_buffer);
 
-    /* the host name in the nodes file did not resolve.
-       We will set up a process to check periodically
-       to see if the node will resolve later */
-
-    host_info = (node_info *)calloc(1, sizeof(node_info));
-    if(host_info == NULL)
-      {
-      log_err(-1, "create_pbs_node calloc failed", log_buffer);
-      return(PBSE_SYSTEM);
-      }
-
-    CLEAR_HEAD(host_info->atrlist);
-
-    /* allocate and copy the objname plist and perms */
-    host_info->perms = perms;
-    pal = plist;
-
-    while(pal != NULL)
-      {
-      pattrl = attrlist_create(pal->al_atopl.name, 0, strlen(pal->al_atopl.value) + 1);
-      if(pattrl == NULL)
-        {
-        log_err(-1, "cannot create node attribute", log_buffer);
-        return(PBSE_SYSTEM);
-        }
-
-      strcpy(pattrl->al_value, pal->al_atopl.value);
-      pattrl->al_flags = SET;
-
-      append_link(&host_info->atrlist, &pattrl->al_link, pattrl);
-      pal = GET_NEXT(pal->al_link);
-      }
-
-    pattrl = GET_NEXT(host_info->atrlist);
-    host_info->plist = pattrl;
-
-    if(objname != NULL)
-      {
-      host_info->nodename = (char *)calloc(1, strlen(objname)+1);
-      if(host_info->nodename == NULL)
-        {
-        free(host_info);
-        log_err(-1, "create_pbs_node calloc failed", log_buffer);
-        return(PBSE_SYSTEM);
-        }
-      strcpy(host_info->nodename, objname);
-      }
-
-
-    set_task(WORK_Timed, time_now + 30 /*PBS_LOG_CHECK_RATE  five minutes */, recheck_for_node, host_info);
-
     return(rc);
     }
 
@@ -1570,13 +1786,14 @@ int create_pbs_node(
     return(PBSE_NODEEXIST);
     }
 
-  for (iht = 0;iht < svr_totnodes;iht++)
+  for (iht = 0; iht < svr_totnodes; iht++)
     {
     if (pbsndmast[iht]->nd_state & INUSE_DELETED)
       {
       /*available, use*/
 
       pnode = pbsndmast[iht];
+      reused_entry = 1;
 
       break;
       }
@@ -1600,7 +1817,7 @@ int create_pbs_node(
 
     /* expand pbsndmast array exactly svr_totnodes long*/
 
-    tmpndlist = (struct pbsnode **)realloc(
+/*    tmpndlist = (struct pbsnode **)realloc(
                   pbsndmast,
                   sizeof(struct pbsnode *) * (svr_totnodes + 1));
 
@@ -1611,11 +1828,11 @@ int create_pbs_node(
       free(pname);
 
       return(PBSE_SYSTEM);
-      }
+      }*/
 
-    /* add in the new entry etc */
+    /*add in the new entry etc*/
 
-    pbsndmast = tmpndlist;
+/*    pbsndmast = tmpndlist;
 
     pbsndmast[svr_totnodes++] = pnode;
 
@@ -1638,7 +1855,7 @@ int create_pbs_node(
       pbsndmast,
       svr_totnodes * sizeof(struct pbsnode *));
 
-    pbsndlist = tmpndlist;
+    pbsndlist = tmpndlist;*/
     }
 
   initialize_pbsnode(pnode, pname, pul, ntype);
@@ -1673,62 +1890,89 @@ int create_pbs_node(
     return(rc);
     }
 
+    /* expand pbsndmast array exactly svr_totnodes long*/
+
+  tmpndlist = (struct pbsnode **)realloc(
+                pbsndmast,
+                sizeof(struct pbsnode *) * (svr_totnodes + 1));
+
+  if (tmpndlist == NULL)
+    {
+    free(pnode);
+    free(pul);
+    free(pname);
+
+    return(PBSE_SYSTEM);
+    }
+
+  if (!reused_entry)
+    {
+    /*add in the new entry etc*/
+  
+    pbsndmast = tmpndlist;
+  
+    pbsndmast[svr_totnodes++] = pnode;
+  
+    tmpndlist = (struct pbsnode **)realloc(
+                  pbsndlist,
+                  sizeof(struct pbsnode *) * (svr_totnodes + 1));
+  
+    if (tmpndlist == NULL)
+      {
+      free(pnode);
+      free(pul);
+      free(pname);
+  
+      return(PBSE_SYSTEM);
+      }
+  
+    memcpy(
+  
+      tmpndlist,
+      pbsndmast,
+      svr_totnodes * sizeof(struct pbsnode *));
+  
+    pbsndlist = tmpndlist;
+    }
+
+
+  for (i = 0;pul[i];i++)
+    {
+    if (LOGLEVEL >= 6)
+      {
+      sprintf(log_buffer, "node '%s' allows trust for ipaddr %ld.%ld.%ld.%ld\n",
+              pnode->nd_name,
+              (pul[i] & 0xff000000) >> 24,
+              (pul[i] & 0x00ff0000) >> 16,
+              (pul[i] & 0x0000ff00) >> 8,
+              (pul[i] & 0x000000ff));
+
+      log_record(
+        PBSEVENT_SCHED,
+        PBS_EVENTCLASS_REQUEST,
+        id,
+        log_buffer);
+      }
+    
+/*    addr = pul[i] + pnode->nd_mom_port + pnode->nd_mom_rm_port; */
+    addr = pul[i];
+/*    tinsert(addr, pnode, &ipaddrs);*/
+    ipaddrs = AVL_insert(addr, pnode->nd_mom_port, pnode, ipaddrs);
+      
+    }  /* END for (i) */
+
+  setup_numa_nodes(pnode,pul);
+
   recompute_ntype_cnts();
 
   return(PBSE_NONE);     /*create completely successful*/
-  }  /* END create_pbs_node() */
+  } /* End create_pbs_node */
 
 
 
 
 
 
-int create_a_gpusubnode(
-
-  struct pbsnode *pnode)
-
-  {
-  static char *id = "create_a_gpusubnode";
-  struct gpusubn *tmp;
-
-  tmp = malloc(sizeof(struct gpusubn) * (1 + pnode->nd_ngpus));
-
-  if (tmp == NULL)
-    {
-    log_err(ENOMEM,id,"Couldn't allocate memory for a subnode. EPIC FAILURE");
-    return(ENOMEM);
-    }
-
-  if (pnode->nd_ngpus > 0)
-    {
-    /* copy old memory to the new place */
-    memcpy(tmp,pnode->nd_gpusn,(sizeof(struct gpusubn) * pnode->nd_ngpus));
-    }
-
-  /* now use the new memory */
-  free(pnode->nd_gpusn);
-  pnode->nd_gpusn = tmp;
-
-  /* initialize the node */
-  pnode->nd_gpus_real = FALSE;
-  pnode->nd_gpusn[pnode->nd_ngpus].pjob = NULL;
-  pnode->nd_gpusn[pnode->nd_ngpus].inuse = 0;
-  pnode->nd_gpusn[pnode->nd_ngpus].mode = gpu_normal;
-  pnode->nd_gpusn[pnode->nd_ngpus].state = gpu_unallocated;
-  pnode->nd_gpusn[pnode->nd_ngpus].flag = 0;
-  pnode->nd_gpusn[pnode->nd_ngpus].index = pnode->nd_ngpus;
-  pnode->nd_gpusn[pnode->nd_ngpus].gpuid = NULL;
-
-  /* increment the number of gpu subnodes and gpus free */
-  pnode->nd_ngpus++;
-  pnode->nd_ngpus_free++;
-
-  return(PBSE_NONE);
-  } /* END create_a_gpusubnode() */
-
-
-
-
 /*
  * parse_node_token - parse tokens in the nodes file
  *
@@ -1789,10 +2033,7 @@ static char *parse_node_token(
 
   *term = *pt;
 
-  if(*pt != '\0')
-    {
-    *pt++ = '\0';
-    }
+  *pt++ = '\0';
 
   return(ts);
   }  /* END parse_node_token() */
@@ -1870,8 +2111,8 @@ int setup_nodes(void)
 
   next_resource_tag = time(0); /* initialize next resource handle */
 
-  tfree(&streams);
-  tfree(&ipaddrs);
+  /*tfree(&streams);
+  tfree(&ipaddrs);*/
 
   svr_totnodes = 0;
 
@@ -2004,7 +2245,7 @@ int setup_nodes(void)
       int num_digits;
 
       start = atoi(open_bracket+1);
-
+      
       dash = strchr(open_bracket,'-');
       close_bracket = strchr(open_bracket,']');
 
@@ -2036,7 +2277,7 @@ int setup_nodes(void)
         int tmp = 10;
 
         strcpy(tmp_node_name,nodename);
-
+       
         /* determine the length of the number */
         while (start / tmp > 0)
           {
@@ -2086,10 +2327,9 @@ int setup_nodes(void)
 
     if (err != 0)
       {
-      sprintf(log_buffer, "could not create node \"%s\", error = %d (%s)",
+      sprintf(log_buffer, "could not create node \"%s\", error = %d",
               nodename,
-              err,
-              pbs_strerror(err));
+              err);
 
       log_record(
         PBSEVENT_SCHED,
@@ -2260,9 +2500,9 @@ static void delete_a_subnode(
 
 
 
-/*
+/* 
  * deletes the last gpu subnode
- * frees the node and decrements the number to adjust
+ * frees the node and decrements the number to adjust 
  */
 static void delete_a_gpusubnode(
 
@@ -2297,12 +2537,12 @@ static void delete_a_gpusubnode(
  * node_np_action - action routine for node's np attribute
  */
 
-int
-node_np_action(
+int node_np_action(
+    
   attribute *new,  /*derive props into this attribute*/
-  void *pobj,  /*pointer to a pbsnode struct     */
-  int actmode /*action mode; "NEW" or "ALTER"   */
-)
+  void *pobj,      /*pointer to a pbsnode struct     */
+  int actmode)     /*action mode; "NEW" or "ALTER"   */
+  
   {
 
   struct pbsnode *pnode = (struct pbsnode *)pobj;
@@ -2347,6 +2587,69 @@ node_np_action(
   } /* END node_np_action */
 
 
+/*
+ * node_mom_port_action - action routine for node's port attribute
+ */
+
+int node_mom_port_action(new, pobj, actmode)
+attribute *new;  /*derive props into this attribute*/
+void  *pobj;  /*pointer to a pbsnode struct     */
+int   actmode; /*action mode; "NEW" or "ALTER"   */
+  {
+
+  struct pbsnode *pnode = (struct pbsnode *)pobj;
+  int rc = 0;
+
+  switch (actmode)
+    {
+
+    case ATR_ACTION_NEW:
+      new->at_val.at_long = pnode->nd_mom_port;
+      break;
+
+    case ATR_ACTION_ALTER:
+      pnode->nd_mom_port = new->at_val.at_long;
+      break;
+
+    default:
+
+      rc = PBSE_INTERNAL;
+    }
+
+  return rc;
+  }
+
+/*
+ * node_mom_rm_port_action - action routine for node's port attribute
+ */
+
+int node_mom_rm_port_action(new, pobj, actmode)
+  attribute *new;  /*derive props into this attribute*/
+  void  *pobj;  /*pointer to a pbsnode struct     */
+  int   actmode; /*action mode; "NEW" or "ALTER"   */
+  {
+
+  struct pbsnode *pnode = (struct pbsnode *)pobj;
+  int rc = 0;
+
+  switch (actmode)
+    {
+
+    case ATR_ACTION_NEW:
+      new->at_val.at_long = pnode->nd_mom_rm_port;
+      break;
+
+    case ATR_ACTION_ALTER:
+      pnode->nd_mom_rm_port = new->at_val.at_long;
+      break;
+
+    default:
+
+      rc = PBSE_INTERNAL;
+    }
+
+  return rc;
+  }
 
 
 
@@ -2402,12 +2705,99 @@ int node_gpus_action(
 
 
 
+int node_numa_action(
+
+  attribute *new,           /*derive status into this attribute*/
+  void      *pnode,         /*pointer to a pbsnode struct     */
+  int        actmode)       /*action mode; "NEW" or "ALTER"   */
+
+  {
+
+  struct pbsnode *np = (struct pbsnode *)pnode;
+  int rc = 0;
+
+  switch (actmode)
+    {
+    case ATR_ACTION_NEW:
+      new->at_val.at_long = np->num_numa_nodes;
+      break;
+
+    case ATR_ACTION_ALTER:
+      np->num_numa_nodes = new->at_val.at_long;
+      break;
+
+    default:
+      rc = PBSE_INTERNAL;
+    }
+
+  return(rc);
+  } /* END node_numa_action */
+
+
+
+
+int numa_str_action(
+
+  attribute *new,           /*derive status into this attribute*/
+  void      *pnode,         /*pointer to a pbsnode struct     */
+  int        actmode)       /*action mode; "NEW" or "ALTER"   */
+
+  {
+  struct pbsnode *np = (struct pbsnode *)pnode;
+  int len;
+
+  switch (actmode)
+    {
+    case ATR_ACTION_NEW:
+
+      if (np->numa_str != NULL)
+        {
+        len = strlen(np->numa_str) + 1;
+        new->at_val.at_str = (char *)malloc(len * sizeof(char));
+
+        if (new->at_val.at_str == NULL)
+          return(PBSE_SYSTEM);
+
+        strcpy(new->at_val.at_str,np->numa_str);
+        }
+      else
+        new->at_val.at_str = NULL;
+
+      break;
+
+    case ATR_ACTION_ALTER:
+
+      if (new->at_val.at_str != NULL)
+        {
+        len = strlen(new->at_val.at_str) + 1;
+        np->numa_str = (char *)malloc(len * sizeof(char));
+
+        if (np->numa_str == NULL)
+          return(PBSE_SYSTEM);
+
+        strcpy(np->numa_str,new->at_val.at_str);
+        }
+      else
+        np->numa_str = NULL;
+
+      break;
+
+    default:
+      return(PBSE_INTERNAL);
+    }
+
+  return(0);
+  } /* END numa_str_action */
+
+
+
+
 /* create_partial_pbs_node - similar to create_pbs_node except there will
    only be a name for the new node and no attributes or properties */
 
 int create_partial_pbs_node(
-  char *nodename,
-  unsigned long addr,
+  char *nodename, 
+  unsigned long addr, 
   int perms)
 
   {
@@ -2422,7 +2812,7 @@ int create_partial_pbs_node(
 
   for(iht = 0; iht < svr_totnodes; iht++)
     {
-    if(pbsndmast[iht]->nd_state & INUSE_DELETED)
+    if (pbsndmast[iht]->nd_state & INUSE_DELETED)
       {
       /* there is a slot available */
       pnode = pbsndmast[iht];
@@ -2483,7 +2873,7 @@ int create_partial_pbs_node(
 
   ntype = NTYPE_CLUSTER;
   pul = (u_long *)malloc(sizeof(u_long) * 2);
-  if(!pul)
+  if (!pul)
     {
     free(pnode);
     return(PBSE_SYSTEM);
@@ -2523,8 +2913,98 @@ int create_partial_pbs_node(
 
   recompute_ntype_cnts();
 
-  tinsert(addr, pnode, &ipaddrs);
+  AVL_insert(addr, pnode->nd_mom_port, pnode, ipaddrs);
 
   return(PBSE_NONE);     /*create completely successful*/
   } /* END create_partial_pbs_node */
 
+
+
+
+
+/*
+ * @return a pointer to an initialized node iterator 
+ */
+node_iterator *get_node_iterator()
+
+  {
+  node_iterator *iter = (node_iterator *)malloc(sizeof(node_iterator));
+
+  if (iter != NULL)
+    {
+    iter->node_index = -1;
+    iter->numa_index = -1;
+    }
+
+  return(iter);
+  } /* END get_node_iterator() */
+
+
+
+
+
+/* 
+ * initializes an allocated node iterator 
+ */
+void reinitialize_node_iterator(
+
+  node_iterator *iter)
+
+  {
+  if (iter != NULL)
+    {
+    iter->node_index = -1;
+    iter->numa_index = -1;
+    }
+  } /* END reinitialize_node_iterator() */
+
+
+
+
+/* 
+ * @return the next node, from 0->end, accounting for numa nodes
+ */
+struct pbsnode *next_node(
+
+  node_iterator *iter)
+
+  {
+  struct pbsnode *next = NULL;
+
+  if (iter == NULL)
+    {
+    return(NULL);
+    }
+
+  /* conditionally advance the node index pointer */
+  if ((iter->node_index == -1) ||
+      (iter->numa_index+1 >= pbsndlist[iter->node_index]->num_numa_nodes))
+    {
+    iter->node_index++;
+    iter->numa_index = 0;
+    }
+  else
+    {
+    iter->numa_index++;
+    }
+
+  /* conditionally return the correct node */
+  if (iter->node_index < svr_totnodes)
+    {
+    next = pbsndlist[iter->node_index];
+    
+    if (next->num_numa_nodes > 0)
+      next = AVL_find(iter->numa_index,next->nd_mom_port,next->numa_nodes);
+
+    return(next);
+    }
+  else
+    {
+    /* end of list, no more nodes */
+    return(NULL);
+    }
+  } /* END next_node() */
+
+
+
+
