@@ -128,7 +128,7 @@
 /* global Data Items */
 
 extern char *msg_startup3;
-extern char     *msg_daemonname;
+extern char *msg_daemonname;
 extern char *msg_init_abt;
 extern char *msg_init_queued;
 extern char *msg_init_substate;
@@ -151,7 +151,7 @@ extern char *log_file;
 extern char *job_log_file;
 extern char *path_home;
 extern char *path_acct;
-extern char path_log[];
+extern char  path_log[];
 extern char *path_priv;
 extern char *path_arrays;
 extern char *path_jobs;
@@ -170,6 +170,7 @@ extern char *path_nodenote_new;
 extern char *path_checkpoint;
 extern char *path_jobinfo_log;
 
+
 extern int  queue_rank;
 extern char  server_name[];
 extern int  svr_delay_entry;
@@ -302,17 +303,17 @@ int DArrayAppend(
 
   {
 
-  if(Array->AppendIndex >= Array->Length)
+  if (Array->AppendIndex >= Array->Length)
     {
     int newLength = Array->Length * 2;
 
-    if(newLength <= 10)
+    if (newLength <= 10)
       newLength = 10;
 
     Array->Length = newLength;
     Array->Data = realloc(Array->Data,sizeof(Array->Data[0]) * Array->Length);
 
-    if(Array->Data == NULL)
+    if (Array->Data == NULL)
       {
       Array->Length = 0;
       Array->AppendIndex = 0;
@@ -344,8 +345,8 @@ static int SortPrioAscend(
   {
   job *pjob1 = *((job **)A);
   job *pjob2 = *((job **)B);
-  int prio1 = pjob1->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long;
-  int prio2 = pjob2->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long;
+  int prio1 = pjob1->ji_wattr[JOB_ATR_qrank].at_val.at_long;
+  int prio2 = pjob2->ji_wattr[JOB_ATR_qrank].at_val.at_long;
   return(prio1 - prio2);
   } /*END SortPrioAscend */
 
@@ -357,10 +358,10 @@ void  update_default_np()
   long default_np;
   long npfreediff;
 
-  default_np = server.sv_attr[(int)SRV_ATR_NPDefault].at_val.at_long;
+  default_np = server.sv_attr[SRV_ATR_NPDefault].at_val.at_long;
 
 
-  if(default_np > 0)
+  if (default_np > 0)
     {
     for(i = 0; i < svr_totnodes; i++)
       {
@@ -387,10 +388,7 @@ void add_server_names_to_acl_hosts(void)
   char buffer[PBS_MAXSERVERNAME+1];
   attribute temp;
 
-  struct attribute *patr = &server.sv_attr[(int)SRV_ATR_acl_hosts];
-
-  memset(buffer, 0, PBS_MAXSERVERNAME+1);
-  memset(&temp, 0, sizeof(attribute));
+  struct attribute *patr = &server.sv_attr[SRV_ATR_acl_hosts];
 
   server_list_ptr = pbs_get_server_list();
   list_len = csv_length(server_list_ptr);
@@ -643,7 +641,7 @@ int pbsd_init(
 
   /* 2. set up the various paths and other global variables we need */
 
-  if(path_priv == NULL)
+  if (path_priv == NULL)
     {
     path_priv      = build_path(path_home, PBS_SVR_PRIVATE, suffix_slash);
     }
@@ -660,7 +658,7 @@ int pbsd_init(
 
   path_acct  = build_path(path_priv, PBS_ACCT,     suffix_slash);
 
-  if(path_svrdb == NULL)
+  if (path_svrdb == NULL)
     {
     path_svrdb     = build_path(path_priv, PBS_SERVERDB, NULL);
     }
@@ -793,18 +791,18 @@ int pbsd_init(
   for (i = 0;i < SRV_ATR_LAST;i++)
     clear_attr(&server.sv_attr[i], &svr_attr_def[i]);
 
-  server.sv_attr[(int)SRV_ATR_scheduler_iteration].at_val.at_long =
+  server.sv_attr[SRV_ATR_scheduler_iteration].at_val.at_long =
     PBS_SCHEDULE_CYCLE;
 
-  server.sv_attr[(int)SRV_ATR_scheduler_iteration].at_flags =
+  server.sv_attr[SRV_ATR_scheduler_iteration].at_flags =
     ATR_VFLAG_SET;
 
-  server.sv_attr[(int)SRV_ATR_State].at_val.at_long = SV_STATE_INIT;
+  server.sv_attr[SRV_ATR_State].at_val.at_long = SV_STATE_INIT;
 
-  server.sv_attr[(int)SRV_ATR_State].at_flags = ATR_VFLAG_SET;
+  server.sv_attr[SRV_ATR_State].at_flags = ATR_VFLAG_SET;
 
-  svr_attr_def[(int)SRV_ATR_mailfrom].at_decode(
-    &server.sv_attr[(int)SRV_ATR_mailfrom],
+  svr_attr_def[SRV_ATR_mailfrom].at_decode(
+    &server.sv_attr[SRV_ATR_mailfrom],
     0,
     0,
     PBS_DEFAULT_MAIL);
@@ -817,28 +815,26 @@ int pbsd_init(
     server.sv_attr[(int)SRV_ATR_ping_rate].at_flags = ATR_VFLAG_SET;
   */
 
-  server.sv_attr[(int)SRV_ATR_tcp_timeout].at_val.at_long =
+  server.sv_attr[SRV_ATR_tcp_timeout].at_val.at_long =
     PBS_TCPTIMEOUT;
 
-  server.sv_attr[(int)SRV_ATR_tcp_timeout].at_flags = ATR_VFLAG_SET;
+  server.sv_attr[SRV_ATR_tcp_timeout].at_flags = ATR_VFLAG_SET;
 
-  server.sv_attr[(int)SRV_ATR_check_rate].at_val.at_long =
+  server.sv_attr[SRV_ATR_check_rate].at_val.at_long =
     PBS_NORMAL_PING_RATE / 2;
 
-  server.sv_attr[(int)SRV_ATR_check_rate].at_flags = ATR_VFLAG_SET;
+  server.sv_attr[SRV_ATR_check_rate].at_flags = ATR_VFLAG_SET;
 
-  server.sv_attr[(int)SRV_ATR_JobStatRate].at_val.at_long =
+  server.sv_attr[SRV_ATR_JobStatRate].at_val.at_long =
     PBS_RESTAT_JOB;
 
-  server.sv_attr[(int)SRV_ATR_PollJobs].at_val.at_long = PBS_POLLJOBS;
-
-  server.sv_attr[SRV_ATR_MoabArrayCompatible].at_val.at_long = TRUE;
+  server.sv_attr[SRV_ATR_PollJobs].at_val.at_long = PBS_POLLJOBS;
 
   /* 4. force logging of all types */
 
-  server.sv_attr[(int)SRV_ATR_log_events].at_val.at_long = PBSEVENT_MASK;
+  server.sv_attr[SRV_ATR_log_events].at_val.at_long = PBSEVENT_MASK;
 
-  server.sv_attr[(int)SRV_ATR_log_events].at_flags = ATR_VFLAG_SET;
+  server.sv_attr[SRV_ATR_log_events].at_flags = ATR_VFLAG_SET;
 
   /* 5. If not a "create" initialization, recover server db */
 
@@ -848,26 +844,17 @@ int pbsd_init(
     {
     /* Open the server database (save file) and read it in */
 
-#ifdef SERVER_XML
     if ((rc != 0) || ((rc = svr_recov_xml(path_svrdb, FALSE)) == -1)) 
       {
       log_err(rc, "pbsd_init", msg_init_baddb);
-      
-      return(-1);
-      }
-#else
-    if ((rc != 0) || ((rc = svr_recov(path_svrdb, FALSE)) == -1)) 
-      {
-      log_err(rc, "pbsd_init", msg_init_baddb);
-      
-      return(-1);
-      }
-#endif /* SERVER_XML */
 
-    if (server.sv_attr[(int)SRV_ATR_resource_assn].at_flags & ATR_VFLAG_SET)
+      return(-1);
+      }
+
+    if (server.sv_attr[SRV_ATR_resource_assn].at_flags & ATR_VFLAG_SET)
       {
-      svr_attr_def[(int)SRV_ATR_resource_assn].at_free(
-        &server.sv_attr[(int)SRV_ATR_resource_assn]);
+      svr_attr_def[SRV_ATR_resource_assn].at_free(
+        &server.sv_attr[SRV_ATR_resource_assn]);
       }
     }
   else
@@ -891,28 +878,23 @@ int pbsd_init(
       }
     }
 
-  svr_attr_def[(int)SRV_ATR_version].at_decode(
+  svr_attr_def[SRV_ATR_version].at_decode(
 
-    &server.sv_attr[(int)SRV_ATR_version],
+    &server.sv_attr[SRV_ATR_version],
     0,
     0,
     PACKAGE_VERSION);
 
-  /* 6. open accounting file and job log file if logging is set */
+  /* 6. open accounting file */
 
   if (acct_open(acct_file) != 0)
     {
     return(-1);
     }
 
-  if(server.sv_attr[(int)SRV_ATR_RecordJobInfo].at_val.at_long)
+  if (server.sv_attr[SRV_ATR_RecordJobInfo].at_val.at_long)
     {
-    rc = job_log_open(job_log_file, path_jobinfo_log);
-    if(rc != 0)
-      {
-      fprintf(stderr, "Could not open job_logs \n");
-      return(-1);
-      }
+    job_log_open(job_log_file, path_jobinfo_log);
     }
 
 
@@ -922,8 +904,8 @@ int pbsd_init(
     {
     /* a_option was set, overrides saved value of scheduling attr */
 
-    server.sv_attr[(int)SRV_ATR_scheduling].at_val.at_long = a_opt_init;
-    server.sv_attr[(int)SRV_ATR_scheduling].at_flags |=
+    server.sv_attr[SRV_ATR_scheduling].at_val.at_long = a_opt_init;
+    server.sv_attr[SRV_ATR_scheduling].at_flags |=
       ATR_VFLAG_SET;
     }
 
@@ -994,10 +976,10 @@ int pbsd_init(
           msg_daemonname,
           log_buffer);
 
-        if (pque->qu_attr[(int)QE_ATR_ResourceAssn].at_flags & ATR_VFLAG_SET)
+        if (pque->qu_attr[QE_ATR_ResourceAssn].at_flags & ATR_VFLAG_SET)
           {
-          que_attr_def[(int)QE_ATR_ResourceAssn].at_free(
-            &pque->qu_attr[(int)QE_ATR_ResourceAssn]);
+          que_attr_def[QE_ATR_ResourceAssn].at_free(
+            &pque->qu_attr[QE_ATR_ResourceAssn]);
           }
         }
       }
@@ -1053,9 +1035,9 @@ int pbsd_init(
           continue;
 
 
-        rc = array_recov(pdirent->d_name, &pa);
+        pa = array_recov(pdirent->d_name);
 
-        if (rc != PBSE_NONE)
+        if (pa == NULL)
           {
 
           sprintf(log_buffer,
@@ -1065,7 +1047,7 @@ int pbsd_init(
 
           log_err(errno, "pbsd_init", log_buffer);
 
-          return(rc);
+          continue;
 
           }
 
@@ -1215,7 +1197,7 @@ int pbsd_init(
 
       if ((type != RECOV_COLD) &&
           (type != RECOV_CREATE) &&
-          (!(pjob->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)) &&
+          (!(pjob->ji_wattr[JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)) &&
           (pjob->ji_qs.ji_svrflags & JOB_SVFLG_SCRIPT))
         {
         strcpy(basen, pjob->ji_qs.ji_jobid);
@@ -1268,9 +1250,9 @@ int pbsd_init(
 
     while (pjob != NULL)
       {
-      pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long = ++queue_rank;
+      pjob->ji_wattr[JOB_ATR_qrank].at_val.at_long = ++queue_rank;
 
-      job_save(pjob, SAVEJOB_FULL);
+     job_save(pjob, SAVEJOB_FULL, 0);
 
       pjob = (job *)GET_NEXT(pjob->ji_alljobs);
       }
@@ -1284,15 +1266,6 @@ int pbsd_init(
 
   while (pa != NULL)
     {
-    /* see if we need to upgrade the array version. */
-    /* We will upgrade from version 3 or later */
-    if(pa->ai_qs.struct_version == 3)
-      {
-      pa->ai_qs.struct_version = ARRAY_QS_STRUCT_VERSION;
-      pa->ai_qs.num_purged = pa->ai_qs.num_jobs - pa->jobs_recovered;
-      array_save(pa);
-      }
-
     pa->template_job = find_array_template(pa->ai_qs.parent_id);
 
     if (pa->ai_qs.num_cloned != pa->ai_qs.num_jobs)
@@ -1503,19 +1476,19 @@ static int pbsd_init_job(
 
   /* update at_server attribute in case name changed */
 
-  job_attr_def[(int)JOB_ATR_at_server].at_free(
-    &pjob->ji_wattr[(int)JOB_ATR_at_server]);
+  job_attr_def[JOB_ATR_at_server].at_free(
+    &pjob->ji_wattr[JOB_ATR_at_server]);
 
-  job_attr_def[(int)JOB_ATR_at_server].at_decode(
-    &pjob->ji_wattr[(int)JOB_ATR_at_server],
+  job_attr_def[JOB_ATR_at_server].at_decode(
+    &pjob->ji_wattr[JOB_ATR_at_server],
     NULL,
     NULL,
     server_name);
 
   /* update queue_rank if this job is higher than current */
 
-  if ((unsigned long)pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long > (unsigned long)queue_rank)
-    queue_rank = pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long;
+  if ((unsigned long)pjob->ji_wattr[JOB_ATR_qrank].at_val.at_long > (unsigned long)queue_rank)
+    queue_rank = pjob->ji_wattr[JOB_ATR_qrank].at_val.at_long;
 
   /* now based on the initialization type */
 
@@ -1761,10 +1734,10 @@ static int pbsd_init_job(
   if ((pjob->ji_qs.ji_un_type == JOB_UNION_TYPE_EXEC) &&
       (pjob->ji_qs.ji_un.ji_exect.ji_momaddr != 0))
     {
-    if (pjob->ji_wattr[(int)JOB_ATR_exec_host].at_flags & ATR_VFLAG_SET)
+    if (pjob->ji_wattr[JOB_ATR_exec_host].at_flags & ATR_VFLAG_SET)
       {
       pjob->ji_qs.ji_un.ji_exect.ji_momaddr = get_hostaddr(
-                                                parse_servername(pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str, &d));
+                                                parse_servername(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, &d));
       }
     else
       {
@@ -2034,7 +2007,7 @@ static void change_logs(
 
   acct_open(acct_file);
 
-  if(server.sv_attr[(int)SRV_ATR_RecordJobInfo].at_val.at_long)
+  if (server.sv_attr[SRV_ATR_RecordJobInfo].at_val.at_long)
     {
     job_log_open(job_log_file, path_jobinfo_log);
     }
@@ -2063,28 +2036,28 @@ static void change_log_level(
     /* increase log level */
 
     if (plogenv == NULL)
-      LOGLEVEL = server.sv_attr[(int)SRV_ATR_LogLevel].at_val.at_long;
+      LOGLEVEL = server.sv_attr[SRV_ATR_LogLevel].at_val.at_long;
 
     LOGLEVEL = MIN(LOGLEVEL + 1, 7);
 
     if (plogenv == NULL)
       {
-      server.sv_attr[(int)SRV_ATR_LogLevel].at_val.at_long = LOGLEVEL;
-      server.sv_attr[(int)SRV_ATR_LogLevel].at_flags = ATR_VFLAG_SET;
+      server.sv_attr[SRV_ATR_LogLevel].at_val.at_long = LOGLEVEL;
+      server.sv_attr[SRV_ATR_LogLevel].at_flags = ATR_VFLAG_SET;
       }
     }
   else if (sig == SIGUSR2)
     {
     /* decrease log level */
     if (plogenv == NULL)
-      LOGLEVEL = server.sv_attr[(int)SRV_ATR_LogLevel].at_val.at_long;
+      LOGLEVEL = server.sv_attr[SRV_ATR_LogLevel].at_val.at_long;
 
     LOGLEVEL = MAX(LOGLEVEL - 1, 0);
 
     if (plogenv == NULL)
       {
-      server.sv_attr[(int)SRV_ATR_LogLevel].at_val.at_long = LOGLEVEL;
-      server.sv_attr[(int)SRV_ATR_LogLevel].at_flags = ATR_VFLAG_SET;
+      server.sv_attr[SRV_ATR_LogLevel].at_val.at_long = LOGLEVEL;
+      server.sv_attr[SRV_ATR_LogLevel].at_flags = ATR_VFLAG_SET;
       }
     }
 
@@ -2122,7 +2095,7 @@ static void stop_me(
   int sig)
 
   {
-  server.sv_attr[(int)SRV_ATR_State].at_val.at_long = SV_STATE_SHUTSIG;
+  server.sv_attr[SRV_ATR_State].at_val.at_long = SV_STATE_SHUTSIG;
 
   return;
   }
@@ -2367,11 +2340,11 @@ int get_svr_attr(
     {
     /* Open the server database (save file) and read it in */
 
-    if(path_priv == NULL)
+    if (path_priv == NULL)
       {
       path_priv = build_path(path_home, PBS_SVR_PRIVATE, suffix_slash);
       }
-    if(path_svrdb == NULL)
+    if (path_svrdb == NULL)
       {
       path_svrdb     = build_path(path_priv, PBS_SERVERDB, NULL);
       }
@@ -2387,22 +2360,14 @@ int get_svr_attr(
         }
       }
 
-#ifdef SERVER_XML
     if (((rc = chk_save_file(path_svrdb))!= 0) || ((rc = svr_recov_xml(path_svrdb, TRUE)) == -1)) 
       {
       log_err(rc, id ,msg_init_baddb);
-      
+
       return(-1);
       }
-#else
-    if (((rc = chk_save_file(path_svrdb))!= 0) || ((rc = svr_recov(path_svrdb, TRUE)) == -1)) 
-      {
-      log_err(rc, id ,msg_init_baddb);
-      
-      return(-1);
-      }
-#endif /* SERVER_XML */
-    }
+
+    } 
 
   return(0);
   }  /* END get_svr_attr() */
