@@ -125,6 +125,7 @@
 #include "dis_init.h"
 #include "batch_request.h"
 #include "pbs_proto.h"
+#include "u_tree.h"
 #ifdef USE_HA_THREADS
 #include <pthread.h>
 #endif /* USE_HA_THREADS */
@@ -183,6 +184,7 @@ static int try_lock_out (int,int);
 
 extern int    svr_chngNodesfile;
 extern int    svr_totnodes;
+extern AvlTree streams;
 
 /* External Functions */
 
@@ -191,7 +193,7 @@ extern int    get_svr_attr (int);
 /* Local Private Functions */
 
 static int    get_port (char *, unsigned int *, pbs_net_t *);
-static int    daemonize_server (int,int *);
+static int    daemonize_server (int, pid_t *);
 int mutex_lock (mutex_t *);
 int mutex_unlock (mutex_t *);
 int get_file_info (char *,unsigned long *,long *,bool_t *,bool_t *);
@@ -257,7 +259,6 @@ int   server_init_type = RECOV_WARM;
 char         server_name[PBS_MAXSERVERNAME + 1]; /* host_name[:service|port] */
 int  svr_delay_entry = 0;
 int  svr_do_schedule = SCH_SCHEDULE_NULL;
-extern int  listener_command;
 tlist_head svr_queues;            /* list of queues                   */
 tlist_head svr_alljobs;           /* list of all jobs in server       */
 tlist_head svr_jobs_array_sum;    /* list of jobs in server, arrays summarized as single "placeholder" job */
@@ -280,6 +281,8 @@ int             TForceUpdate = 0;  /* (boolean) */
 char           *ProgName;
 char           *NodeSuffix = NULL;
 
+int MultiMomMode = 0;
+
 int allow_any_mom = FALSE;
 
 void DIS_rpp_reset(void)
@@ -319,7 +322,7 @@ void do_rpp(
   int  ret, proto, version;
 
   void is_request(int, int, int *);
-  void stream_eof(int, u_long, int);
+  void stream_eof(int, u_long, uint16_t, int);
 
   if (LOGLEVEL >= 4)
     {
@@ -350,10 +353,29 @@ void do_rpp(
 
       struct pbsnode *node;
 
-      extern tree *streams;        /* tree of stream numbers */
+      /* extern tree *streams; */        /* tree of stream numbers */
 
-      node = tfind((u_long)stream, &streams);
+/*      node = tfind((u_long)stream, &streams); */
+      node = AVL_find((u_long)stream, 0, streams);
 
+      if (ret == DIS_EOF)
+        {
+        sprintf(log_buffer, "stream %d closed.(node: \"%s\", %s) rc=%d (%s)",
+                stream,
+                (node != NULL) ? node->nd_name : "NULL",
+                netaddr(rpp_getaddr(stream)),
+                ret,
+                dis_emsg[ret]);
+
+        log_record(
+          PBSEVENT_SCHED,
+          PBS_EVENTCLASS_REQUEST,
+          id,
+          log_buffer);
+
+        }
+      else
+        {
       sprintf(log_buffer, "corrupt rpp request received on stream %d (node: \"%s\", %s) - invalid protocol - rc=%d (%s)",
               stream,
               (node != NULL) ? node->nd_name : "NULL",
@@ -367,8 +389,9 @@ void do_rpp(
         id,
         log_buffer);
       }
+      }
 
-    stream_eof(stream, 0, ret);
+    stream_eof(stream, 0, 0, ret);
 
     return;
     }  /* END if (ret != DIS_SUCCESS) */
@@ -391,7 +414,7 @@ void do_rpp(
         log_buffer);
       }
 
-    stream_eof(stream, 0, ret);
+    stream_eof(stream, 0, 0, ret);
 
     return;
     }
@@ -445,6 +468,12 @@ void rpp_request(
   {
   static char id[] = "rpp_request";
   int         stream;
+  struct      sockaddr name;
+  socklen_t   len;
+  int ret;
+
+  len = (socklen_t)sizeof(struct sockaddr);
+  ret = getsockname(fd, &name, &len);
 
   for (;;)
     {
@@ -475,7 +504,6 @@ clear_listeners(void)   /* I */
     listener_conns[i].address = 0;
     listener_conns[i].port = 0;
     listener_conns[i].sock = -1;
-    listener_conns[i].first_time = 1;
     }
 
   return;
@@ -639,7 +667,7 @@ void parse_command_line(int argc, char *argv[])
       case 'a':
 
         if (decode_b(
-              &server.sv_attr[(int)SRV_ATR_scheduling],
+              &server.sv_attr[SRV_ATR_scheduling],
               NULL,
               NULL,
               optarg) != 0)
@@ -649,7 +677,7 @@ void parse_command_line(int argc, char *argv[])
 
           exit(1);
           }
-        a_opt_init = server.sv_attr[(int)SRV_ATR_scheduling].at_val.at_long;
+        a_opt_init = server.sv_attr[SRV_ATR_scheduling].at_val.at_long;
 
         break;
 
@@ -757,8 +785,6 @@ void parse_command_line(int argc, char *argv[])
 
       case 'l':
 
-        clear_listeners();
-
         if (get_port(
               optarg,
               &listener_port,
@@ -842,6 +868,11 @@ void parse_command_line(int argc, char *argv[])
 
         break;
 
+      case 'm':
+
+        MultiMomMode = 1;
+        break;
+
       case 'M':
 
         if (get_port(optarg, &pbs_mom_port, &pbs_mom_addr))
@@ -942,7 +973,7 @@ static time_t next_task()
   struct work_task  *nxt;
 
   struct work_task  *ptask;
-  time_t      tilwhen = server.sv_attr[(int)SRV_ATR_scheduler_iteration].at_val.at_long;
+  time_t      tilwhen = server.sv_attr[SRV_ATR_scheduler_iteration].at_val.at_long;
 
   time_now = time((time_t *)0);
 
@@ -984,10 +1015,7 @@ static time_t next_task()
   /* should the scheduler be run?  If so, adjust the delay time  */
 
   if ((delay = server.sv_next_schedule - time_now) <= 0)
-    {
     svr_do_schedule = SCH_SCHEDULE_TIME;
-    listener_command = SCH_SCHEDULE_TIME;
-    }
   else if (delay < tilwhen)
     tilwhen = delay;
 
@@ -1080,7 +1108,7 @@ main_loop(void)
   /* do not check nodes immediately as they will initially be marked
      down unless they have already reported in */
 
-  when = server.sv_attr[(int)SRV_ATR_check_rate].at_val.at_long + time_now;
+  when = server.sv_attr[SRV_ATR_check_rate].at_val.at_long + time_now;
 
   if (svr_totnodes > 1024)
     {
@@ -1096,7 +1124,7 @@ main_loop(void)
 
   /* Just check the nodes with check_nodes above and don't ping anymore. */
 
-  set_task(WORK_Timed, time_now + 5, ping_nodes, NULL);
+/*  set_task(WORK_Timed, time_now + 5, ping_nodes, NULL); */
 
   set_task(WORK_Timed, time_now + 5, check_log, NULL);
 
@@ -1107,9 +1135,9 @@ main_loop(void)
    * following section constitutes the "main" loop of the server
    */
 
-  state = &server.sv_attr[(int)SRV_ATR_State].at_val.at_long;
+  state = &server.sv_attr[SRV_ATR_State].at_val.at_long;
 
-  DIS_tcp_settimeout(server.sv_attr[(int)SRV_ATR_tcp_timeout].at_val.at_long);
+  DIS_tcp_settimeout(server.sv_attr[SRV_ATR_tcp_timeout].at_val.at_long);
 
   if (server_init_type == RECOV_HOT)
     *state = SV_STATE_HOT;
@@ -1122,12 +1150,12 @@ main_loop(void)
     {
     /* first process any task whose time delay has expired */
 
-    if (server.sv_attr[(int)SRV_ATR_PollJobs].at_val.at_long)
+    if (server.sv_attr[SRV_ATR_PollJobs].at_val.at_long)
       waittime = MIN(next_task(), JobStatRate - (time_now - last_jobstat_time));
     else
       waittime = next_task();
 
-    waittime = MAX(1, waittime);
+    waittime = MAX(0, waittime);
 
     if (*state == SV_STATE_RUN)
       {
@@ -1136,11 +1164,11 @@ main_loop(void)
       /* if time or event says to run scheduler, do it */
 
       if ((svr_do_schedule != SCH_SCHEDULE_NULL) &&
-          server.sv_attr[(int)SRV_ATR_scheduling].at_val.at_long)
+          server.sv_attr[SRV_ATR_scheduling].at_val.at_long)
         {
         server.sv_next_schedule =
           time_now +
-          server.sv_attr[(int)SRV_ATR_scheduler_iteration].at_val.at_long;
+          server.sv_attr[SRV_ATR_scheduler_iteration].at_val.at_long;
 
         schedule_jobs();
         notify_listeners();
@@ -1205,11 +1233,11 @@ main_loop(void)
      */
 
     if (plogenv == NULL) /* If no specification of loglevel from env */
-      LOGLEVEL = server.sv_attr[(int)SRV_ATR_LogLevel].at_val.at_long;
+      LOGLEVEL = server.sv_attr[SRV_ATR_LogLevel].at_val.at_long;
 
     /* any running jobs need a status update? */
 
-    if (server.sv_attr[(int)SRV_ATR_PollJobs].at_val.at_long &&
+    if (server.sv_attr[SRV_ATR_PollJobs].at_val.at_long &&
         (last_jobstat_time + JobStatRate <= time_now))
       {
 
@@ -1223,7 +1251,7 @@ main_loop(void)
           {
           /* spread these out over the next JobStatRate seconds */
 
-          when = pjob->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long %
+          when = pjob->ji_wattr[JOB_ATR_qrank].at_val.at_long %
                  JobStatRate;
 
           ptask = set_task(WORK_Timed, when + time_now, poll_job_task, pjob);
@@ -1266,7 +1294,7 @@ main_loop(void)
        pjob = (job *)GET_NEXT(pjob->ji_alljobs))
     {
     if (pjob->ji_modified)
-      job_save(pjob, SAVEJOB_FULL);
+      job_save(pjob, SAVEJOB_FULL, 0);
     }
 
   if (svr_chngNodesfile)
@@ -1454,7 +1482,7 @@ int main(
 
   if (IamRoot() == 0)
     {
-	return(1);
+    return(1);
     }
 
   /*
@@ -1476,10 +1504,10 @@ int main(
    * If not use default location for it
    */
    
-  if ((server.sv_attr[(int)SRV_ATR_lockfile].at_flags & ATR_VFLAG_SET) &&
-               (server.sv_attr[(int)SRV_ATR_lockfile].at_val.at_str))
+  if ((server.sv_attr[SRV_ATR_lockfile].at_flags & ATR_VFLAG_SET) &&
+               (server.sv_attr[SRV_ATR_lockfile].at_val.at_str))
     {
-    char *LockfilePtr = server.sv_attr[(int)SRV_ATR_lockfile].at_val.at_str;
+    char *LockfilePtr = server.sv_attr[SRV_ATR_lockfile].at_val.at_str;
 
     /* check if an absolute path is specified or not */
 
@@ -1521,8 +1549,8 @@ int main(
   /* HA EVENTS MUST HAPPEN HERE */
 
   strcpy(HALockFile,lockfile);
-  HALockCheckTime = server.sv_attr[(int)SRV_ATR_LockfileCheckTime].at_val.at_long;
-  HALockUpdateTime = server.sv_attr[(int)SRV_ATR_LockfileUpdateTime].at_val.at_long;
+  HALockCheckTime = server.sv_attr[SRV_ATR_LockfileCheckTime].at_val.at_long;
+  HALockUpdateTime = server.sv_attr[SRV_ATR_LockfileUpdateTime].at_val.at_long;
 
   /* apply HA defaults */
 
@@ -1543,7 +1571,7 @@ int main(
 
   /* handle running in the background or not if we're debugging */
 
-  if(high_availability_mode)
+  if (high_availability_mode)
     {
     if (daemonize_server(TDoBackground,&sid) == FAILURE)
       {
@@ -1635,7 +1663,6 @@ int main(
     msg_daemonname,
     log_buffer);
 
-
   /* initialize the server objects and perform specified recovery */
   /* will be left in the server's private directory  */
   /* NOTE:  env cleared in pbsd_init() */
@@ -1682,7 +1709,7 @@ int main(
 
   /* handle running in the background or not if we're debugging */
 
-  if(!high_availability_mode)
+  if (!high_availability_mode)
     {
     if (daemonize_server(TDoBackground,&sid) == FAILURE)
       {
@@ -1867,12 +1894,12 @@ void check_log(
 
  /* remove logs older than LogKeepDays */
 
- if ((server.sv_attr[(int)SRV_ATR_LogKeepDays].at_flags 
+ if ((server.sv_attr[SRV_ATR_LogKeepDays].at_flags 
      & ATR_VFLAG_SET) != 0)
    {
    snprintf(log_buffer,sizeof(log_buffer),"checking for old pbs_server logs in dir '%s' (older than %ld days)",
      path_svrlog,
-     server.sv_attr[(int)SRV_ATR_LogKeepDays].at_val.at_long);
+     server.sv_attr[SRV_ATR_LogKeepDays].at_val.at_long);
  
    log_event(
      PBSEVENT_SYSTEM | PBSEVENT_FORCE,
@@ -1880,17 +1907,17 @@ void check_log(
      msg_daemonname,
      log_buffer);
 
-   if (log_remove_old(path_svrlog,server.sv_attr[(int)SRV_ATR_LogKeepDays].at_val.at_long * SECS_PER_DAY) != 0)
+   if (log_remove_old(path_svrlog,server.sv_attr[SRV_ATR_LogKeepDays].at_val.at_long * SECS_PER_DAY) != 0)
      {
      log_err(-1,"check_log","failure occurred when checking for old pbs_server logs");
      }
    }
 
-  if ((server.sv_attr[(int)SRV_ATR_LogFileMaxSize].at_flags
+  if ((server.sv_attr[SRV_ATR_LogFileMaxSize].at_flags
        & ATR_VFLAG_SET) != 0)
     {
-    if ((log_size() >= server.sv_attr[(int)SRV_ATR_LogFileMaxSize].at_val.at_long)
-       && (server.sv_attr[(int)SRV_ATR_LogFileMaxSize].at_val.at_long > 0))
+    if ((log_size() >= server.sv_attr[SRV_ATR_LogFileMaxSize].at_val.at_long)
+       && (server.sv_attr[SRV_ATR_LogFileMaxSize].at_val.at_long > 0))
       {
       log_event(
         PBSEVENT_SYSTEM | PBSEVENT_FORCE,
@@ -1898,10 +1925,10 @@ void check_log(
         msg_daemonname,
         "Rolling log file");
 
-      if ((server.sv_attr[(int)SRV_ATR_LogFileRollDepth].at_flags
+      if ((server.sv_attr[SRV_ATR_LogFileRollDepth].at_flags
            & ATR_VFLAG_SET) != 0)
         {
-        depth = server.sv_attr[(int)SRV_ATR_LogFileRollDepth].at_val.at_long;
+        depth = server.sv_attr[SRV_ATR_LogFileRollDepth].at_val.at_long;
         }
 
       if ((depth >= INT_MAX) || (depth < 1))
@@ -1918,7 +1945,7 @@ void check_log(
   /* periodically record the version and loglevel */
 
   sprintf(log_buffer, msg_info_server,
-    server.sv_attr[(int)SRV_ATR_version].at_val.at_str, LOGLEVEL);
+    server.sv_attr[SRV_ATR_version].at_val.at_str, LOGLEVEL);
 
   log_event(
     PBSEVENT_SYSTEM | PBSEVENT_FORCE,
@@ -1938,11 +1965,11 @@ void check_acct_log(
  struct work_task *ptask) /* I */
 
  {
-  if (((server.sv_attr[(int)SRV_ATR_AcctKeepDays].at_flags & ATR_VFLAG_SET) != 0)
-       && (server.sv_attr[(int)SRV_ATR_AcctKeepDays].at_val.at_long >= 0))
+  if (((server.sv_attr[SRV_ATR_AcctKeepDays].at_flags & ATR_VFLAG_SET) != 0)
+       && (server.sv_attr[SRV_ATR_AcctKeepDays].at_val.at_long >= 0))
     {
     sprintf(log_buffer,"Checking accounting files - keep days = %ld",
-      server.sv_attr[(int)SRV_ATR_AcctKeepDays].at_val.at_long);
+      server.sv_attr[SRV_ATR_AcctKeepDays].at_val.at_long);
 
     log_event(
       PBSEVENT_SYSTEM | PBSEVENT_FORCE,
@@ -1950,7 +1977,7 @@ void check_acct_log(
       msg_daemonname,
       log_buffer);
      
-    acct_cleanup(server.sv_attr[(int)SRV_ATR_AcctKeepDays].at_val.at_long);
+    acct_cleanup(server.sv_attr[SRV_ATR_AcctKeepDays].at_val.at_long);
     
     }
 
@@ -2041,7 +2068,7 @@ char *extract_dir(
 
   Dir[0] = '\0';
 
-  snprintf(Dir, DirSize,"%s",FullPath);
+  snprintf(Dir,sizeof(Dir),"%s",FullPath);
 
   ptr = strrchr(Dir,'/');
 
@@ -2073,19 +2100,19 @@ int is_ha_lock_file_valid(
   struct stat Stat;
   bool_t      GoodPermissions = FALSE;
 
-  if (lockfile[0] == '\0')
+  if (HALockFile[0] == '\0')
     {
     return(FALSE);
     }
 
-  extract_dir(lockfile, LockDir, sizeof(LockDir));
+  extract_dir(HALockFile,LockDir,sizeof(LockDir));
 
   if (stat(LockDir,&Stat) != 0)
     {
-    char *tmpLine;
+    char tmpLine[MAX_LINE];
 
     /* stat failed */
-    tmpLine = strerror(errno);
+    strerror_r(errno,ErrorString,sizeof(ErrorString));
 
     snprintf(tmpLine,sizeof(tmpLine),"could not stat the lockfile dir '%s': %s",
       LockDir,
@@ -2355,12 +2382,12 @@ void *update_ha_lock_thread(
 
     if ((rc == -1) && !ISEMPTYSTR(HALockFile))
       {
-      char *ErrorString;
+      char ErrorString[MAX_LINE];
       /* error occurred--immediate shutdown needed */
 
       if (LocalErrno != 0)
         {
-        ErrorString = strerror(LocalErrno);
+        strerror_r(LocalErrno,ErrorString,sizeof(ErrorString));
 
         sprintf(log_buffer,"could not update HA lock file '%s' in heartbeat thread (%s - errno %d:%s)",
           HALockFile,
@@ -2541,7 +2568,10 @@ static void lock_out_ha()
 
     NumChecks++;
 
-    UseFLock = is_ha_lock_file_valid(HALockFile);
+    if (is_ha_lock_file_valid(HALockFile) == FALSE)
+      {
+      UseFLock = TRUE;
+      }
 
     if (UseFLock == TRUE)
       {
@@ -3080,31 +3110,31 @@ void restore_attr_default(
     {
     case SRV_ATR_log_events:
 
-      server.sv_attr[(int)SRV_ATR_log_events].at_val.at_long = PBSEVENT_MASK;
+      server.sv_attr[SRV_ATR_log_events].at_val.at_long = PBSEVENT_MASK;
 
       break;
 
     case SRV_ATR_tcp_timeout:
 
-      server.sv_attr[(int)SRV_ATR_tcp_timeout].at_val.at_long = PBS_TCPTIMEOUT;
+      server.sv_attr[SRV_ATR_tcp_timeout].at_val.at_long = PBS_TCPTIMEOUT;
 
       break;
 
     case SRV_ATR_JobStatRate:
 
-      server.sv_attr[(int)SRV_ATR_JobStatRate].at_val.at_long = PBS_RESTAT_JOB;
+      server.sv_attr[SRV_ATR_JobStatRate].at_val.at_long = PBS_RESTAT_JOB;
 
       break;
 
     case SRV_ATR_PollJobs:
 
-      server.sv_attr[(int)SRV_ATR_PollJobs].at_val.at_long = PBS_POLLJOBS;
+      server.sv_attr[SRV_ATR_PollJobs].at_val.at_long = PBS_POLLJOBS;
 
       break;
 
     case SRV_ATR_LogLevel:
 
-      server.sv_attr[(int)SRV_ATR_LogLevel].at_val.at_long = 0;
+      server.sv_attr[SRV_ATR_LogLevel].at_val.at_long = 0;
 
       break; 
 
