@@ -112,6 +112,7 @@
 #include "svrfunc.h"
 #include "pbs_proto.h"
 #include "csv.h"
+#include "u_tree.h"
 
 #ifndef PBS_MOM
 #include "array.h"
@@ -148,6 +149,8 @@ extern char  *msg_request;
 extern char            server_name[];
 #endif
 
+extern AvlTree okclients;
+
 extern int LOGLEVEL;
 
 /* private functions local to this file */
@@ -184,7 +187,6 @@ void req_holdjob(struct batch_request *preq);
 void req_checkpointjob(struct batch_request *preq);
 void req_messagejob(struct batch_request *preq);
 void req_modifyjob(struct batch_request *preq);
-void req_gpuctrl(struct batch_request *preq);
 #ifndef PBS_MOM
 void req_orderjob(struct batch_request *preq);
 void req_rescreserve(struct batch_request *preq);
@@ -285,7 +287,7 @@ int get_creds(int sd, char *username, char *hostname)
 
   if (!(credentials = (ucreds *)malloc(size))) return 0;
 
-  memcpy(credentials, CMSG_DATA(cmptr), size);
+  *credentials = *(ucreds *)CMSG_DATA(cmptr);
 
   cpwd = getpwuid(SPC_PEER_UID(credentials));
 
@@ -323,11 +325,7 @@ void process_request(
 
   int                   rc;
 
-  struct batch_request *request = NULL;
-
-#ifndef PBS_MOM
-  char *auth_err = NULL;
-#endif
+  struct batch_request *request;
 
   time_now = time(NULL);
 
@@ -458,7 +456,7 @@ void process_request(
     strcpy(request->rq_host, server_name);
     }
 
-  if (server.sv_attr[(int)SRV_ATR_acl_host_enable].at_val.at_long)
+  if (server.sv_attr[SRV_ATR_acl_host_enable].at_val.at_long)
     {
     /* acl enabled, check it; always allow myself and nodes */
 
@@ -469,7 +467,7 @@ void process_request(
     if ((isanode == NULL) &&
         (strcmp(server_host, request->rq_host) != 0) &&
         (acl_check(
-           &server.sv_attr[(int)SRV_ATR_acl_hosts],
+           &server.sv_attr[SRV_ATR_acl_hosts],
            request->rq_host,
            ACL_Host) == 0))
       {
@@ -543,37 +541,21 @@ void process_request(
 
     if (ENABLE_TRUSTED_AUTH == TRUE )
       rc = 0;  /* bypass the authentication of the user--trust the client completely */
-    else if(munge_on)
+    else if (munge_on)
       {
-      /* If munge_on is true we will validate the connection now */
-      if ( request->rq_type == PBS_BATCH_AltAuthenUser)
-        {
-        rc = req_altauthenuser(request);
-        if (rc == PBSE_NONE)
-          {
-          conn_credent[sfds].timestamp = time_now;
-          svr_conn[sfds].cn_authen = PBS_NET_CONN_AUTHENTICATED;
-          }
-        return;
-        }
-      else if (svr_conn[sfds].cn_authen != PBS_NET_CONN_AUTHENTICATED)
-        /* skip checking user if we did not get an authenticated credential */
-        rc = PBSE_BADCRED;
-      else
-        {
-        rc = authenticate_user(request, &conn_credent[sfds], &auth_err);
-        }
+      /* If munge_on is true we will validate the connection later */
+      conn_credent[sfds].timestamp = time_now;
+      svr_conn[sfds].cn_authen = PBS_NET_CONN_AUTHENTICATED;
+      rc = 0;
       }
     else if (svr_conn[sfds].cn_authen != PBS_NET_CONN_AUTHENTICATED)
       rc = PBSE_BADCRED;
     else
-      rc = authenticate_user(request, &conn_credent[sfds], &auth_err);
+      rc = authenticate_user(request, &conn_credent[sfds]);
 
     if (rc != 0)
       {
-      req_reject(rc, 0, request, NULL, auth_err);
-      if (auth_err != NULL)
-        free(auth_err);
+      req_reject(rc, 0, request, NULL, NULL);
 
       close_client(sfds);
 
@@ -607,11 +589,11 @@ void process_request(
           (pjob->ji_qs.ji_state == JOB_STATE_RUNNING))
         {
 
-        if ((pjob->ji_wattr[(int)JOB_ATR_checkpoint].at_flags & ATR_VFLAG_SET) &&
-          ((csv_find_string(pjob->ji_wattr[(int)JOB_ATR_checkpoint].at_val.at_str, "s") != NULL) ||
-          (csv_find_string(pjob->ji_wattr[(int)JOB_ATR_checkpoint].at_val.at_str, "c") != NULL) ||
-          (csv_find_string(pjob->ji_wattr[(int)JOB_ATR_checkpoint].at_val.at_str, "enabled") != NULL)) &&
-          (strstr(pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str, short_host) != NULL))
+        if ((pjob->ji_wattr[JOB_ATR_checkpoint].at_flags & ATR_VFLAG_SET) &&
+          ((csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "s") != NULL) ||
+          (csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "c") != NULL) ||
+          (csv_find_string(pjob->ji_wattr[JOB_ATR_checkpoint].at_val.at_str, "enabled") != NULL)) &&
+          (strstr(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, short_host) != NULL))
           {
 
           request->rq_perm = svr_get_privilege(request->rq_user, server_host);
@@ -632,7 +614,7 @@ void process_request(
 
   /* if server shutting down, disallow new jobs and new running */
 
-  if (server.sv_attr[(int)SRV_ATR_State].at_val.at_long > SV_STATE_RUN)
+  if (server.sv_attr[SRV_ATR_State].at_val.at_long > SV_STATE_RUN)
     {
     switch (request->rq_type)
       {
@@ -657,7 +639,7 @@ void process_request(
 #else /* THIS CODE FOR MOM ONLY */
 
     {
-    extern tree *okclients;
+    /*extern tree *okclients; */
 
     extern void mom_server_update_receive_time_by_ip(u_long ipaddr, const char *cmd);
 
@@ -676,7 +658,8 @@ void process_request(
         log_buffer);
       }
 
-    if (!tfind(svr_conn[sfds].cn_addr, &okclients))
+/*    if (!tfind(svr_conn[sfds].cn_addr, &okclients)) */
+    if (!AVL_is_in_tree(svr_conn[sfds].cn_addr, 0, okclients))
       {
       sprintf(log_buffer, "request type %s from host %s rejected (host not authorized)",
         reqtype_to_txt(request->rq_type),
@@ -973,12 +956,6 @@ void dispatch_request(
 
       break;
 
-    case PBS_BATCH_GpuCtrl:
-
-      req_gpuctrl(request);
-
-      break;
-
     case PBS_BATCH_StatusJob:
 
       req_stat_job(request);
@@ -1041,6 +1018,9 @@ void dispatch_request(
       break;
 
     case PBS_BATCH_AltAuthenUser:
+      /* Use given authentication method to determine
+         if user is valid */
+       req_altauthenuser(request); 
 
       break;
 
@@ -1140,7 +1120,7 @@ struct batch_request *alloc_br(
 
   {
 
-  struct batch_request *req = NULL;
+  struct batch_request *req;
 
   req = (struct batch_request *)malloc(sizeof(struct batch_request));
 
