@@ -121,6 +121,7 @@ extern struct server server;
 extern time_t time_now;
 extern int   LOGLEVEL;
 
+
 /* Private Functions in this file */
 
 static void post_delete_route(struct work_task *);
@@ -141,11 +142,14 @@ void change_restart_comment_if_needed(struct job *);
 
 static char *deldelaystr = DELDELAY;
 static char *delpurgestr = DELPURGE;
+static char *delasyncstr = DELASYNC;
 
 /* Extern Functions */
 
 extern void set_resc_assigned(job *, enum batch_op);
-
+extern job  *chk_job_request(char *, struct batch_request *);
+extern struct batch_request *cpy_stage(struct batch_request *, job *, enum job_atr, int);
+extern int   svr_chk_owner(struct batch_request *, job *);
 
 /*
  * remove_stagein() - request that mom delete staged-in files for a job
@@ -159,6 +163,7 @@ void remove_stagein(
   {
 
   struct batch_request *preq = 0;
+  u_long addr;
 
   preq = cpy_stage(preq, pjob, JOB_ATR_stagein, 0);
 
@@ -172,8 +177,12 @@ void remove_stagein(
 
     preq->rq_extra = NULL;
 
+    addr = pjob->ji_qs.ji_un.ji_exect.ji_momaddr;
+    addr += pjob->ji_qs.ji_un.ji_exect.ji_mom_rmport;
+    addr += pjob->ji_qs.ji_un.ji_exect.ji_momport;
+
     if (relay_to_mom(
-          pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
+          pjob,
           preq,
           release_req) == 0)
       {
@@ -233,7 +242,7 @@ void ensure_deleted(
   
   job_purge(pjob);
 
-  } /* ensure_deleted */
+  } /* END ensure_deleted() */
 
 
 
@@ -335,6 +344,7 @@ void req_deletejob(
   if (preq->rq_extend != NULL)
     {
     if (strncmp(preq->rq_extend, deldelaystr, strlen(deldelaystr)) &&
+        strncmp(preq->rq_extend, delasyncstr, strlen(delasyncstr)) &&
         strncmp(preq->rq_extend, delpurgestr, strlen(delpurgestr)))
       {
       /* have text message in request extension, add it */
@@ -415,16 +425,10 @@ void req_deletejob(
     return;
     }
 
-  if (pjob->ji_qs.ji_substate == JOB_SUBSTATE_PRERUN ||
-      pjob->ji_qs.ji_substate == JOB_SUBSTATE_RERUN ||
-      pjob->ji_qs.ji_substate == JOB_SUBSTATE_RERUN1 ||
-      pjob->ji_qs.ji_substate == JOB_SUBSTATE_RERUN2 ||
-      pjob->ji_qs.ji_substate == JOB_SUBSTATE_RERUN3 )
+  if (pjob->ji_qs.ji_substate == JOB_SUBSTATE_PRERUN)
     {
-    /* If JOB_SUBSTATE_PRERUN being sent to MOM, wait till she gets it going */
+    /* being sent to MOM, wait till she gets it going */
     /* retry in one second                            */
-    /* If JOB_SUBSTATE_RERUN, RERUN1, RERUN2 or RERUN3 the
-       job is being requeued. Wait until finished */
 
     static time_t  cycle_check_when = 0;
     static char    cycle_check_jid[PBS_MAXSVRJOBID + 1];
@@ -550,6 +554,48 @@ jump:
      * comments at job_delete_nanny()).
      */
 
+    if (has_job_delete_nanny(pjob))
+      {
+      req_reject(PBSE_IVALREQ, 0, preq, NULL, "job cancel in progress");
+
+      return;
+      }
+
+    apply_job_delete_nanny(pjob, time_now + 60);
+
+    /* check if we are getting a asynchronous delete */
+
+    if ((preq->rq_extend != NULL) &&
+          !strncmp(preq->rq_extend,DELASYNC,strlen(DELASYNC)))
+      {
+      struct batch_request *preq_tmp = NULL;
+      /*
+       * Respond with an ack now instead of after MOM processing
+       * Create a new batch request and fill it in. It will be freed by reply_ack
+       */
+
+      LOG_EVENT(
+        PBSEVENT_JOB,
+        PBS_EVENTCLASS_JOB,
+        pjob->ji_qs.ji_jobid,
+        log_buffer);
+
+      preq_tmp = alloc_br(PBS_BATCH_DeleteJob);
+      preq_tmp->rq_perm = preq->rq_perm;
+      preq_tmp->rq_ind.rq_manager.rq_cmd = preq->rq_ind.rq_manager.rq_cmd;
+      preq_tmp->rq_ind.rq_manager.rq_objtype = preq->rq_ind.rq_manager.rq_objtype;
+      preq_tmp->rq_fromsvr = preq->rq_fromsvr;
+      preq_tmp->rq_extsz = preq->rq_extsz;
+      preq_tmp->rq_conn = preq->rq_conn;
+      memcpy(preq_tmp->rq_ind.rq_manager.rq_objname,
+          preq->rq_ind.rq_manager.rq_objname, PBS_MAXSVRJOBID + 1);
+      memcpy(preq_tmp->rq_user, preq->rq_user, PBS_MAXUSER + 1);
+      memcpy(preq_tmp->rq_host, preq->rq_host, PBS_MAXHOSTNAME + 1);
+
+      reply_ack(preq_tmp);
+      preq->rq_noreply = TRUE; /* set for no more replies */
+      }
+  
     /* make a cleanup task if set */
     if ((server.sv_attr[SRV_ATR_JobForceCancelTime].at_flags & ATR_VFLAG_SET) &&
         (server.sv_attr[SRV_ATR_JobForceCancelTime].at_val.at_long > 0))
@@ -564,15 +610,6 @@ jump:
         append_link(&pjob->ji_svrtask, &pwtcheck->wt_linkobj, pwtcheck);
       }
 
-    if (has_job_delete_nanny(pjob))
-      {
-      req_reject(PBSE_IVALREQ, 0, preq, NULL, "job cancel in progress");
-
-      return;
-      }
-
-    apply_job_delete_nanny(pjob, time_now + 60);
-
     /*
      * Send signal request to MOM.  The server will automagically
      * pick up and "finish" off the client request when MOM replies.
@@ -645,7 +682,7 @@ jump:
           
           svr_evaljobstate(tmp, &newstate, &newsub, 1);
           svr_setjobstate(tmp, newstate, newsub);
-          job_save(tmp, SAVEJOB_FULL);
+          job_save(tmp, SAVEJOB_FULL, 0);
 
           break;
           }
@@ -697,8 +734,8 @@ jump:
       }
 
     KeepSeconds = attr_ifelse_long(
-                    &pque->qu_attr[(int)QE_ATR_KeepCompleted],
-                    &server.sv_attr[(int)SRV_ATR_KeepCompleted],
+                    &pque->qu_attr[QE_ATR_KeepCompleted],
+                    &server.sv_attr[SRV_ATR_KeepCompleted],
                     0);
     ptask = set_task(WORK_Timed, time_now + KeepSeconds, on_job_exit, pjob);
 
@@ -727,8 +764,8 @@ void change_restart_comment_if_needed(
   struct job *pjob)
 
   {
-  if ((pjob->ji_wattr[(int)JOB_ATR_start_count].at_val.at_long > 1) &&
-    (pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_flags & ATR_VFLAG_SET))
+  if ((pjob->ji_wattr[JOB_ATR_start_count].at_val.at_long > 1) &&
+    (pjob->ji_wattr[JOB_ATR_checkpoint_restart_status].at_flags & ATR_VFLAG_SET))
     {
       char *token1 = NULL;
       char *token2 = NULL;
@@ -736,7 +773,7 @@ void change_restart_comment_if_needed(
       char *ptr;
       
       strncpy(commentMsg,
-        pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_val.at_str, 24);
+        pjob->ji_wattr[JOB_ATR_checkpoint_restart_status].at_val.at_str, 24);
       
       token1 = strtok(commentMsg," ");
       if (token1 != NULL)
@@ -745,11 +782,11 @@ void change_restart_comment_if_needed(
       if ((token2 != NULL) && 
         ((memcmp(token2,"failure",7) == 0) || (memcmp(token2,"restarted",9) == 0)))
         {
-        ptr = pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_val.at_str;
+        ptr = pjob->ji_wattr[JOB_ATR_checkpoint_restart_status].at_val.at_str;
         if (isupper(*ptr))
           {
             *ptr = tolower(*ptr);
-            pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_flags
+            pjob->ji_wattr[JOB_ATR_checkpoint_restart_status].at_flags
               |= ATR_VFLAG_SET;
             pjob->ji_modified = 1;
           }
@@ -876,8 +913,8 @@ static void post_delete_mom1(
     {
     pque = pjob->ji_qhdr;
 
-    delay = attr_ifelse_long(&pque->qu_attr[(int)QE_ATR_KillDelay],
-                             &server.sv_attr[(int)SRV_ATR_KillDelay],
+    delay = attr_ifelse_long(&pque->qu_attr[QE_ATR_KillDelay],
+                             &server.sv_attr[SRV_ATR_KillDelay],
                              2);
     }
 
@@ -965,7 +1002,7 @@ static int forced_jobpurge(
     if (!strncmp(preq->rq_extend, delpurgestr, strlen(delpurgestr)))
       {
       if (((preq->rq_perm & (ATR_DFLAG_OPRD | ATR_DFLAG_OPWR | ATR_DFLAG_MGRD | ATR_DFLAG_MGWR)) != 0) ||
-          ((svr_chk_owner(preq, pjob) == 0) && (server.sv_attr[(int)SRV_ATR_OwnerPurge].at_val.at_long)))
+          ((svr_chk_owner(preq, pjob) == 0) && (server.sv_attr[SRV_ATR_OwnerPurge].at_val.at_long)))
         {
         sprintf(log_buffer, "purging job without checking MOM");
 
@@ -1048,6 +1085,18 @@ void remove_job_delete_nanny(
 
   struct work_task *pwtiter, *pwtdel;
 
+  if (pjob->ji_svrtask.ll_next == NULL)
+    {
+    /* no nanny, nothing to delete */
+    return;
+    }
+
+  if (pjob->ji_svrtask.ll_next == NULL)
+    {
+    /* no nanny, nothing to delete */
+    return;
+    }
+
   if (pjob->ji_svrtask.ll_next == NULL)
     {
     /* no nanny, nothing to delete */
@@ -1096,7 +1145,7 @@ struct work_task *apply_job_delete_nanny(
 
   /* short-circuit if nanny isn't enabled */
 
-  if (!server.sv_attr[(int)SRV_ATR_JobNanny].at_val.at_long)
+  if (!server.sv_attr[SRV_ATR_JobNanny].at_val.at_long)
     {
     remove_job_delete_nanny(pjob); /* in case it was recently disabled */
 
@@ -1172,7 +1221,7 @@ static void job_delete_nanny(
 
   /* short-circuit if nanny isn't enabled */
 
-  if (!server.sv_attr[(int)SRV_ATR_JobNanny].at_val.at_long)
+  if (!server.sv_attr[SRV_ATR_JobNanny].at_val.at_long)
     {
     release_req(pwt);
 
@@ -1228,7 +1277,7 @@ static void post_job_delete_nanny(
   rc       = preq_sig->rq_reply.brp_code;
 
 
-  if (!server.sv_attr[(int)SRV_ATR_JobNanny].at_val.at_long)
+  if (!server.sv_attr[SRV_ATR_JobNanny].at_val.at_long)
     {
     /* the admin disabled nanny within the last minute or so */
 
@@ -1328,14 +1377,14 @@ void purge_completed_jobs(
         pjob = (job *)GET_NEXT(pjob->ji_alljobs)) 
     {
     if ((pjob->ji_qs.ji_substate == JOB_SUBSTATE_COMPLETE) &&
-      (pjob->ji_wattr[(int)JOB_ATR_comp_time].at_val.at_long <= purge_time) &&
-      ((pjob->ji_wattr[(int)JOB_ATR_reported].at_flags & ATR_VFLAG_SET) != 0) &&
-      (pjob->ji_wattr[(int)JOB_ATR_reported].at_val.at_long == 0))
+      (pjob->ji_wattr[JOB_ATR_comp_time].at_val.at_long <= purge_time) &&
+      ((pjob->ji_wattr[JOB_ATR_reported].at_flags & ATR_VFLAG_SET) != 0) &&
+      (pjob->ji_wattr[JOB_ATR_reported].at_val.at_long == 0))
       {
       if (LOGLEVEL >= 4)
         {
         sprintf(log_buffer,"Reported job is COMPLETED (%ld), setting reported to TRUE",
-          pjob->ji_wattr[(int)JOB_ATR_comp_time].at_val.at_long);
+          pjob->ji_wattr[JOB_ATR_comp_time].at_val.at_long);
           
         log_event(
           PBSEVENT_JOB, 
@@ -1344,11 +1393,11 @@ void purge_completed_jobs(
           log_buffer);
         }
 
-      pjob->ji_wattr[(int)JOB_ATR_reported].at_val.at_long = 1;
-      pjob->ji_wattr[(int)JOB_ATR_reported].at_flags =
+      pjob->ji_wattr[JOB_ATR_reported].at_val.at_long = 1;
+      pjob->ji_wattr[JOB_ATR_reported].at_flags =
         ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
           
-      job_save(pjob,SAVEJOB_FULL); 
+     job_save(pjob, SAVEJOB_FULL, 0); 
       }
     }
 
