@@ -116,6 +116,7 @@
 /* External Global Data Items */
 
 extern unsigned int   pbs_mom_port;
+extern unsigned int   pbs_rm_port;
 extern char *path_spool;
 extern int   server_init_type;
 extern pbs_net_t pbs_server_addr;
@@ -125,13 +126,11 @@ extern char *msg_job_end_sig;
 extern char *msg_job_end_stat;
 extern char *msg_momnoexec1;
 extern char *msg_momnoexec2;
-extern char *msg_momjoboverlimit;
 extern char *msg_obitnojob;
 extern char *msg_obitnocpy;
 extern char *msg_obitnodel;
 extern char  server_host[];
 extern int   svr_do_schedule;
-extern int   listener_command;
 extern time_t time_now;
 
 extern int   LOGLEVEL;
@@ -230,17 +229,17 @@ struct batch_request *setup_cpyfiles(
     strcpy(pcf->rq_jobid, pjob->ji_qs.ji_jobid);
 
     get_jobowner(
-      pjob->ji_wattr[(int)JOB_ATR_job_owner].at_val.at_str,
+      pjob->ji_wattr[JOB_ATR_job_owner].at_val.at_str,
       pcf->rq_owner);
 
     get_jobowner(
-      pjob->ji_wattr[(int)JOB_ATR_euser].at_val.at_str,
+      pjob->ji_wattr[JOB_ATR_euser].at_val.at_str,
       pcf->rq_user);
 
-    if (((pjob->ji_wattr[(int)JOB_ATR_egroup].at_flags & ATR_VFLAG_DEFLT) == 0) &&
-        (pjob->ji_wattr[(int)JOB_ATR_egroup].at_val.at_str != 0))
+    if (((pjob->ji_wattr[JOB_ATR_egroup].at_flags & ATR_VFLAG_DEFLT) == 0) &&
+        (pjob->ji_wattr[JOB_ATR_egroup].at_val.at_str != 0))
       {
-      strcpy(pcf->rq_group, pjob->ji_wattr[(int)JOB_ATR_egroup].at_val.at_str);
+      strcpy(pcf->rq_group, pjob->ji_wattr[JOB_ATR_egroup].at_val.at_str);
       }
     else
       {
@@ -308,7 +307,7 @@ static int is_joined(
     return(0);
     }
 
-  pattr = &pjob->ji_wattr[(int)JOB_ATR_join];
+  pattr = &pjob->ji_wattr[JOB_ATR_join];
 
   if (pattr->at_flags & ATR_VFLAG_SET)
     {
@@ -336,13 +335,13 @@ static struct batch_request *return_stdfile(
 
   {
 
-  if ((pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags) &&
-      (pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long))
+  if ((pjob->ji_wattr[JOB_ATR_interactive].at_flags) &&
+      (pjob->ji_wattr[JOB_ATR_interactive].at_val.at_long))
     {
     return NULL;
     }
 
-  if ((pjob->ji_wattr[(int)JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET) == 0)
+  if ((pjob->ji_wattr[JOB_ATR_checkpoint_name].at_flags & ATR_VFLAG_SET) == 0)
     {
     return NULL;
     }
@@ -390,13 +389,13 @@ static struct batch_request *cpy_stdfile(
   char  key;
 
   attribute *jkpattr;
-  attribute *pathattr = &pjob->ji_wattr[(int)ati];
+  attribute *pathattr = &pjob->ji_wattr[ati];
 
   char *suffix;
   char *to = NULL;
 
-  if ((pjob->ji_wattr[(int)JOB_ATR_interactive].at_flags) &&
-      (pjob->ji_wattr[(int)JOB_ATR_interactive].at_val.at_long))
+  if ((pjob->ji_wattr[JOB_ATR_interactive].at_flags) &&
+      (pjob->ji_wattr[JOB_ATR_interactive].at_val.at_long))
     {
     /* the job is interactive, don't bother to return output file */
 
@@ -444,7 +443,7 @@ static struct batch_request *cpy_stdfile(
    * the keep list, MOM has already placed the file in the user's HOME
    * directory.  It doesn't need to be copied.
    */
-  jkpattr = &pjob->ji_wattr[(int)JOB_ATR_keep];
+  jkpattr = &pjob->ji_wattr[JOB_ATR_keep];
 
   if ((jkpattr->at_flags & ATR_VFLAG_SET) &&
       (strchr(jkpattr->at_val.at_str, (int)key)))
@@ -534,7 +533,7 @@ struct batch_request *cpy_stage(
   char       *prmt;
   char       *to;
 
-  pattr = &pjob->ji_wattr[(int)ati];
+  pattr = &pjob->ji_wattr[ati];
 
   if (pattr->at_flags & ATR_VFLAG_SET)
     {
@@ -640,13 +639,13 @@ int mom_comm(
     if (pjob->ji_qs.ji_un.ji_exect.ji_momaddr == 0)
       {
       pjob->ji_qs.ji_un.ji_exect.ji_momaddr = get_hostaddr(
-                                                parse_servername(pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str, &dummy));
+                                                parse_servername(pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str, &dummy));
       }
 
     pjob->ji_momhandle = svr_connect(
 
                            pjob->ji_qs.ji_un.ji_exect.ji_momaddr,
-                           pbs_mom_port,
+                           pjob->ji_qs.ji_un.ji_exect.ji_momport,
                            process_Dreply,
                            ToServerDIS);
 
@@ -705,7 +704,6 @@ void rel_resc(
   /* mark that scheduler should be called */
 
   svr_do_schedule = SCH_SCHEDULE_TERM;
-  listener_command = SCH_SCHEDULE_TERM;
 
   return;
   }  /* END rel_resc() */
@@ -738,7 +736,9 @@ void on_job_exit(
   {
   int    handle = -1;
   job   *pjob;
+#ifdef VNODETESTING
   job   *pj;
+#endif
 
   struct batch_request *preq;
 
@@ -767,7 +767,9 @@ void on_job_exit(
     pjob = (job *)preq->rq_extra;
     }
 
-  /* make sure the job is actually still there */
+#ifdef VNODETESTING
+  /* FIXME: there might be a race with calling on_job_exit after a job has
+   * already been free'd.  This is temp code */
 
   pj = (job *)GET_NEXT(svr_alljobs);
 
@@ -779,25 +781,29 @@ void on_job_exit(
     pj = (job *)GET_NEXT(pj->ji_alljobs);
     }
 
-  /* if the job doesn't exist, just exit */
   if (pj == NULL)
     {
     sprintf(log_buffer, "on_job_exit called with INVALID pjob: %p",
-            (void *)pjob);
-  
-    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,"NULL",log_buffer);
-
-    return;
+            pjob);
     }
   else
     {
     sprintf(log_buffer, "on_job_exit valid pjob: %p (substate=%d)",
-            (void *)pjob,
+            pjob,
             pjob->ji_qs.ji_substate);
-    
-    log_event(PBSEVENT_JOB,PBS_EVENTCLASS_JOB,"NULL",log_buffer);
     }
 
+  log_event(
+
+    PBSEVENT_JOB,
+    PBS_EVENTCLASS_JOB,
+    pjob->ji_qs.ji_jobid,
+    log_buffer);
+
+  DBPRT(("%s\n",
+         log_buffer));
+#endif /* END VNODETESTING */
+
   /*
    * we don't need a handle if we are complete. On starting up we will NOT have
    * a connection to mom, but still want to try and process the completed job
@@ -833,7 +839,7 @@ void on_job_exit(
 
       /* see if job has any dependencies */
 
-      if (pjob->ji_wattr[(int)JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
+      if (pjob->ji_wattr[JOB_ATR_depend].at_flags & ATR_VFLAG_SET)
         {
         depend_on_term(pjob);
         }
@@ -872,8 +878,8 @@ void on_job_exit(
         if ((pque = pjob->ji_qhdr) && (pque->qu_attr != NULL))
           {
           KeepSeconds = attr_ifelse_long(
-                          &pque->qu_attr[(int)QE_ATR_KeepCompleted],
-                          &server.sv_attr[(int)SRV_ATR_KeepCompleted],
+                          &pque->qu_attr[QE_ATR_KeepCompleted],
+                          &server.sv_attr[SRV_ATR_KeepCompleted],
                           0);
           }
 
@@ -996,7 +1002,7 @@ void on_job_exit(
         if (LOGLEVEL >= 3)
           {
           snprintf(log_buffer, LOG_BUF_SIZE, "request to return spool files failed on node '%s' for job %s%s",
-                   pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+                   pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str,
                    pjob->ji_qs.ji_jobid,
                    (IsFaked == 1) ? "*" : "");
 
@@ -1139,7 +1145,7 @@ void on_job_exit(
 
         snprintf(log_buffer, LOG_BUF_SIZE, msg_obitnocpy,
                  pjob->ji_qs.ji_jobid,
-                 pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
+                 pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
 
         log_event(
           PBSEVENT_ERROR | PBSEVENT_ADMIN | PBSEVENT_JOB,
@@ -1150,7 +1156,7 @@ void on_job_exit(
         if (LOGLEVEL >= 3)
           {
           snprintf(log_buffer, LOG_BUF_SIZE, "request to copy stageout files failed on node '%s' for job %s%s",
-                   pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+                   pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str,
                    pjob->ji_qs.ji_jobid,
                    (IsFaked == 1) ? "*" : "");
 
@@ -1346,7 +1352,7 @@ void on_job_exit(
 
         snprintf(log_buffer, LOG_BUF_SIZE, msg_obitnodel,
                  pjob->ji_qs.ji_jobid,
-                 pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
+                 pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
 
         log_event(
           PBSEVENT_JOB,
@@ -1357,7 +1363,7 @@ void on_job_exit(
         if (LOGLEVEL >= 3)
           {
           snprintf(log_buffer, LOG_BUF_SIZE, "request to remove stage-in files failed on node '%s' for job %s%s",
-                   pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+                   pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str,
                    pjob->ji_qs.ji_jobid,
                    (IsFaked == 1) ? "*" : "");
 
@@ -1433,7 +1439,7 @@ void on_job_exit(
 
       /* see if restarted job failed */
 
-      if (pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_flags & ATR_VFLAG_SET)
+      if (pjob->ji_wattr[JOB_ATR_checkpoint_restart_status].at_flags & ATR_VFLAG_SET)
         {
         char *pfailtype = NULL;
         char *pfailure = NULL;
@@ -1441,7 +1447,7 @@ void on_job_exit(
         char errMsg[21];
         
         strncpy(errMsg,
-          pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_val.at_str, 20);
+          pjob->ji_wattr[JOB_ATR_checkpoint_restart_status].at_val.at_str, 20);
         
         pfailtype = strtok(errMsg," ");
         if (pfailtype != NULL)
@@ -1460,7 +1466,7 @@ void on_job_exit(
                 {
                 sprintf(log_buffer,
                   "Requeueing job after checkpoint restart failure: %s",
-                  pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_val.at_str);
+                  pjob->ji_wattr[JOB_ATR_checkpoint_restart_status].at_val.at_str);
 
                 log_event(
                   PBSEVENT_JOB,
@@ -1478,16 +1484,16 @@ void on_job_exit(
               {
               /* put job on hold */
 
-              hold_val = &pjob->ji_wattr[(int)JOB_ATR_hold].at_val.at_long;
+              hold_val = &pjob->ji_wattr[JOB_ATR_hold].at_val.at_long;
               *hold_val |= HOLD_s;
-              pjob->ji_wattr[(int)JOB_ATR_hold].at_flags |= ATR_VFLAG_SET;
+              pjob->ji_wattr[JOB_ATR_hold].at_flags |= ATR_VFLAG_SET;
               pjob->ji_modified = 1;
               svr_setjobstate(pjob, JOB_STATE_HELD, JOB_SUBSTATE_HELD);
               if (LOGLEVEL >= 4)
                 {
                 sprintf(log_buffer,
                   "Placing job on hold after checkpoint restart failure: %s",
-                  pjob->ji_wattr[(int)JOB_ATR_checkpoint_restart_status].at_val.at_str);
+                  pjob->ji_wattr[JOB_ATR_checkpoint_restart_status].at_val.at_str);
 
                 log_event(
                   PBSEVENT_JOB,
@@ -1526,8 +1532,8 @@ void on_job_exit(
       if ((pque = pjob->ji_qhdr) && (pque->qu_attr != NULL))
         {
         KeepSeconds = attr_ifelse_long(
-                        &pque->qu_attr[(int)QE_ATR_KeepCompleted],
-                        &server.sv_attr[(int)SRV_ATR_KeepCompleted],
+                        &pque->qu_attr[QE_ATR_KeepCompleted],
+                        &server.sv_attr[SRV_ATR_KeepCompleted],
                         0);
         }
 
@@ -1535,19 +1541,20 @@ void on_job_exit(
         {
         /* first time in */
 
-        if (((server.sv_attr[(int)SRV_ATR_JobMustReport].at_flags & ATR_VFLAG_SET) != 0)
-          && (server.sv_attr[(int)SRV_ATR_JobMustReport].at_val.at_long > 0))
+        if (((server.sv_attr[SRV_ATR_JobMustReport].at_flags & ATR_VFLAG_SET) != 0)
+          && (server.sv_attr[SRV_ATR_JobMustReport].at_val.at_long > 0))
           {
           MustReport = TRUE;
-          pjob->ji_wattr[(int)JOB_ATR_reported].at_val.at_long = 0;
+          pjob->ji_wattr[JOB_ATR_reported].at_val.at_long = 0;
 
-          pjob->ji_wattr[(int)JOB_ATR_reported].at_flags =
+          pjob->ji_wattr[JOB_ATR_reported].at_flags =
             ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
-          job_save(pjob,SAVEJOB_FULL);
+
+          job_save(pjob,SAVEJOB_FULL, 0);
           }
         }
-      else if (((pjob->ji_wattr[(int)JOB_ATR_reported].at_flags & ATR_VFLAG_SET) != 0)
-        && (pjob->ji_wattr[(int)JOB_ATR_reported].at_val.at_long == 0))
+      else if (((pjob->ji_wattr[JOB_ATR_reported].at_flags & ATR_VFLAG_SET) != 0)
+        && (pjob->ji_wattr[JOB_ATR_reported].at_val.at_long == 0))
         {
         MustReport = TRUE;
         }
@@ -1575,7 +1582,7 @@ void on_job_exit(
         /* is it first time in or server restart recovery */
         
         if ((handle == -1) &&
-            (pjob->ji_wattr[(int)JOB_ATR_comp_time].at_flags & ATR_VFLAG_SET))
+            (pjob->ji_wattr[JOB_ATR_comp_time].at_flags & ATR_VFLAG_SET))
           {
           /*
            * server restart - if we already have a completion_time then we
@@ -1583,18 +1590,18 @@ void on_job_exit(
            * use the comp_time to determine task invocation time
            */
           ptask = set_task(WORK_Timed,
-            pjob->ji_wattr[(int)JOB_ATR_comp_time].at_val.at_long + KeepSeconds,
+            pjob->ji_wattr[JOB_ATR_comp_time].at_val.at_long + KeepSeconds,
             on_job_exit, pjob);
           }
         else
           {
           /* First time in - Set the job completion time */
 
-          pjob->ji_wattr[(int)JOB_ATR_comp_time].at_val.at_long = (long)time(NULL);
-          pjob->ji_wattr[(int)JOB_ATR_comp_time].at_flags |= ATR_VFLAG_SET;
+          pjob->ji_wattr[JOB_ATR_comp_time].at_val.at_long = (long)time(NULL);
+          pjob->ji_wattr[JOB_ATR_comp_time].at_flags |= ATR_VFLAG_SET;
 
           ptask = set_task(WORK_Timed, time_now + KeepSeconds, on_job_exit, pjob);
-          job_save(pjob, SAVEJOB_FULL);
+          job_save(pjob, SAVEJOB_FULL, 0);
           }
 
         if (ptask != NULL)
@@ -1614,8 +1621,8 @@ void on_job_exit(
          */
         
         PurgeIt = TRUE;
-        if (((pjob->ji_wattr[(int)JOB_ATR_reported].at_flags & ATR_VFLAG_SET) != 0)
-          && (pjob->ji_wattr[(int)JOB_ATR_reported].at_val.at_long == 0))
+        if (((pjob->ji_wattr[JOB_ATR_reported].at_flags & ATR_VFLAG_SET) != 0)
+          && (pjob->ji_wattr[JOB_ATR_reported].at_val.at_long == 0))
           {
           if (LOGLEVEL >= 7)
             {
@@ -1763,7 +1770,7 @@ void on_job_rerun(
         if (LOGLEVEL >= 3)
           {
           snprintf(log_buffer, LOG_BUF_SIZE, "request to save output files failed on node '%s' for job %s%s",
-                   pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+                   pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str,
                    pjob->ji_qs.ji_jobid,
                    (IsFaked == 1) ? "*" : "");
 
@@ -1778,7 +1785,7 @@ void on_job_rerun(
 
         snprintf(log_buffer, LOG_BUF_SIZE, msg_obitnocpy,
                  pjob->ji_qs.ji_jobid,
-                 pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
+                 pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
 
         log_event(
           PBSEVENT_ERROR | PBSEVENT_ADMIN | PBSEVENT_JOB,
@@ -1857,7 +1864,7 @@ void on_job_rerun(
         if (LOGLEVEL >= 3)
           {
           snprintf(log_buffer, LOG_BUF_SIZE, "request to save stageout files failed on node '%s' for job %s%s",
-                   pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+                   pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str,
                    pjob->ji_qs.ji_jobid,
                    (IsFaked == TRUE) ? "*" : "");
 
@@ -1871,7 +1878,7 @@ void on_job_rerun(
         snprintf(log_buffer, LOG_BUF_SIZE, msg_obitnocpy,
 
                  pjob->ji_qs.ji_jobid,
-                 pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
+                 pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
 
         log_event(
           PBSEVENT_ERROR | PBSEVENT_ADMIN | PBSEVENT_JOB,
@@ -1958,7 +1965,7 @@ void on_job_rerun(
         if (LOGLEVEL >= 3)
           {
           snprintf(log_buffer, LOG_BUF_SIZE, "request to delete stagein files failed on node '%s' for job %s%s",
-                   pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str,
+                   pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str,
                    pjob->ji_qs.ji_jobid,
                    (IsFaked == TRUE) ? "*" : "");
 
@@ -1973,7 +1980,7 @@ void on_job_rerun(
 
         snprintf(log_buffer, LOG_BUF_SIZE, msg_obitnocpy,
                  pjob->ji_qs.ji_jobid,
-                 pjob->ji_wattr[(int)JOB_ATR_exec_host].at_val.at_str);
+                 pjob->ji_wattr[JOB_ATR_exec_host].at_val.at_str);
 
         log_event(
           PBSEVENT_ERROR | PBSEVENT_ADMIN | PBSEVENT_JOB,
@@ -2029,15 +2036,11 @@ void on_job_rerun(
         /* in case of server shutdown, don't clear exec_host */
         /* or session_id will use it on hotstart when next comes up */
 
-        job_attr_def[(int)JOB_ATR_exec_host].at_free(
-          &pjob->ji_wattr[(int)JOB_ATR_exec_host]);
-
-        job_attr_def[(int)JOB_ATR_session_id].at_free(
-          &pjob->ji_wattr[(int)JOB_ATR_session_id]);
-
-        job_attr_def[(int)JOB_ATR_exec_gpus].at_free(
-          &pjob->ji_wattr[JOB_ATR_exec_gpus]);
+        job_attr_def[JOB_ATR_exec_host].at_free(
+          &pjob->ji_wattr[JOB_ATR_exec_host]);
 
+        job_attr_def[JOB_ATR_session_id].at_free(
+          &pjob->ji_wattr[JOB_ATR_session_id]);
         }
 
       pjob->ji_modified = 1; /* force full job save */
@@ -2083,7 +2086,7 @@ static int setrerun(
   job *pjob)
 
   {
-  if (pjob->ji_wattr[(int)JOB_ATR_rerunable].at_val.at_long)
+  if (pjob->ji_wattr[JOB_ATR_rerunable].at_val.at_long)
     {
     /* job is rerunnable */
 
@@ -2349,21 +2352,12 @@ void req_jobobit(
    */
 
   exitstatus = preq->rq_ind.rq_jobobit.rq_status;
-  if (LOGLEVEL >= 6)
-    {
-    sprintf(log_buffer, "PBS_BATCH_JobObit exit status: %d", exitstatus);
-    log_event(
-      PBSEVENT_DEBUG,
-      PBSEVENT_JOB,
-      jobid,
-      log_buffer);
-    }
 
   pjob->ji_qs.ji_un.ji_exect.ji_exitstat = exitstatus;
 
-  pjob->ji_wattr[(int)JOB_ATR_exitstat].at_val.at_long = exitstatus;
+  pjob->ji_wattr[JOB_ATR_exitstat].at_val.at_long = exitstatus;
 
-  pjob->ji_wattr[(int)JOB_ATR_exitstat].at_flags |= ATR_VFLAG_SET;
+  pjob->ji_wattr[JOB_ATR_exitstat].at_flags |= ATR_VFLAG_SET;
 
   patlist = (svrattrl *)GET_NEXT(preq->rq_ind.rq_jobobit.rq_attr);
 
@@ -2470,13 +2464,6 @@ void req_jobobit(
     switch (exitstatus)
       {
 
-      case JOB_EXEC_OVERLIMIT:
-        /* the job exceeded some resource limit such as walltime, mem, pmem, cput, etc */
-        svr_mailowner(pjob, MAIL_ABORT, MAIL_FORCE, msg_momjoboverlimit);
-        alreadymailed = 1;
-
-        break;
-
       case JOB_EXEC_FAIL1:
 
       default:
@@ -2636,7 +2623,7 @@ void req_jobobit(
 
     account_jobend(pjob, acctbuf);
 
-    if (server.sv_attr[(int)SRV_ATR_log_events].at_val.at_long & PBSEVENT_JOB_USAGE)
+    if (server.sv_attr[SRV_ATR_log_events].at_val.at_long & PBSEVENT_JOB_USAGE)
       {
       /* log events set to record usage */
 
@@ -2685,7 +2672,7 @@ void req_jobobit(
 
     /* remove checkpoint restart file if there is one */
     
-    if (pjob->ji_wattr[(int)JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
+    if (pjob->ji_wattr[JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
       {
       cleanup_restart_file(pjob);
       }
@@ -2699,7 +2686,7 @@ void req_jobobit(
 
     if ((pjob->ji_qs.ji_svrflags & (JOB_SVFLG_CHECKPOINT_FILE | JOB_SVFLG_CHECKPOINT_MIGRATEABLE)) == 0)
       {
-      job_attr_def[(int)JOB_ATR_resc_used].at_free(&pjob->ji_wattr[(int)JOB_ATR_resc_used]);
+      job_attr_def[JOB_ATR_resc_used].at_free(&pjob->ji_wattr[JOB_ATR_resc_used]);
       }
     else if (pjob->ji_qs.ji_svrflags & JOB_SVFLG_CHECKPOINT_FILE)
       {
@@ -2758,7 +2745,7 @@ void req_jobobit(
 
     /* remove checkpoint restart file if there is one */
     
-    if (pjob->ji_wattr[(int)JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
+    if (pjob->ji_wattr[JOB_ATR_restart_name].at_flags & ATR_VFLAG_SET)
       {
       cleanup_restart_file(pjob);
       }
