@@ -110,6 +110,7 @@ extern char *pbs_o_host;
 
 extern int svr_movejob(job *, char *, struct batch_request *);
 extern int svr_chkque(job *, pbs_queue *, char *, int, char *);
+extern job  *chk_job_request(char *, struct batch_request *);
 
 /*
  * req_movejob = move a job to a new destination (local or remote)
@@ -163,7 +164,7 @@ void req_movejob(
   switch (svr_movejob(jobp, req->rq_ind.rq_move.rq_destin, req))
     {
 
-    case ROUTE_SUCCESS:
+    case 0:
 
       /* success */
 
@@ -184,9 +185,9 @@ void req_movejob(
 
       break;
 
-    case ROUTE_PERM_FAILURE:
+    case - 1:
 
-    case ROUTE_RETRY:
+    case 1:
 
       /* fail */
 
@@ -196,7 +197,7 @@ void req_movejob(
 
       break;
 
-    case ROUTE_DEFERRED:
+    case 2:
 
       /* deferred, will be handled by    */
       /* post_movejob() when the child completes */
@@ -289,12 +290,12 @@ void req_orderjob(
 
   /* now swap the order of the two jobs in the queue lists */
 
-  rank = pjob1->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long;
+  rank = pjob1->ji_wattr[JOB_ATR_qrank].at_val.at_long;
 
-  pjob1->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long =
-    pjob2->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long;
+  pjob1->ji_wattr[JOB_ATR_qrank].at_val.at_long =
+    pjob2->ji_wattr[JOB_ATR_qrank].at_val.at_long;
 
-  pjob2->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long = rank;
+  pjob2->ji_wattr[JOB_ATR_qrank].at_val.at_long = rank;
 
   if (pjob1->ji_qhdr != pjob2->ji_qhdr)
     {
@@ -315,9 +316,9 @@ void req_orderjob(
 
   /* need to update disk copy of both jobs to save new order */
 
-  job_save(pjob1, SAVEJOB_FULL);
+  job_save(pjob1, SAVEJOB_FULL, 0);
 
-  job_save(pjob2, SAVEJOB_FULL);
+  job_save(pjob2, SAVEJOB_FULL, 0);
 
   reply_ack(req);
 
