@@ -132,6 +132,7 @@ extern int  svr_authorize_jobreq(struct batch_request *, job *);
 extern int  svr_chkque(job *, pbs_queue *, char *, int, char *);
 extern int  job_route(job *);
 extern int node_avail_complex(char *, int *, int *, int *, int*);
+extern void  set_chkpt_deflt(job *, pbs_queue *);
 
 /* Global Data Items: */
 
@@ -187,61 +188,168 @@ static int user_account_read_user(char *);
 
 static char *pbs_o_que = "PBS_O_QUEUE=";
 
-/****************************************************************** 
- * set_nodes_attr - Check to see is the node resource was requested
- *                 on the qsub line. If not add the node attribute
- *                 to the Resource_List and set the value to 1. This
- *                 makes it so that if  procct is set on a queue
- *                 jobs without a node resource request will still
- *                 be properly routed.
- * Returns: 0 if OK 
- *          Non-Zero on failure 
- *****************************************************************/
 
-int set_nodes_attr(job *pjob)
+
+/*
+ * sum_select_mem_request
+ *
+ * parses the select statement for the total memory requested by the job
+ * if there is no select statement, this does nothing
+ */
+
+void sum_select_mem_request(
+
+  job * pj)
+
   {
-  resource *pres;
-	int  nodect_set = 0;
-  int  rc = 0;
-  char *pname;
+  char  id[] = "sum_select_mem_request";
+  char  select[] = "select";
+  char  mem_str[] = "mem=";
+  char  memval_str[MAXPATHLEN];
+  char *end;
+  char *current;
+  char *clause_end;
 
-  if(pjob->ji_wattr[JOB_ATR_resource].at_flags & ATR_VFLAG_SET)
+  int   mem_str_len = strlen(mem_str);
+  int   multiplier = 1;
+
+  unsigned long mem_total = 0;
+
+  if ((!(pj->ji_wattr[JOB_ATR_submit_args].at_flags & ATR_VFLAG_SET)) ||
+      (pj->ji_wattr[JOB_ATR_submit_args].at_val.at_str == NULL))
+    return;
+
+  current = strstr(pj->ji_wattr[JOB_ATR_submit_args].at_val.at_str,select);
+  
+  if (current != NULL)
     {
-    pres = (resource *)GET_NEXT(pjob->ji_wattr[JOB_ATR_resource].at_val.at_list);
-    while(pres != NULL)
+    /* comma delimits different options for -W */
+    end = strchr(current,',');
+    
+    /* make current the last character if no comma */
+    if (end == NULL)
+      end = current + strlen(current);
+
+    if (isdigit(*(current + 1 + strlen(select))))
+      multiplier = atoi(current + strlen(select) + 1);
+
+    clause_end = strchr(current,'+');
+
+    current = strstr(current,mem_str);
+    
+    /* find each mem request */
+    while ((current != NULL) &&
+           (current <  end - mem_str_len))
       {
-      if(pres->rs_defin != NULL)
+      unsigned long tmp;
+     
+      /* make sure we have the right number of "tasks" for this mem 
+       * request */
+      if (clause_end != NULL)
         {
-        pname = pres->rs_defin->rs_name;
-        if(pname == NULL || *pname == 0)
+        if (current > clause_end)
           {
-          pres = (resource *)GET_NEXT(pres->rs_link);
-          continue;
-          }
-        
-        if(strncmp(pname, "nodes", 5) == 0 
-          || strncmp(pname, "procs", 5) == 0)
-          {
-          nodect_set = 1;
-          break;
+          if (isdigit(*(clause_end+1)))
+            multiplier = atoi(clause_end+1);
+          else
+            multiplier = 1;
+
+          clause_end = strchr(clause_end+1,'+');
           }
         }
-        pres = (resource *)GET_NEXT(pres->rs_link);
-      }
-    }                                                                         
 
-    if(nodect_set == 0)
+      current += mem_str_len;
+      
+      tmp = atoi(current);
+      
+      /* advance past the digits to the units */
+      while ((current != NULL) &&
+             (isdigit(*current)))
+        current++;
+
+      if (current == NULL)
+        {
+        /* no units, assume kb */
+        mem_total += tmp;
+        
+        break;
+        }
+      
+      /* if not kb, convert */
+      switch (*current)
+        {
+        case 'k':
+        case 'K':
+          
+          /* do nothing, we're converting to kb */
+          
+          break;
+          
+        case 'm':
+        case 'M':
+          
+          tmp = tmp << 10;
+          
+          break;
+          
+        case 'g':
+        case 'G':
+          
+          tmp = tmp << 20;
+          
+          break;
+          
+        case 't':
+        case 'T':
+          
+          tmp = tmp << 30;
+          
+          break;
+          
+        default:
+          
+          snprintf(log_buffer,sizeof(log_buffer),
+            "WARNING:   Unknown unit %cb used in memory request\n",
+            *current);
+          log_err(-1,id,log_buffer);
+          
+          break;
+        }
+
+      mem_total += tmp * multiplier;
+      
+      current = strstr(current,mem_str);
+      }
+    }
+
+  /* set the memory requirement so we can enfore it where applicable */
+  if (mem_total != 0)
+    {
+    resource *mem_rc = find_resc_entry(
+        &pj->ji_wattr[JOB_ATR_resource],
+        find_resc_def(svr_resc_def,"mem",svr_resc_size));
+
+    if (mem_rc != NULL)
       {
-      int temp_perm;
-      /* neither procs nor nodes were requested. set procct to 1 */
-      temp_perm = resc_access_perm;
-      resc_access_perm = ATR_DFLAG_WRACC | ATR_DFLAG_MGWR | ATR_DFLAG_RMOMIG;
-      rc = decode_resc(&pjob->ji_wattr[JOB_ATR_resource], "Resource_List", "procct", "1");
-      resc_access_perm = temp_perm;
+      mem_rc->rs_value.at_val.at_size.atsv_num = mem_total;
+      mem_rc->rs_value.at_val.at_size.atsv_shift = 10;
       }
+    else
+      {
+      snprintf(memval_str,sizeof(memval_str),
+        "%lukb",
+        mem_total);
+
+      decode_resc(&pj->ji_wattr[JOB_ATR_resource],
+        ATTR_l,
+        "mem",
+        memval_str);
+      }
+    }
+
+  } /* END sum_select_mem_request() */
+
 
-  return(rc);
-  }   /* END set_nodes_attr() */
 
 
 /*
@@ -255,7 +363,7 @@ void req_quejob(
   struct batch_request *preq) /* ptr to the decoded request   */
 
   {
-  char          *id = "req_quejob";
+  char  *id = "req_quejob";
 
   char   basename[PBS_JOBBASE + 1];
   int   created_here = 0;
@@ -276,7 +384,6 @@ void req_quejob(
   pbs_queue *pque;
   char  *qname;
   attribute  tempattr;
-  struct stat stat_buf;
   char           EMsg[1024];
 
   /* set basic (user) level access permission */
@@ -311,7 +418,6 @@ void req_quejob(
     }
   else
     {
-    /* Create a job id */
     char host_server[PBS_MAXSERVERNAME + 1];
     int  server_suffix = TRUE;
 
@@ -379,9 +485,9 @@ void req_quejob(
       server.sv_qs.sv_jobidnumber = 0; /* wrap it */
 
     /* Make the current job number visible in qmgr print server commnad. */
-    server.sv_attr[(int)SRV_ATR_NextJobNumber].at_val.at_long = server.sv_qs.sv_jobidnumber;
+    server.sv_attr[SRV_ATR_NextJobNumber].at_val.at_long = server.sv_qs.sv_jobidnumber;
 
-    server.sv_attr[(int)SRV_ATR_NextJobNumber].at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
+    server.sv_attr[SRV_ATR_NextJobNumber].at_flags |= ATR_VFLAG_SET | ATR_VFLAG_MODIFY;
 
 
     if (svr_save(&server, SVR_SAVE_QUICK))
@@ -540,6 +646,7 @@ void req_quejob(
 
   while (psatl != NULL)
     {
+
     if (psatl->al_atopl.resource)
       {
       if (strcmp(psatl->al_atopl.resource, "nodes") == 0)
@@ -547,8 +654,8 @@ void req_quejob(
         pj->ji_have_nodes_request = 1;
         }
       }
-
     /* identify the attribute by name */
+
     attr_index = find_attr(job_attr_def, psatl->al_name, JOB_ATR_LAST);
 
     if (attr_index < 0)
@@ -619,27 +726,15 @@ void req_quejob(
       }    /* END if (rc != 0) */
 
     psatl = (svrattrl *)GET_NEXT(psatl->al_link);
-    } /* END while (psatl != NULL) */
+    }      /* END while (psatl != NULL) */
 
-  rc = set_nodes_attr(pj);
-  if(rc)
-    {
-    /* just record that we could not set node count */
-    sprintf(log_buffer, "Could not set default node count. Error not fatal. Will continue submitting job: %d",
-            rc);
-    LOG_EVENT(PBSEVENT_JOB,
-              PBS_EVENTCLASS_JOB,
-              id,
-              log_buffer);
-    }
-  
   /* perform any at_action routine declared for the attributes */
-  for (i = 0; i < JOB_ATR_LAST; ++i)
+
+  for (i = 0;i < JOB_ATR_LAST;++i)
     {
     pdef = &job_attr_def[i];
 
-    if ((pj->ji_wattr[i].at_flags & ATR_VFLAG_SET) && 
-        (pdef->at_action))
+    if ((pj->ji_wattr[i].at_flags & ATR_VFLAG_SET) && (pdef->at_action))
       {
       rc = pdef->at_action(&pj->ji_wattr[i], pj, ATR_ACTION_NEW);
 
@@ -654,6 +749,7 @@ void req_quejob(
       }
     }    /* END for (i) */
 
+  sum_select_mem_request(pj);
 
   /*
    * Now that the attributes have been decoded, we can setup some
@@ -692,41 +788,6 @@ void req_quejob(
         }
       }
 
-    /* check if a job id was supplied, and if so overwrite the job id */
-    if (pj->ji_wattr[JOB_ATR_job_id].at_flags & ATR_VFLAG_SET)
-      {
-      char *dot = strchr(pj->ji_qs.ji_jobid,'.');
-      char  tmp_extension[PBS_MAXSVRJOBID + 1];
-      job  *tmpjob;
-
-      if (dot != NULL)
-        {
-        strcpy(tmp_extension,dot);
-
-        snprintf(pj->ji_qs.ji_jobid, sizeof(pj->ji_qs.ji_jobid),
-          "%s%s",
-          pj->ji_wattr[JOB_ATR_job_id].at_val.at_str,
-          tmp_extension);
-        }
-      else
-        {
-        strcpy(pj->ji_qs.ji_jobid,pj->ji_wattr[JOB_ATR_job_id].at_val.at_str);
-        }
-
-      if ((tmpjob = find_job(pj->ji_qs.ji_jobid)) != NULL)
-        {
-        /* not unique, reject job */
-        job_purge(pj);
-
-        snprintf(log_buffer,sizeof(log_buffer),
-          "Job with id %s already exists, cannot set job id\n",
-          pj->ji_qs.ji_jobid);
-        req_reject(PBSE_JOBEXIST,0,preq,NULL,log_buffer);
-
-        return;
-        }
-      }
-
     /* set job owner attribute to user@host */
 
     job_attr_def[JOB_ATR_job_owner].at_free(
@@ -765,6 +826,9 @@ void req_quejob(
       pj->ji_wattr[JOB_ATR_rerunable].at_flags |= ATR_VFLAG_SET;
       }
 
+    /* need to set certain environmental variables per POSIX */
+
+    clear_attr(&tempattr, &job_attr_def[JOB_ATR_variables]);
 
     strcpy(buf, pbs_o_que);
 
@@ -776,39 +840,8 @@ void req_quejob(
       strcat(buf, pbs_o_host);
       strcat(buf, "=");
       strcat(buf, preq->rq_host);
-
-      /* We need to add the hostname to the list of attributes
-         for this job */
-      clear_attr(&tempattr, &job_attr_def[JOB_ATR_submit_host]);
-      rc = job_attr_def[JOB_ATR_submit_host].at_decode(
-               &tempattr,
-               NULL,
-               NULL,
-               preq->rq_host);
-      if (!rc)
-        {
-        rc = job_attr_def[JOB_ATR_submit_host].at_set(
-                 &pj->ji_wattr[JOB_ATR_submit_host],
-                 &tempattr,
-                 INCR);
-        }
-
-      if (rc)
-        {
-        sprintf(log_buffer, "failed to add submit_host %s. Minor error", preq->rq_host);
-        log_event(
-          PBSEVENT_JOB | PBSEVENT_SYSTEM,
-          PBS_EVENTCLASS_SERVER,
-          msg_daemonname,
-          log_buffer);
-        }
-      job_attr_def[JOB_ATR_submit_host].at_free(&tempattr);
       }
 
-    /* need to set certain environmental variables per POSIX */
-
-    clear_attr(&tempattr, &job_attr_def[JOB_ATR_variables]);
-
     job_attr_def[JOB_ATR_variables].at_decode(
         &tempattr,
         NULL,
@@ -847,36 +880,6 @@ void req_quejob(
           pj->ji_wattr[JOB_ATR_outpath].at_val.at_str,
           (int)'o')));
       }
-    else if (pj->ji_wattr[JOB_ATR_outpath].at_flags & ATR_VFLAG_SET)
-      {
-      /* check to see if the path pointed to is a directory. If so apendd the default output file name.
-         Otherwise we treat the string as a path to a file */
-      char *ptr;
-
-      ptr = strchr(pj->ji_wattr[JOB_ATR_outpath].at_val.at_str, ':');
-      if(ptr)
-        {
-        ptr++;
-        rc = stat(ptr, &stat_buf);
-        }
-      else 
-        {
-        rc = stat(pj->ji_wattr[JOB_ATR_outpath].at_val.at_str, &stat_buf);
-        }
-      if(rc == 0)
-        {
-        if(S_ISDIR(stat_buf.st_mode))
-          {
-/*          strcat(pj->ji_wattr[JOB_ATR_outpath].at_val.at_str, "/"); */
-          replace_attr_string(
-            &pj->ji_wattr[JOB_ATR_outpath],
-            (add_std_filename(pj,
-            pj->ji_wattr[JOB_ATR_outpath].at_val.at_str,
-            (int)'o')));
-      
-          }
-        }
-      }
 
     if (!(pj->ji_wattr[JOB_ATR_errpath].at_flags & ATR_VFLAG_SET) ||
         (((pj->ji_wattr[JOB_ATR_errpath].at_val.at_str[strlen(pj->ji_wattr[JOB_ATR_errpath].at_val.at_str) - 1] == ':'))))
@@ -901,36 +904,6 @@ void req_quejob(
           pj->ji_wattr[JOB_ATR_errpath].at_val.at_str,
           (int)'e')));
       }
-    else if (pj->ji_wattr[JOB_ATR_errpath].at_flags & ATR_VFLAG_SET)
-      {
-        /* check to see if the path pointed to is a directory. If so apendd the default output file name.
-           Otherwise we treat the string as a path to a file */
-      char *ptr;
-
-      ptr = strchr(pj->ji_wattr[JOB_ATR_errpath].at_val.at_str, ':');
-      if(ptr)
-        {
-        ptr++;
-        rc = stat(ptr, &stat_buf);
-        }
-      else 
-        {
-        rc = stat(pj->ji_wattr[JOB_ATR_errpath].at_val.at_str, &stat_buf);
-        }
-      if(rc == 0)
-        {
-        if(S_ISDIR(stat_buf.st_mode))
-          {
-/*          strcat(pj->ji_wattr[JOB_ATR_outpath].at_val.at_str, "/"); */
-          replace_attr_string(
-            &pj->ji_wattr[JOB_ATR_errpath],
-            (add_std_filename(pj,
-            pj->ji_wattr[JOB_ATR_errpath].at_val.at_str,
-            (int)'e')));
-      
-          }
-        }
-      }
 
     if ((pj->ji_wattr[JOB_ATR_outpath].at_val.at_str == NULL) ||
         (pj->ji_wattr[JOB_ATR_errpath].at_val.at_str == NULL))
@@ -1244,19 +1217,17 @@ void req_jobscript(
     {
     if (errno == 0)
       {
-      snprintf(log_buffer, sizeof(log_buffer),
-        "job %s in unexpected state '%s'",
-        pj->ji_qs.ji_jobid,
-        PJobSubState[pj->ji_qs.ji_substate]);
+      sprintf(log_buffer, "job %s in unexpected state '%s'",
+              pj->ji_qs.ji_jobid,
+              PJobSubState[pj->ji_qs.ji_substate]);
       }
     else
       {
-      snprintf(log_buffer, sizeof(log_buffer),
-        "job %s in unexpected state '%s' (errno=%d - %s)",
-        pj->ji_qs.ji_jobid,
-        PJobSubState[pj->ji_qs.ji_substate],
-        errno,
-        strerror(errno));
+      sprintf(log_buffer, "job %s in unexpected state '%s' (errno=%d - %s)",
+              pj->ji_qs.ji_jobid,
+              PJobSubState[pj->ji_qs.ji_substate],
+              errno,
+              strerror(errno));
       }
 
     log_err(errno, id, log_buffer);
@@ -1457,10 +1428,9 @@ void req_mvjobfile(  /* NOTE:  routine for server only - mom code follows this r
 
   if (LOGLEVEL >= 6)
     {
-    snprintf(log_buffer, sizeof(log_buffer),
-      "successfully moved file '%s' for job '%s'",
-      namebuf,
-      preq->rq_ind.rq_jobfile.rq_jobid);
+    sprintf(log_buffer, "successfully moved file '%s' for job '%s'",
+            namebuf,
+            preq->rq_ind.rq_jobfile.rq_jobid);
 
     log_record(
       PBSEVENT_JOB,
@@ -1546,13 +1516,13 @@ void req_rdytocommit(
   OrigState  = pj->ji_qs.ji_state;
 
   OrigSState = pj->ji_qs.ji_substate;
-  OrigSChar  = pj->ji_wattr[(int)JOB_ATR_state].at_val.at_char;
-  OrigFlags  = pj->ji_wattr[(int)JOB_ATR_state].at_flags;
+  OrigSChar  = pj->ji_wattr[JOB_ATR_state].at_val.at_char;
+  OrigFlags  = pj->ji_wattr[JOB_ATR_state].at_flags;
 
   pj->ji_qs.ji_state    = JOB_STATE_TRANSIT;
   pj->ji_qs.ji_substate = JOB_SUBSTATE_TRANSICM;
-  pj->ji_wattr[(int)JOB_ATR_state].at_val.at_char = 'T';
-  pj->ji_wattr[(int)JOB_ATR_state].at_flags |= ATR_VFLAG_SET;
+  pj->ji_wattr[JOB_ATR_state].at_val.at_char = 'T';
+  pj->ji_wattr[JOB_ATR_state].at_flags |= ATR_VFLAG_SET;
 
   /* if this is a job array template then we'll delete the .JB file that 
      was created for this job since we are going to save it with a different 
@@ -1560,7 +1530,7 @@ void req_rdytocommit(
      XXX: not sure why the .JB file already exists before we do the SAVEJOB_NEW
      save below
    */
-  if (pj->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)
+  if (pj->ji_wattr[JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)
     {
     pj->ji_is_array_template = TRUE;
 
@@ -1572,14 +1542,13 @@ void req_rdytocommit(
 
     }
 
-  if (job_save(pj, SAVEJOB_NEW) == -1)
+  if (job_save(pj, SAVEJOB_NEW, 0) == -1)
     {
     char tmpLine[1024];
 
-    snprintf(tmpLine, sizeof(tmpLine),
-      "cannot save job - errno=%d - %s",
-      errno,
-      strerror(errno));
+    sprintf(tmpLine, "cannot save job - errno=%d - %s",
+            errno,
+            strerror(errno));
 
     log_err(errno, id, tmpLine);
 
@@ -1587,8 +1556,8 @@ void req_rdytocommit(
 
     pj->ji_qs.ji_state    = OrigState;
     pj->ji_qs.ji_substate = OrigSState;
-    pj->ji_wattr[(int)JOB_ATR_state].at_val.at_char = OrigSChar;
-    pj->ji_wattr[(int)JOB_ATR_state].at_flags = OrigFlags;
+    pj->ji_wattr[JOB_ATR_state].at_val.at_char = OrigSChar;
+    pj->ji_wattr[JOB_ATR_state].at_flags = OrigFlags;
 
     req_reject(PBSE_SYSTEM, 0, preq, NULL, tmpLine);
 
@@ -1603,10 +1572,9 @@ void req_rdytocommit(
     {
     /* reply failed, purge the job and close the connection */
 
-    snprintf(log_buffer, sizeof(log_buffer),
-      "cannot report jobid - errno=%d - %s",
-      errno,
-      strerror(errno));
+    sprintf(log_buffer, "cannot report jobid - errno=%d - %s",
+            errno,
+            strerror(errno));
 
     log_err(errno, id, log_buffer);
 
@@ -1705,15 +1673,15 @@ void req_commit(
   OrigState  = pj->ji_qs.ji_state;
 
   OrigSState = pj->ji_qs.ji_substate;
-  OrigSChar  = pj->ji_wattr[(int)JOB_ATR_state].at_val.at_char;
-  OrigFlags  = pj->ji_wattr[(int)JOB_ATR_state].at_flags;
+  OrigSChar  = pj->ji_wattr[JOB_ATR_state].at_val.at_char;
+  OrigFlags  = pj->ji_wattr[JOB_ATR_state].at_flags;
 
   pj->ji_qs.ji_state    = JOB_STATE_TRANSIT;
   pj->ji_qs.ji_substate = JOB_SUBSTATE_TRANSICM;
-  pj->ji_wattr[(int)JOB_ATR_state].at_val.at_char = 'T';
-  pj->ji_wattr[(int)JOB_ATR_state].at_flags |= ATR_VFLAG_SET;
+  pj->ji_wattr[JOB_ATR_state].at_val.at_char = 'T';
+  pj->ji_wattr[JOB_ATR_state].at_flags |= ATR_VFLAG_SET;
 
-  if (pj->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)
+  if (pj->ji_wattr[JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET)
     {
     pj->ji_is_array_template = TRUE;
     
@@ -1783,7 +1751,7 @@ void req_commit(
         }
       return;
       }
-    }  /* end if (pj->ji_wattr[(int)JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET) */
+    }  /* end if (pj->ji_wattr[JOB_ATR_job_array_request].at_flags & ATR_VFLAG_SET) */
 
   svr_evaljobstate(pj, &newstate, &newsub, 1);
 
@@ -1791,9 +1759,9 @@ void req_commit(
 
   /* set the queue rank attribute */
 
-  pj->ji_wattr[(int)JOB_ATR_qrank].at_val.at_long = ++queue_rank;
+  pj->ji_wattr[JOB_ATR_qrank].at_val.at_long = ++queue_rank;
 
-  pj->ji_wattr[(int)JOB_ATR_qrank].at_flags |= ATR_VFLAG_SET;
+  pj->ji_wattr[JOB_ATR_qrank].at_flags |= ATR_VFLAG_SET;
 
   if ((rc = svr_enquejob(pj)))
     {
@@ -1813,7 +1781,7 @@ void req_commit(
     return;
     }
 
-  if (job_save(pj, SAVEJOB_FULL) != 0)
+  if (job_save(pj, SAVEJOB_FULL, 0) != 0)
     {
     job_purge(pj);
 
@@ -1841,7 +1809,7 @@ void req_commit(
 
   if ((preq->rq_fromsvr == 0) &&
       (pque->qu_qs.qu_type == QTYPE_RoutePush) &&
-      (pque->qu_attr[(int)QA_ATR_Started].at_val.at_long != 0))
+      (pque->qu_attr[QA_ATR_Started].at_val.at_long != 0))
     {
     if ((rc = job_route(pj)))
       {
@@ -1869,7 +1837,7 @@ void req_commit(
    * branch_request needs re creation since reply_jobid will free
    * the passed in one
   */
-  pattr = &pj->ji_wattr[(int)JOB_ATR_start_count];
+  pattr = &pj->ji_wattr[JOB_ATR_start_count];
 
   spec = PBS_DEFAULT_NODE;
 
@@ -1896,12 +1864,11 @@ void req_commit(
 
   /* need to format message first, before request goes away */
 
-  snprintf(log_buffer, sizeof(log_buffer),
-    msg_jobnew,
-    preq->rq_user, preq->rq_host,
-    pj->ji_wattr[(int)JOB_ATR_job_owner].at_val.at_str,
-    pj->ji_wattr[(int)JOB_ATR_jobname].at_val.at_str,
-    pj->ji_qhdr->qu_qs.qu_name);
+  sprintf(log_buffer, msg_jobnew,
+          preq->rq_user, preq->rq_host,
+          pj->ji_wattr[JOB_ATR_job_owner].at_val.at_str,
+          pj->ji_wattr[JOB_ATR_jobname].at_val.at_str,
+          pj->ji_qhdr->qu_qs.qu_name);
 
   /* acknowledge the request with the job id */
 
@@ -1936,10 +1903,8 @@ void req_commit(
     {
     if (LOGLEVEL >= 7)
       {
-      snprintf(log_buffer, sizeof(log_buffer),
-        "Trying to AUTORUN job %s",
-        pj->ji_qs.ji_jobid);
-
+      sprintf(log_buffer, "Trying to AUTORUN job %s",
+              pj->ji_qs.ji_jobid);
       log_record(
         PBSEVENT_JOB,
         PBS_EVENTCLASS_JOB,
