@@ -147,7 +147,7 @@ int status_job(
   if (svr_authorize_jobreq(preq, pjob) == 0)
     IsOwner = 1;
 
-  if (!server.sv_attr[(int)SRV_ATR_query_others].at_val.at_long)
+  if (!server.sv_attr[SRV_ATR_query_others].at_val.at_long)
     {
     if (IsOwner == 0)
       {
@@ -164,8 +164,6 @@ int status_job(
     return(PBSE_SYSTEM);
     }
 
-  memset(pstat, 0, sizeof(struct brp_status));
-
   CLEAR_LINK(pstat->brp_stlink);
 
   pstat->brp_objtype = MGR_OBJ_JOB;
@@ -199,77 +197,6 @@ int status_job(
 
 
 
-int add_walltime_remaining(
-   
-  int         index,
-  attribute  *pattr,
-  tlist_head *phead)
-
-  {
-  int       len;
-  char      buf[MAXPATHLEN];
-  char     *pname;
-  svrattrl *pal;
-  resource *pres;
-  
-  int found = 0;
-  unsigned long remaining = 0;
-  unsigned long upperBound = 0;
-
-  /* encode walltime remaining, this is custom because walltime 
-   * remaining isn't an attribute */
-  if ((pattr + JOB_ATR_state)->at_val.at_char != 'R')
-    {
-    /* only for running jobs, do nothing */
-    return(PBSE_NONE);
-    }
-  
-  if (((pattr + JOB_ATR_resource)->at_val.at_list.ll_next != NULL) &&
-      ((pattr + JOB_ATR_resource)->at_flags & ATR_VFLAG_SET))
-    {
-    pres = (resource *)GET_NEXT((pattr + JOB_ATR_resource)->at_val.at_list);
-    
-    if ((pattr + JOB_ATR_comp_time)->at_flags & ATR_VFLAG_SET)
-      upperBound = (pattr + JOB_ATR_comp_time)->at_val.at_long;
-    else
-      upperBound = (unsigned long)time_now;
-    
-    /* find the walltime resource */
-    for (;pres != NULL;pres = (resource *)GET_NEXT(pres->rs_link))
-      {
-      pname = pres->rs_defin->rs_name;
-      
-      if (strcmp(pname, "walltime") == 0)
-        {
-        /* found walltime */
-        unsigned long value = (unsigned long)pres->rs_value.at_val.at_long;
-        remaining = value - (time_now - (pattr + index)->at_val.at_long);
-        found = TRUE;
-        break;
-        }
-      }
-    }
-  
-  if (found == TRUE)
-    {
-    snprintf(buf,sizeof(buf),"%lu",remaining);
-    
-    len = strlen(buf) + 1;
-    pal = attrlist_create("Walltime","Remaining",len);
-    
-    if (pal != NULL)
-      {
-      memcpy(pal->al_value,buf,len);
-      pal->al_flags = ATR_VFLAG_SET;
-      append_link(phead,&pal->al_link,pal);
-      }
-    }
-
-  return(PBSE_NONE);
-  } /* END add_walltime_remaining() */
-
-
-
 
 
 /**
@@ -285,13 +212,13 @@ int add_walltime_remaining(
 
 int status_attrib(
 
-  svrattrl      *pal,      /* I */
+  svrattrl *pal,      /* I */
   attribute_def *padef,
-  attribute     *pattr,
-  int            limit,
-  int            priv,
-  tlist_head    *phead,
-  int           *bad,
+  attribute *pattr,
+  int   limit,
+  int   priv,
+  tlist_head *phead,
+  int  *bad,
   int            IsOwner)  /* 0 == FALSE, 1 == TRUE */
 
   {
@@ -363,7 +290,64 @@ int status_attrib(
         /* add walltime remaining if started */
         if ((index == JOB_ATR_start_time) &&
             ((pattr + index)->at_flags & ATR_VFLAG_SET))
-          add_walltime_remaining(index, pattr, phead);
+          {
+          /* encode walltime remaining, this is custom because walltime 
+           * remaining isn't an attribute */
+          int       len;
+          char      buf[MAXPATHLEN];
+          char     *pname;
+          svrattrl *pal;
+          resource *pres;
+
+          int found = 0;
+          unsigned long remaining = 0;
+          unsigned long upperBound = 0;
+
+          if (((pattr + JOB_ATR_resource)->at_val.at_list.ll_next != NULL) &&
+              ((pattr + JOB_ATR_resource)->at_flags & ATR_VFLAG_SET))
+            {
+            pres = (resource *)GET_NEXT((pattr + JOB_ATR_resource)->at_val.at_list);
+
+            if ((pattr + JOB_ATR_comp_time)->at_flags & ATR_VFLAG_SET)
+              {
+              upperBound = (pattr + JOB_ATR_comp_time)->at_val.at_long;
+              }
+            else
+              {
+              upperBound = (unsigned long)time_now;
+              }
+
+            /* find the walltime resource */
+            for (;pres != NULL;pres = (resource *)GET_NEXT(pres->rs_link))
+              {
+              pname = pres->rs_defin->rs_name;
+
+              if (strcmp(pname, "walltime") == 0)
+                {
+                /* found walltime */
+                unsigned long value = (unsigned long)pres->rs_value.at_val.at_long;
+                remaining = value - (time_now - (pattr + index)->at_val.at_long);
+                found = TRUE;
+                break;
+                }
+              }
+            }
+
+          if (found == TRUE)
+            {
+            snprintf(buf,sizeof(buf),"%lu",remaining);
+
+            len = strlen(buf) + 1;
+            pal = attrlist_create("Walltime","Remaining",len);
+
+            if (pal != NULL)
+              {
+              memcpy(pal->al_value,buf,len);
+              pal->al_flags = ATR_VFLAG_SET;
+              append_link(phead,&pal->al_link,pal);
+              }
+            }
+          } /* END if (index == JOB_ATR_start_time) */
         }
       }
     }    /* END for (index) */
