@@ -150,7 +150,6 @@ extern tlist_head svr_queues;
 extern int    comp_resc_lt;
 extern int    comp_resc_gt;
 extern int    svr_do_schedule;
-extern int    listener_command;
 extern int    LOGLEVEL;
 
 extern time_t time_now;
@@ -165,7 +164,6 @@ extern int  svr_clnodes;
 extern int node_avail_complex(char *, int *, int *, int *, int *);
 extern int procs_available(int proc_ct);
 extern int procs_requested(char *spec);
-extern int remove_procct(job *);
 
 
 /* Private Functions */
@@ -371,7 +369,7 @@ int svr_enquejob(
       {
       /* link after 'current' job in server's list */
 
-      /* only link if it isn't already inserted. routing queues, 
+       /* only link if it isn't already inserted. routing queues, 
        * remapping, etc. can mean it was already inserted */
       if (!is_link_initialized(&pjob->ji_jobs_array_sum))
         {
@@ -532,7 +530,6 @@ int svr_enquejob(
       
     /* notify the scheduler we have a new job */
     svr_do_schedule = SCH_SCHEDULE_NEW;
-    listener_command = SCH_SCHEDULE_NEW;
     }
   else if (pque->qu_qs.qu_type == QTYPE_RoutePush)
     {
@@ -645,7 +642,6 @@ void svr_dequejob(
   /* notify scheduler a job has been removed */
 
   svr_do_schedule = SCH_SCHEDULE_TERM;
-  listener_command = SCH_SCHEDULE_TERM;
 
   return;
   }  /* END svr_dequejob() */
@@ -721,7 +717,6 @@ int svr_setjobstate(
             (newstate == JOB_STATE_QUEUED))
           {
           svr_do_schedule = SCH_SCHEDULE_NEW;
-          listener_command = SCH_SCHEDULE_NEW;
 
           if ((pjob->ji_wattr[JOB_ATR_etime].at_flags & ATR_VFLAG_SET) == 0)
             {
@@ -755,12 +750,12 @@ int svr_setjobstate(
 
   if (pjob->ji_modified)
     {
-    return(job_save(pjob, SAVEJOB_FULL));
+    return(job_save(pjob, SAVEJOB_FULL,0));
     }
 
   if (changed)
     {
-    return(job_save(pjob, SAVEJOB_QUICK));
+    return(job_save(pjob, SAVEJOB_QUICK,0));
     }
 
   return(0);
@@ -828,6 +823,10 @@ void svr_evaljobstate(
     *newsub   = JOB_SUBSTATE_QUEUED;
     }
 
+  if (pjob->ji_is_array_template)
+    {
+    *newsub = JOB_SUBSTATE_ARRAY_TEMP;
+    }
   return;
   }  /* END svr_evaljobstate() */
 
@@ -1063,7 +1062,7 @@ static void chk_svr_resc_limit(
         }
 
       /* Added 6/14/2010 Ken Nielson for ability to parse the procs resource */
-      else if((jbrc->rs_defin == procresc) && (qtype == QTYPE_Execution))
+      if((jbrc->rs_defin == procresc) && (qtype == QTYPE_Execution))
         {
         proc_ct = jbrc->rs_value.at_val.at_long;
         }
@@ -1298,7 +1297,7 @@ static void chk_svr_resc_limit(
             (!pque->qu_attr[QE_ATR_is_transit].at_val.at_long))
           {
           if ((EMsg != NULL) && (EMsg[0] == '\0'))
-            strcpy(EMsg, "cannot locate feasible nodes (nodes file is empty or all systems are busy)");
+            strcpy(EMsg, "cannot locate feasible nodes");
         
           comp_resc_lt++;
           }
@@ -1318,26 +1317,24 @@ static void chk_svr_resc_limit(
               (!pque->qu_attr[QE_ATR_is_transit].at_val.at_long))
             {
             if ((EMsg != NULL) && (EMsg[0] == '\0'))
-              strcpy(EMsg, "cannot locate feasible nodes (nodes file is empty or requested nodes exceed system)");
+              strcpy(EMsg, "cannot locate feasible nodes");
       
             comp_resc_lt++;
             }
         }    
     }
 
-#ifndef CRAY_MOAB_PASSTHRU
   if((proc_ct + req_procs) > svr_clnodes) 
     {
     if ((!(pque->qu_attr[QE_ATR_is_transit].at_flags & ATR_VFLAG_SET)) ||
         (!pque->qu_attr[QE_ATR_is_transit].at_val.at_long))
       {
       if ((EMsg != NULL) && (EMsg[0] == '\0'))
-        strcpy(EMsg, "cannot locate feasible nodes (nodes file is empty or requested nodes exceed all systems)");
+        strcpy(EMsg, "cannot locate feasible nodes");
     
       comp_resc_lt++;
       }
     }
-#endif
 
   if (MPPWidth > 0)
     {
@@ -1535,7 +1532,7 @@ int svr_chkque(
 
   /*
    * 1. If the queue is an Execution queue ...
-   *    This is checked first because 1a - 1c are more damaging
+   *    These is checked first because 1a - 1c are more damaging
    *    (see local_move() in svr_movejob.c)
    */
 
@@ -1552,7 +1549,7 @@ int svr_chkque(
         }
       }
 
-    /* 1b. check site restrictions -- Currently site_acl_check is a stub */
+    /* 1b. check site restrictions */
 
     if (site_acl_check(pjob, pque))
       {
@@ -1742,7 +1739,7 @@ int svr_chkque(
 
         pas = pque->qu_attr[QA_ATR_AclGroup].at_val.at_arst;
 
-        for (i = 0; pas != NULL && i < pas->as_usedptr;i++)
+        for (i = 0;i < pas->as_usedptr;i++)
           {
           if ((grp = getgrnam(pas->as_string[i])) == NULL)
             continue;
@@ -2149,21 +2146,6 @@ static void default_std(
 
 #endif
 
-  if (*pd == '[')
-    {
-    pd++;
-
-    if (isdigit(*pd))
-      {
-      *(to + len++) = '-';
-
-      while (isdigit(*pd))
-        {
-        *(to + len++) = *pd++;
-        }
-      }
-    }
-
   *(to + len) = '\0';
 
   return;
@@ -2231,7 +2213,6 @@ char *prefix_std_file(
 
     if (name)
       {
-      memset(name, 0, len);
       strcpy(name, qsubhost); /* the qsub host name */
       strcat(name, ":");        /* the :  */
 
@@ -2398,12 +2379,11 @@ void set_resc_deflt(
   attribute *ji_wattr) /* I (optional) decoded attributes  */
 
   {
-  attribute     *ja;
+  attribute *ja;
 
-  pbs_queue     *pque;
+  pbs_queue *pque;
   attribute_def *pdef;
-  int            i;
-  int            rc;
+  int i, rc;
 
   pque = pjob->ji_qhdr;
 
@@ -2427,7 +2407,7 @@ void set_resc_deflt(
 
 #ifdef RESOURCEMAXDEFAULT
   set_deflt_resc(ja, &pque->qu_attr[QA_ATR_ResourceMax]);
-/* http://www.adaptivecomputing.com/resources/docs/torque/4.2high-availability.php */
+
   /* server max limits will only be applied to attributes which have
      not yet been set */
 
@@ -2449,12 +2429,6 @@ void set_resc_deflt(
         }
       }
     }
-  
-  if (pque->qu_qs.qu_type == QTYPE_Execution)
-    {
-    /* unset the procct resource if it has been set */
-    remove_procct(pjob);
-    }
 
   return;
   }  /* END set_resc_deflt() */
