@@ -109,7 +109,6 @@
 #include "pbs_error.h"
 #include "resource.h"
 #include "utils.h"
-#include "dynamic_string.h"
 
 #ifndef MAXLINE
 #define MAXLINE 1024
@@ -186,17 +185,6 @@ int svr_recov(
     return(-1);
     }
 
-  if (strstr((char *)&server.sv_qs,"<server_db>") != NULL)
-    {
-    /* attempt to read the server database in xml */
-    log_event(PBSEVENT_SYSTEM,
-      PBS_EVENTCLASS_SERVER,
-      id,
-      "serverdb appears to be in xml format, attempting to read xml\n");
-
-    return(svr_recov_xml(svrfile,read_only));
-    }
-
   /* Save the sv_jobidnumber field in case it is set by the attributes. */
   i = server.sv_qs.sv_jobidnumber;
 
@@ -207,7 +195,7 @@ int svr_recov(
         &server,
         svr_attr_def,
         server.sv_attr,
-        (int)SRV_ATR_LAST,
+        SRV_ATR_LAST,
         0,
         !read_only) != 0 ) 
     {
@@ -224,9 +212,9 @@ int svr_recov(
     {
     server.sv_qs.sv_jobidnumber = i;
 
-    server.sv_attr[(int)SRV_ATR_NextJobNumber].at_val.at_long = i;
+    server.sv_attr[SRV_ATR_NextJobNumber].at_val.at_long = i;
 
-    server.sv_attr[(int)SRV_ATR_NextJobNumber].at_flags |= ATR_VFLAG_SET| ATR_VFLAG_MODIFY;
+    server.sv_attr[SRV_ATR_NextJobNumber].at_flags |= ATR_VFLAG_SET| ATR_VFLAG_MODIFY;
     }
 
   close(sdb);
@@ -341,23 +329,20 @@ int size_to_str(
 /*
  * attr_to_str
  *
- * @param ds - the dynamic string we're printing the attribute into
+ * @param out - the string output
+ * @param size - the largest possible string length
  * @param aindex - the attribute's index
  * @param attr - the attribute
- * @param XML - boolean telling us whether to print XML or not
  */
 int attr_to_str(
 
-  dynamic_string   *ds,     /* O */
+  char             *out,    /* O */
+  int               size,   /* I */
   attribute_def    *at_def, /* I */
   struct attribute  attr,   /* I */
   int               XML)    /* I */
 
   {
-  int rc = 0;
-  /* used to print numbers and chars as strings */
-  char local_buf[MAXLINE];
-
   if ((attr.at_flags & ATR_VFLAG_SET) == FALSE)
     return(NO_ATTR_DATA);
 
@@ -365,15 +350,21 @@ int attr_to_str(
     {
     case ATR_TYPE_LONG:
 
-      snprintf(local_buf, sizeof(local_buf), "%ld", attr.at_val.at_long);
-      rc = append_dynamic_string(ds, local_buf);
+      snprintf(out,size,"%ld",attr.at_val.at_long);
 
       break;
 
     case ATR_TYPE_CHAR:
 
-      sprintf(local_buf, "%c", attr.at_val.at_char);
-      rc = append_dynamic_string(ds, local_buf);
+      if (size > 1)
+        {
+        out[0] = attr.at_val.at_char;
+        out[1] = '\0';
+        }
+      else
+        {
+        return(NO_BUFFER_SPACE);
+        }
 
       break;
 
@@ -386,9 +377,9 @@ int attr_to_str(
         return(NO_ATTR_DATA);
 
       if (XML)
-        rc = append_dynamic_string_xml(ds, attr.at_val.at_str);
+        escape_xml(attr.at_val.at_str,out,size);
       else
-        rc = append_dynamic_string(ds, attr.at_val.at_str);
+        snprintf(out,size,"%s",attr.at_val.at_str);
 
       break;
 
@@ -402,16 +393,36 @@ int attr_to_str(
       if (arst == NULL)
         return(NO_ATTR_DATA);
 
+      if (size < arst->as_bufsize)
+        return(NO_BUFFER_SPACE);
+
+      out[0] = '\0';
+
       /* concatenate all of the array strings into one string */
       for (j = 0; j < arst->as_usedptr; j++)
         {
         if (j > 0)
-          append_dynamic_string(ds, ",");
-
-        if (XML)
-          rc = append_dynamic_string_xml(ds, arst->as_string[j]);
+          {
+          int len;
+          strcat(out,",");
+          if (XML)
+            {
+            len = strlen(out);
+            escape_xml(arst->as_string[j],out+len,size-len);
+            }
+          else
+            strcat(out,arst->as_string[j]);
+          }
         else
-          rc = append_dynamic_string(ds, arst->as_string[j]);
+          {
+          if (XML) 
+            {
+            int len = strlen(out);
+            escape_xml(arst->as_string[j],out+len,size-len);
+            }
+          else
+            strcat(out,arst->as_string[j]);
+          }
         }
       }
 
@@ -419,36 +430,43 @@ int attr_to_str(
 
     case ATR_TYPE_SIZE:
 
-      rc = size_to_dynamic_string(ds, attr.at_val.at_size);
+      size_to_str(attr.at_val.at_size,out,size);
 
       break;
 
     case ATR_TYPE_RESC:
 
       {
+      char *ptr;
+
+      int   lspace;
+      int   len;
+
       resource *current = (resource *)GET_NEXT(attr.at_val.at_list);
 
       if (current == NULL)
         return(NO_ATTR_DATA);
 
+      ptr = out;
+      lspace = size;
+
       /* print all of the resources */
       while (current != NULL)
         {
+
         /* there are only 3 resource types used */
         switch (current->rs_value.at_type)
           {
           case ATR_TYPE_LONG:
 
-            append_dynamic_string(ds, "\t\t<");
-            append_dynamic_string(ds, current->rs_defin->rs_name);
-            append_dynamic_string(ds, ">");
-
-            snprintf(local_buf, sizeof(local_buf), "%ld", current->rs_value.at_val.at_long);
-            append_dynamic_string(ds, local_buf);
-
-            append_dynamic_string(ds, "</");
-            append_dynamic_string(ds, current->rs_defin->rs_name);
-            rc = append_dynamic_string(ds, ">\n");
+            snprintf(ptr,lspace,"\t\t<%s>%ld</%s>\n",
+              current->rs_defin->rs_name,
+              current->rs_value.at_val.at_long,
+              current->rs_defin->rs_name);
+            
+            len = strlen(ptr);
+            ptr    += len;
+            lspace -= len;
 
             break;
 
@@ -459,44 +477,57 @@ int attr_to_str(
              * Bugzilla bug 101 
              */
 
-            if (current->rs_value.at_val.at_str == NULL)
+            if(current->rs_value.at_val.at_str == NULL)
               break;
 
-            if (strlen(current->rs_value.at_val.at_str) == 0)
+            if(strlen(current->rs_value.at_val.at_str) == 0)
               break;
 
-            append_dynamic_string(ds, "\t\t<");
-            append_dynamic_string(ds, current->rs_defin->rs_name);
-            append_dynamic_string(ds, ">");
+            snprintf(ptr,lspace,"\t\t<%s>",
+              current->rs_defin->rs_name);
+            len = strlen(ptr);
+            ptr    += len;
+            lspace -= len;
 
             if (XML)
-              append_dynamic_string_xml(ds, current->rs_value.at_val.at_str);
+              escape_xml(current->rs_value.at_val.at_str,ptr,lspace);
             else
-              append_dynamic_string(ds, current->rs_value.at_val.at_str);
+              snprintf(ptr,lspace,"%s",current->rs_value.at_val.at_str);
 
-            append_dynamic_string(ds, "</");
-            append_dynamic_string(ds, current->rs_defin->rs_name);
-            rc = append_dynamic_string(ds, ">\n");
+            len = strlen(ptr);
+            ptr += len;
+            lspace -= len;
+
+            snprintf(ptr,lspace,"</%s>\n",
+              current->rs_defin->rs_name);
+            len = strlen(ptr);
+            ptr += len;
+            lspace -= len;
 
             break;
 
           case ATR_TYPE_SIZE:
 
-            append_dynamic_string(ds, "\t\t<");
-            append_dynamic_string(ds, current->rs_defin->rs_name);
-            append_dynamic_string(ds, ">");
- 
-            size_to_dynamic_string(ds, current->rs_value.at_val.at_size);
- 
-            append_dynamic_string(ds, "</");
-            append_dynamic_string(ds, current->rs_defin->rs_name);
-            rc = append_dynamic_string(ds, ">\n");
+            snprintf(ptr,lspace,"\t\t<%s>",current->rs_defin->rs_name);
+            len = strlen(ptr);
+            ptr += len;
+            lspace -= len;
+
+            size_to_str(current->rs_value.at_val.at_size,ptr,size);
+            len = strlen(ptr);
+            ptr += len;
+            lspace -= len;
+
+            snprintf(ptr,lspace,"</%s>\n",current->rs_defin->rs_name);
+            len = strlen(ptr);
+            ptr += len;
+            lspace -= len;
 
             break;
           }
 
-        if (rc != PBSE_NONE)
-          return(rc);
+        if (lspace == 0)
+          return(NO_BUFFER_SPACE);
 
         current = (resource *)GET_NEXT(current->rs_link);
         }
@@ -514,7 +545,7 @@ int attr_to_str(
       break;
     } /* END switch attribute type */
 
-  return(PBSE_NONE);
+  return(0);
   } /* END attr_to_str */
 
 
@@ -895,7 +926,7 @@ int svr_save_xml(
   {
   char *id   = "svr_save_xml";
   char  buf[MAXLINE<<8];
-  dynamic_string *ds;
+  char  valbuf[MAXLINE<<7];
 
   int   fds;
   int   rc;
@@ -928,21 +959,18 @@ int svr_save_xml(
   if ((rc = write_buffer(buf,strlen(buf),fds)))
     return(rc);
 
-  if ((ds = get_dynamic_string(-1, NULL)) == NULL)
-    {
-    log_err(ENOMEM, id, "");
-    return(ENOMEM);
-    }
-
   i = 0;
   while (i != SRV_ATR_LAST)
     {
     if (ps->sv_attr[i].at_flags & ATR_VFLAG_SET)
       {
       buf[0] = '\0';
-      clear_dynamic_string(ds);
-
-      if ((rc = attr_to_str(ds, svr_attr_def + i, ps->sv_attr[i], TRUE) != 0))
+      valbuf[0] = '\0';
+      if ((rc = attr_to_str(valbuf,
+              sizeof(valbuf),
+              svr_attr_def + i, 
+              ps->sv_attr[i],
+              TRUE) != 0))
         {
         if (rc != NO_ATTR_DATA)
           {
@@ -952,7 +980,6 @@ int svr_save_xml(
             svr_attr_def[i].at_name);
           log_err(-1,id,log_buffer);
 
-          free_dynamic_string(ds);
           return(rc);
           }
         }
@@ -960,24 +987,19 @@ int svr_save_xml(
         {
         snprintf(buf,sizeof(buf),"<%s>%s</%s>\n",
           svr_attr_def[i].at_name,
-          ds->str,
+          valbuf,
           svr_attr_def[i].at_name);
         
         if (buf[0] != '\0')
           {
           if ((rc = write_buffer(buf,strlen(buf),fds)))
-            {
-            free_dynamic_string(ds);
             return(rc);
-            }
           }
         }      
       }
 
     i++;
     }
-
-  free_dynamic_string(ds);
   
   snprintf(buf,sizeof(buf),"</attributes>\n");
   if ((rc = write_buffer(buf,strlen(buf),fds)))
@@ -1012,20 +1034,15 @@ int svr_save(
   int            mode)
 
   {
-#ifndef SERVER_XML
-  static char *this_function_name = "svr_save";
+/*  static char *this_function_name = "svr_save";
   int i;
   int sdb;
   int save_acl(attribute *, attribute_def *, char *, char *);
-#endif /* ndef SERVER_XML */
+*/
 
-  /* save the server in xml only if configured */
-#ifdef SERVER_XML
-      return(svr_save_xml(ps,mode));
-#endif /* def SERVER_XML */
+  return(svr_save_xml(ps,mode));
 
-#ifndef SERVER_XML
-  if (mode == SVR_SAVE_QUICK)
+/*  if (mode == SVR_SAVE_QUICK)
     {
     sdb = open(path_svrdb, O_WRONLY | O_CREAT | O_Sync, 0600);
 
@@ -1052,10 +1069,10 @@ int svr_save(
     close(sdb);
     }
   else
-    {
+    {*/
     /* SVR_SAVE_FULL Save */
 
-    sdb = open(path_svrdb_new, O_WRONLY | O_CREAT | O_Sync, 0600);
+/*    sdb = open(path_svrdb_new, O_WRONLY | O_CREAT | O_Sync, 0600);
 
     if (sdb < 0)
       {
@@ -1081,7 +1098,7 @@ int svr_save(
       return(-1);
       }
 
-    if (save_attr(svr_attr_def, ps->sv_attr, (int)SRV_ATR_LAST) != 0)
+    if (save_attr(svr_attr_def, ps->sv_attr, SRV_ATR_LAST) != 0)
       {
       close(sdb);
 
@@ -1094,10 +1111,10 @@ int svr_save(
 
       return(-1);
       }
-
+*/
     /* new db successfully created, remove original db */
 
-    close(sdb);
+/*    close(sdb);
 
     unlink(path_svrdb);
 
@@ -1113,20 +1130,19 @@ int svr_save(
       {
       unlink(path_svrdb_new);
       }
-
+*/
     /* save the server acls to their own files: */
     /*  priv/svracl/(attr name)   */
 
-    for (i = 0;i < SRV_ATR_LAST;i++)
+/*    for (i = 0;i < SRV_ATR_LAST;i++)
       {
       if (ps->sv_attr[i].at_type == ATR_TYPE_ACL)
         save_acl(&ps->sv_attr[i], &svr_attr_def[i],
                  PBS_SVRACL, svr_attr_def[i].at_name);
       }
-    }    /* END else (mode == SVR_SAVE_QUICK) */
+    }*/    /* END else (mode == SVR_SAVE_QUICK) */
 
-  return(0);
-#endif /* ndef SERVER_XML */
+/*  return(0);*/
   }  /* END svr_save() */
 
 
